@@ -22,11 +22,12 @@ use std::path::Path;
 use std::path::PathBuf;
 
 use bstr::BStr;
+use bstr::BString;
 use futures::executor::block_on_stream;
 use futures::stream::BoxStream;
 use futures::StreamExt as _;
 use futures::TryStreamExt as _;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::backend::BackendError;
 use jj_lib::backend::BackendResult;
 use jj_lib::backend::CommitId;
@@ -38,6 +39,7 @@ use jj_lib::config::ConfigGetResultExt as _;
 use jj_lib::conflicts::materialize_merge_result_to_bytes;
 use jj_lib::conflicts::materialized_diff_stream;
 use jj_lib::conflicts::ConflictMarkerStyle;
+use jj_lib::conflicts::MaterializedFileValue;
 use jj_lib::conflicts::MaterializedTreeDiffEntry;
 use jj_lib::conflicts::MaterializedTreeValue;
 use jj_lib::copies::CopiesTreeDiffEntry;
@@ -50,7 +52,6 @@ use jj_lib::diff::CompareBytesIgnoreAllWhitespace;
 use jj_lib::diff::CompareBytesIgnoreWhitespaceAmount;
 use jj_lib::diff::Diff;
 use jj_lib::diff::DiffHunk;
-use jj_lib::diff::DiffHunkContentVec;
 use jj_lib::diff::DiffHunkKind;
 use jj_lib::files::DiffLineHunkSide;
 use jj_lib::files::DiffLineIterator;
@@ -58,7 +59,7 @@ use jj_lib::files::DiffLineNumber;
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::MergedTree;
-use jj_lib::object_id::ObjectId;
+use jj_lib::object_id::ObjectId as _;
 use jj_lib::repo::Repo;
 use jj_lib::repo_path::InvalidRepoPathError;
 use jj_lib::repo_path::RepoPath;
@@ -66,7 +67,7 @@ use jj_lib::repo_path::RepoPathUiConverter;
 use jj_lib::rewrite::rebase_to_dest_parent;
 use jj_lib::settings::UserSettings;
 use jj_lib::store::Store;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use thiserror::Error;
 use tracing::instrument;
 use unicode_width::UnicodeWidthStr as _;
@@ -143,6 +144,18 @@ pub enum DiffFormat {
     Tool(Box<ExternalMergeTool>),
 }
 
+impl DiffFormat {
+    fn is_short(&self) -> bool {
+        match self {
+            DiffFormat::Summary
+            | DiffFormat::Stat(_)
+            | DiffFormat::Types
+            | DiffFormat::NameOnly => true,
+            DiffFormat::Git(_) | DiffFormat::ColorWords(_) | DiffFormat::Tool(_) => false,
+        }
+    }
+}
+
 /// Returns a list of requested diff formats, which will never be empty.
 pub fn diff_formats_for(
     settings: &UserSettings,
@@ -164,8 +177,10 @@ pub fn diff_formats_for_log(
     patch: bool,
 ) -> Result<Vec<DiffFormat>, ConfigGetError> {
     let mut formats = diff_formats_from_args(settings, args)?;
-    // --patch implies default if no format other than --summary is specified
-    if patch && matches!(formats.as_slice(), [] | [DiffFormat::Summary]) {
+    // --patch implies default if no "long" format is specified
+    if patch && formats.iter().all(DiffFormat::is_short) {
+        // TODO: maybe better to error out if the configured default isn't a
+        // "long" format?
         formats.push(default_diff_format(settings, args)?);
         formats.dedup();
     }
@@ -177,15 +192,22 @@ fn diff_formats_from_args(
     args: &DiffFormatArgs,
 ) -> Result<Vec<DiffFormat>, ConfigGetError> {
     let mut formats = Vec::new();
+    // "short" format first:
     if args.summary {
         formats.push(DiffFormat::Summary);
     }
+    if args.stat {
+        let mut options = DiffStatOptions::default();
+        options.merge_args(args);
+        formats.push(DiffFormat::Stat(Box::new(options)));
+    }
     if args.types {
         formats.push(DiffFormat::Types);
     }
     if args.name_only {
         formats.push(DiffFormat::NameOnly);
     }
+    // "long" format follows:
     if args.git {
         let mut options = UnifiedDiffOptions::from_settings(settings)?;
         options.merge_args(args);
@@ -196,11 +218,6 @@ fn diff_formats_from_args(
         options.merge_args(args);
         formats.push(DiffFormat::ColorWords(Box::new(options)));
     }
-    if args.stat {
-        let mut options = DiffStatOptions::default();
-        options.merge_args(args);
-        formats.push(DiffFormat::Stat(Box::new(options)));
-    }
     if let Some(name) = &args.tool {
         let tool = merge_tools::get_external_tool_config(settings, name)?
             .unwrap_or_else(|| ExternalMergeTool::with_program(name));
@@ -223,15 +240,13 @@ fn default_diff_format(
         .unwrap_or_else(|| ExternalMergeTool::with_diff_args(&args));
         return Ok(DiffFormat::Tool(Box::new(tool)));
     }
-    let name = if let Some(name) = settings.get_string("ui.diff.format").optional()? {
-        name
-    } else if let Some(name) = settings.get_string("diff.format").optional()? {
-        name // old config name
-    } else {
-        "color-words".to_owned()
-    };
-    match name.as_ref() {
+    match settings.get_string("ui.diff.format")?.as_ref() {
         "summary" => Ok(DiffFormat::Summary),
+        "stat" => {
+            let mut options = DiffStatOptions::default();
+            options.merge_args(args);
+            Ok(DiffFormat::Stat(Box::new(options)))
+        }
         "types" => Ok(DiffFormat::Types),
         "name-only" => Ok(DiffFormat::NameOnly),
         "git" => {
@@ -244,12 +259,7 @@ fn default_diff_format(
             options.merge_args(args);
             Ok(DiffFormat::ColorWords(Box::new(options)))
         }
-        "stat" => {
-            let mut options = DiffStatOptions::default();
-            options.merge_args(args);
-            Ok(DiffFormat::Stat(Box::new(options)))
-        }
-        _ => Err(ConfigGetError::Type {
+        name => Err(ConfigGetError::Type {
             name: "ui.diff.format".to_owned(),
             error: format!("Invalid diff format: {name}").into(),
             source_path: None,
@@ -298,7 +308,7 @@ impl<'a> DiffRenderer<'a> {
     }
 
     /// Generates diff between `from_tree` and `to_tree`.
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub fn show_diff(
         &self,
         ui: &Ui, // TODO: remove Ui dependency if possible
@@ -322,7 +332,7 @@ impl<'a> DiffRenderer<'a> {
         })
     }
 
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     fn show_diff_inner(
         &self,
         ui: &Ui,
@@ -477,7 +487,7 @@ pub fn get_copy_records<'a>(
     root: &CommitId,
     head: &CommitId,
     matcher: &'a dyn Matcher,
-) -> BackendResult<impl Iterator<Item = BackendResult<CopyRecord>> + 'a> {
+) -> BackendResult<impl Iterator<Item = BackendResult<CopyRecord>> + use<'a>> {
     // TODO: teach backend about matching path prefixes?
     let stream = store.get_copy_records(None, root, head)?;
     // TODO: test record.source as well? should be AND-ed or OR-ed?
@@ -575,11 +585,10 @@ impl ColorWordsDiffOptions {
 
 fn show_color_words_diff_hunks(
     formatter: &mut dyn Formatter,
-    left: &[u8],
-    right: &[u8],
+    contents: [&BStr; 2],
     options: &ColorWordsDiffOptions,
 ) -> io::Result<()> {
-    let line_diff = diff_by_line([left, right], &options.line_diff);
+    let line_diff = diff_by_line(contents, &options.line_diff);
     let mut line_number = DiffLineNumber { left: 1, right: 1 };
     // Matching entries shouldn't appear consecutively in diff of two inputs.
     // However, if the inputs have conflicts, there may be a hunk that can be
@@ -588,8 +597,9 @@ fn show_color_words_diff_hunks(
     let mut emitted = false;
 
     for hunk in line_diff.hunks() {
+        let hunk_contents: [&BStr; 2] = hunk.contents[..].try_into().unwrap();
         match hunk.kind {
-            DiffHunkKind::Matching => contexts.push(hunk.contents),
+            DiffHunkKind::Matching => contexts.push(hunk_contents),
             DiffHunkKind::Different => {
                 let num_after = if emitted { options.context } else { 0 };
                 line_number = show_color_words_context_lines(
@@ -603,7 +613,7 @@ fn show_color_words_diff_hunks(
                 contexts.clear();
                 emitted = true;
                 line_number =
-                    show_color_words_diff_lines(formatter, &hunk.contents, line_number, options)?;
+                    show_color_words_diff_lines(formatter, hunk_contents, line_number, options)?;
             }
         }
     }
@@ -624,7 +634,7 @@ fn show_color_words_diff_hunks(
 /// Prints `num_after` lines, ellipsis, and `num_before` lines.
 fn show_color_words_context_lines(
     formatter: &mut dyn Formatter,
-    contexts: &[DiffHunkContentVec],
+    contexts: &[[&BStr; 2]],
     mut line_number: DiffLineNumber,
     options: &ColorWordsDiffOptions,
     num_after: usize,
@@ -642,15 +652,13 @@ fn show_color_words_context_lines(
         (after_lines, before_lines, num_skipped)
     };
     let show = |formatter: &mut dyn Formatter,
-                left_lines: &[&[u8]],
-                right_lines: &[&[u8]],
+                [left_lines, right_lines]: [&[&[u8]]; 2],
                 mut line_number: DiffLineNumber| {
         if left_lines == right_lines {
             for line in left_lines {
                 show_color_words_line_number(
                     formatter,
-                    Some(line_number.left),
-                    Some(line_number.right),
+                    [Some(line_number.left), Some(line_number.right)],
                 )?;
                 show_color_words_inline_hunks(
                     formatter,
@@ -665,7 +673,7 @@ fn show_color_words_context_lines(
             let right = right_lines.concat();
             show_color_words_diff_lines(
                 formatter,
-                &[BStr::new(&left), BStr::new(&right)],
+                [&left, &right].map(BStr::new),
                 line_number,
                 options,
             )
@@ -674,7 +682,7 @@ fn show_color_words_context_lines(
 
     let (left_after, mut left_before, num_left_skipped) = extract(0);
     let (right_after, mut right_before, num_right_skipped) = extract(1);
-    line_number = show(formatter, &left_after, &right_after, line_number)?;
+    line_number = show(formatter, [&left_after, &right_after], line_number)?;
     if num_left_skipped > 0 || num_right_skipped > 0 {
         write!(formatter, "{SKIPPED_CONTEXT_LINE}")?;
         line_number.left += num_left_skipped;
@@ -690,13 +698,13 @@ fn show_color_words_context_lines(
     }
     left_before.reverse();
     right_before.reverse();
-    line_number = show(formatter, &left_before, &right_before, line_number)?;
+    line_number = show(formatter, [&left_before, &right_before], line_number)?;
     Ok(line_number)
 }
 
 fn show_color_words_diff_lines(
     formatter: &mut dyn Formatter,
-    contents: &[&BStr],
+    contents: [&BStr; 2],
     mut line_number: DiffLineNumber,
     options: &ColorWordsDiffOptions,
 ) -> io::Result<DiffLineNumber> {
@@ -715,25 +723,27 @@ fn show_color_words_diff_lines(
         for diff_line in diff_line_iter.by_ref() {
             show_color_words_line_number(
                 formatter,
-                diff_line
-                    .has_left_content()
-                    .then_some(diff_line.line_number.left),
-                diff_line
-                    .has_right_content()
-                    .then_some(diff_line.line_number.right),
+                [
+                    diff_line
+                        .has_left_content()
+                        .then_some(diff_line.line_number.left),
+                    diff_line
+                        .has_right_content()
+                        .then_some(diff_line.line_number.right),
+                ],
             )?;
             show_color_words_inline_hunks(formatter, &diff_line.hunks)?;
         }
         line_number = diff_line_iter.next_line_number();
     } else {
-        let (left_lines, right_lines) = unzip_diff_hunks_to_lines(&word_diff_hunks);
+        let [left_lines, right_lines] = unzip_diff_hunks_to_lines(&word_diff_hunks);
         for tokens in &left_lines {
-            show_color_words_line_number(formatter, Some(line_number.left), None)?;
+            show_color_words_line_number(formatter, [Some(line_number.left), None])?;
             show_color_words_single_sided_line(formatter, tokens, "removed")?;
             line_number.left += 1;
         }
         for tokens in &right_lines {
-            show_color_words_line_number(formatter, None, Some(line_number.right))?;
+            show_color_words_line_number(formatter, [None, Some(line_number.right)])?;
             show_color_words_single_sided_line(formatter, tokens, "added")?;
             line_number.right += 1;
         }
@@ -743,8 +753,7 @@ fn show_color_words_diff_lines(
 
 fn show_color_words_line_number(
     formatter: &mut dyn Formatter,
-    left_line_number: Option<u32>,
-    right_line_number: Option<u32>,
+    [left_line_number, right_line_number]: [Option<u32>; 2],
 ) -> io::Result<()> {
     if let Some(line_number) = left_line_number {
         formatter.with_label("removed", |formatter| {
@@ -844,14 +853,14 @@ fn split_diff_hunks_by_matching_newline<'a, 'b>(
 struct FileContent {
     /// false if this file is likely text; true if it is likely binary.
     is_binary: bool,
-    contents: Vec<u8>,
+    contents: BString,
 }
 
 impl FileContent {
     fn empty() -> Self {
         Self {
             is_binary: false,
-            contents: vec![],
+            contents: BString::default(),
         }
     }
 
@@ -860,7 +869,10 @@ impl FileContent {
     }
 }
 
-fn file_content_for_diff(reader: &mut dyn io::Read) -> io::Result<FileContent> {
+fn file_content_for_diff(
+    path: &RepoPath,
+    file: &mut MaterializedFileValue,
+) -> BackendResult<FileContent> {
     // If this is a binary file, don't show the full contents.
     // Determine whether it's binary by whether the first 8k bytes contain a null
     // character; this is the same heuristic used by git as of writing: https://github.com/git/git/blob/eea0e59ffbed6e33d171ace5be13cde9faa41639/xdiff-interface.c#L192-L198
@@ -868,9 +880,7 @@ fn file_content_for_diff(reader: &mut dyn io::Read) -> io::Result<FileContent> {
     // TODO: currently we look at the whole file, even though for binary files we
     // only need to know the file size. To change that we'd have to extend all
     // the data backends to support getting the length.
-    let mut contents = vec![];
-    reader.read_to_end(&mut contents)?;
-
+    let contents = BString::new(file.read_all(path)?);
     let start = &contents[..PEEK_SIZE.min(contents.len())];
     Ok(FileContent {
         is_binary: start.contains(&b'\0'),
@@ -887,22 +897,17 @@ fn diff_content(
         MaterializedTreeValue::Absent => Ok(FileContent::empty()),
         MaterializedTreeValue::AccessDenied(err) => Ok(FileContent {
             is_binary: false,
-            contents: format!("Access denied: {err}").into_bytes(),
+            contents: format!("Access denied: {err}").into(),
         }),
-        MaterializedTreeValue::File { id, mut reader, .. } => file_content_for_diff(&mut reader)
-            .map_err(|err| BackendError::ReadFile {
-                path: path.to_owned(),
-                id,
-                source: err.into(),
-            }),
+        MaterializedTreeValue::File(mut file) => file_content_for_diff(path, &mut file),
         MaterializedTreeValue::Symlink { id: _, target } => Ok(FileContent {
             // Unix file paths can't contain null bytes.
             is_binary: false,
-            contents: target.into_bytes(),
+            contents: target.into(),
         }),
         MaterializedTreeValue::GitSubmodule(id) => Ok(FileContent {
             is_binary: false,
-            contents: format!("Git submodule checked out at {id}").into_bytes(),
+            contents: format!("Git submodule checked out at {id}").into(),
         }),
         // TODO: are we sure this is never binary?
         MaterializedTreeValue::FileConflict {
@@ -911,11 +916,11 @@ fn diff_content(
             executable: _,
         } => Ok(FileContent {
             is_binary: false,
-            contents: materialize_merge_result_to_bytes(&contents, conflict_marker_style).into(),
+            contents: materialize_merge_result_to_bytes(&contents, conflict_marker_style),
         }),
         MaterializedTreeValue::OtherConflict { id } => Ok(FileContent {
             is_binary: false,
-            contents: id.describe().into_bytes(),
+            contents: id.describe().into(),
         }),
         MaterializedTreeValue::Tree(id) => {
             panic!("Unexpected tree with id {id:?} in diff at path {path:?}");
@@ -929,8 +934,8 @@ fn basic_diff_file_type(value: &MaterializedTreeValue) -> &'static str {
             panic!("absent path in diff");
         }
         MaterializedTreeValue::AccessDenied(_) => "access denied",
-        MaterializedTreeValue::File { executable, .. } => {
-            if *executable {
+        MaterializedTreeValue::File(file) => {
+            if file.executable {
                 "executable file"
             } else {
                 "regular file"
@@ -992,25 +997,20 @@ pub fn show_color_words_diff(
                 } else if right_content.is_binary {
                     writeln!(formatter.labeled("binary"), "    (binary)")?;
                 } else {
-                    show_color_words_diff_hunks(formatter, &[], &right_content.contents, options)?;
+                    show_color_words_diff_hunks(
+                        formatter,
+                        [BStr::new(""), right_content.contents.as_ref()],
+                        options,
+                    )?;
                 }
             } else if right_value.is_present() {
                 let description = match (&left_value, &right_value) {
-                    (
-                        MaterializedTreeValue::File {
-                            executable: left_executable,
-                            ..
-                        },
-                        MaterializedTreeValue::File {
-                            executable: right_executable,
-                            ..
-                        },
-                    ) => {
-                        if *left_executable && *right_executable {
+                    (MaterializedTreeValue::File(left), MaterializedTreeValue::File(right)) => {
+                        if left.executable && right.executable {
                             "Modified executable file".to_string()
-                        } else if *left_executable {
+                        } else if left.executable {
                             "Executable file became non-executable at".to_string()
-                        } else if *right_executable {
+                        } else if right.executable {
                             "Non-executable file became executable at".to_string()
                         } else {
                             "Modified regular file".to_string()
@@ -1066,8 +1066,7 @@ pub fn show_color_words_diff(
                 } else {
                     show_color_words_diff_hunks(
                         formatter,
-                        &left_content.contents,
-                        &right_content.contents,
+                        [&left_content.contents, &right_content.contents].map(BStr::new),
                         options,
                     )?;
                 }
@@ -1083,7 +1082,11 @@ pub fn show_color_words_diff(
                 } else if left_content.is_binary {
                     writeln!(formatter.labeled("binary"), "    (binary)")?;
                 } else {
-                    show_color_words_diff_hunks(formatter, &left_content.contents, &[], options)?;
+                    show_color_words_diff_hunks(
+                        formatter,
+                        [left_content.contents.as_ref(), BStr::new("")],
+                        options,
+                    )?;
                 }
             }
         }
@@ -1193,14 +1196,10 @@ fn git_diff_part(
                 source: err,
             });
         }
-        MaterializedTreeValue::File {
-            id,
-            executable,
-            mut reader,
-        } => {
-            mode = if executable { "100755" } else { "100644" };
-            hash = id.hex();
-            content = file_content_for_diff(&mut reader)?;
+        MaterializedTreeValue::File(mut file) => {
+            mode = if file.executable { "100755" } else { "100644" };
+            hash = file.id.hex();
+            content = file_content_for_diff(path, &mut file)?;
         }
         MaterializedTreeValue::Symlink { id, target } => {
             mode = "120000";
@@ -1208,7 +1207,7 @@ fn git_diff_part(
             content = FileContent {
                 // Unix file paths can't contain null bytes.
                 is_binary: false,
-                contents: target.into_bytes(),
+                contents: target.into(),
             };
         }
         MaterializedTreeValue::GitSubmodule(id) => {
@@ -1226,8 +1225,7 @@ fn git_diff_part(
             hash = DUMMY_HASH.to_owned();
             content = FileContent {
                 is_binary: false, // TODO: are we sure this is never binary?
-                contents: materialize_merge_result_to_bytes(&contents, conflict_marker_style)
-                    .into(),
+                contents: materialize_merge_result_to_bytes(&contents, conflict_marker_style),
             };
         }
         MaterializedTreeValue::OtherConflict { id } => {
@@ -1235,7 +1233,7 @@ fn git_diff_part(
             hash = DUMMY_HASH.to_owned();
             content = FileContent {
                 is_binary: false,
-                contents: id.describe().into_bytes(),
+                contents: id.describe().into(),
             };
         }
         MaterializedTreeValue::Tree(_) => {
@@ -1322,8 +1320,7 @@ impl<'content> UnifiedDiffHunk<'content> {
 }
 
 fn unified_diff_hunks<'content>(
-    left_content: &'content [u8],
-    right_content: &'content [u8],
+    contents: [&'content BStr; 2],
     options: &UnifiedDiffOptions,
 ) -> Vec<UnifiedDiffHunk<'content>> {
     let mut hunks = vec![];
@@ -1332,7 +1329,7 @@ fn unified_diff_hunks<'content>(
         right_line_range: 0..0,
         lines: vec![],
     };
-    let diff = diff_by_line([left_content, right_content], &options.line_diff);
+    let diff = diff_by_line(contents, &options.line_diff);
     let mut diff_hunks = diff.hunks().peekable();
     while let Some(hunk) = diff_hunks.next() {
         match hunk.kind {
@@ -1368,7 +1365,7 @@ fn unified_diff_hunks<'content>(
                 current_hunk.extend_context_lines(before_lines.into_iter().rev());
             }
             DiffHunkKind::Different => {
-                let (left_lines, right_lines) =
+                let [left_lines, right_lines] =
                     unzip_diff_hunks_to_lines(Diff::by_word(hunk.contents).hunks());
                 current_hunk.extend_removed_lines(left_lines);
                 current_hunk.extend_added_lines(right_lines);
@@ -1381,10 +1378,8 @@ fn unified_diff_hunks<'content>(
     hunks
 }
 
-/// Splits `(left, right)` hunk pairs into `(left_lines, right_lines)`.
-fn unzip_diff_hunks_to_lines<'content, I>(
-    diff_hunks: I,
-) -> (Vec<DiffTokenVec<'content>>, Vec<DiffTokenVec<'content>>)
+/// Splits `[left, right]` hunk pairs into `[left_lines, right_lines]`.
+fn unzip_diff_hunks_to_lines<'content, I>(diff_hunks: I) -> [Vec<DiffTokenVec<'content>>; 2]
 where
     I: IntoIterator,
     I::Item: Borrow<DiffHunk<'content>>,
@@ -1435,13 +1430,12 @@ where
     if !right_tokens.is_empty() {
         right_lines.push(right_tokens);
     }
-    (left_lines, right_lines)
+    [left_lines, right_lines]
 }
 
 fn show_unified_diff_hunks(
     formatter: &mut dyn Formatter,
-    left_content: &[u8],
-    right_content: &[u8],
+    contents: [&BStr; 2],
     options: &UnifiedDiffOptions,
 ) -> io::Result<()> {
     // "If the chunk size is 0, the first number is one lower than one would
@@ -1459,7 +1453,7 @@ fn show_unified_diff_hunks(
         }
     }
 
-    for hunk in unified_diff_hunks(left_content, right_content, options) {
+    for hunk in unified_diff_hunks(contents, options) {
         writeln!(
             formatter.labeled("hunk_header"),
             "@@ -{},{} +{},{} @@",
@@ -1588,8 +1582,7 @@ pub fn show_git_diff(
                 })?;
                 show_unified_diff_hunks(
                     formatter,
-                    &left_part.content.contents,
-                    &right_part.content.contents,
+                    [&left_part.content.contents, &right_part.content.contents].map(BStr::new),
                     options,
                 )?;
             }
@@ -1671,7 +1664,11 @@ impl DiffStats {
                 let (left, right) = values?;
                 let left_content = diff_content(path.source(), left, conflict_marker_style)?;
                 let right_content = diff_content(path.target(), right, conflict_marker_style)?;
-                let stat = get_diff_stat_entry(path, &left_content, &right_content, options);
+                let stat = get_diff_stat_entry(
+                    path,
+                    [&left_content.contents, &right_content.contents].map(BStr::new),
+                    options,
+                );
                 BackendResult::Ok(stat)
             })
             .try_collect()
@@ -1704,17 +1701,13 @@ pub struct DiffStatEntry {
 
 fn get_diff_stat_entry(
     path: CopiesTreeDiffEntryPath,
-    left_content: &FileContent,
-    right_content: &FileContent,
+    contents: [&BStr; 2],
     options: &DiffStatOptions,
 ) -> DiffStatEntry {
     // TODO: this matches git's behavior, which is to count the number of newlines
     // in the file. but that behavior seems unhelpful; no one really cares how
     // many `0x0a` characters are in an image.
-    let diff = diff_by_line(
-        [&left_content.contents, &right_content.contents],
-        &options.line_diff,
-    );
+    let diff = diff_by_line(contents, &options.line_diff);
     let mut added = 0;
     let mut removed = 0;
     for hunk in diff.hunks() {
