@@ -16,8 +16,8 @@
 
 use std::error;
 use std::io;
-use std::io::Read;
-use std::io::Write;
+use std::io::Read as _;
+use std::io::Write as _;
 use std::iter;
 use std::mem;
 use std::path::Path;
@@ -28,20 +28,21 @@ use std::time::Instant;
 
 use crossterm::terminal::Clear;
 use crossterm::terminal::ClearType;
-use itertools::Itertools;
+use indoc::writedoc;
+use itertools::Itertools as _;
 use jj_lib::fmt_util::binary_prefix;
 use jj_lib::git;
-use jj_lib::git::FailedRefExport;
 use jj_lib::git::FailedRefExportReason;
+use jj_lib::git::GitExportStats;
 use jj_lib::git::GitImportStats;
-use jj_lib::git::RefName;
+use jj_lib::git::GitRefKind;
 use jj_lib::op_store::RefTarget;
 use jj_lib::op_store::RemoteRef;
+use jj_lib::ref_name::RemoteRefSymbol;
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo;
-use jj_lib::store::Store;
 use jj_lib::workspace::Workspace;
-use unicode_width::UnicodeWidthStr;
+use unicode_width::UnicodeWidthStr as _;
 
 use crate::cleanup_guard::CleanupGuard;
 use crate::command_error::cli_error;
@@ -51,11 +52,6 @@ use crate::formatter::Formatter;
 use crate::ui::ProgressOutput;
 use crate::ui::Ui;
 
-// TODO: migrate to gitoxide or subprocess, and remove this function
-pub(crate) fn get_git_repo(store: &Store) -> Result<git2::Repository, CommandError> {
-    Ok(git::get_git_backend(store)?.open_git_repo()?)
-}
-
 pub fn is_colocated_git_workspace(workspace: &Workspace, repo: &ReadonlyRepo) -> bool {
     let Ok(git_backend) = git::get_git_backend(repo.store()) else {
         return false;
@@ -294,22 +290,23 @@ pub fn print_git_import_stats(
         return Ok(());
     };
     if show_ref_stats {
-        let refs_stats = stats
-            .changed_remote_refs
-            .iter()
-            .map(|(ref_name, (remote_ref, ref_target))| {
-                RefStatus::new(ref_name, remote_ref, ref_target, repo)
-            })
-            .collect_vec();
-
-        let has_both_ref_kinds = refs_stats
-            .iter()
-            .any(|x| matches!(x.ref_kind, RefKind::Branch))
-            && refs_stats
+        let refs_stats = [
+            (GitRefKind::Bookmark, &stats.changed_remote_bookmarks),
+            (GitRefKind::Tag, &stats.changed_remote_tags),
+        ]
+        .into_iter()
+        .flat_map(|(kind, changes)| {
+            changes
                 .iter()
-                .any(|x| matches!(x.ref_kind, RefKind::Tag));
+                .map(move |(symbol, (remote_ref, ref_target))| {
+                    RefStatus::new(kind, symbol.as_ref(), remote_ref, ref_target, repo)
+                })
+        })
+        .collect_vec();
 
-        let max_width = refs_stats.iter().map(|x| x.ref_name.width()).max();
+        let has_both_ref_kinds =
+            !stats.changed_remote_bookmarks.is_empty() && !stats.changed_remote_tags.is_empty();
+        let max_width = refs_stats.iter().map(|x| x.symbol.width()).max();
         if let Some(max_width) = max_width {
             for status in refs_stats {
                 status.output(max_width, has_both_ref_kinds, &mut *formatter)?;
@@ -325,6 +322,30 @@ pub fn print_git_import_stats(
         )?;
     }
 
+    if !stats.failed_ref_names.is_empty() {
+        writeln!(ui.warning_default(), "Failed to import some Git refs:")?;
+        let mut formatter = ui.stderr_formatter();
+        for name in &stats.failed_ref_names {
+            write!(formatter, "  ")?;
+            write!(formatter.labeled("git_ref"), "{name}")?;
+            writeln!(formatter)?;
+        }
+    }
+    if stats
+        .failed_ref_names
+        .iter()
+        .any(|name| name.starts_with(git::RESERVED_REMOTE_REF_NAMESPACE.as_bytes()))
+    {
+        writedoc!(
+            ui.hint_default(),
+            "
+            Git remote named '{name}' is reserved for local Git repository.
+            Use `jj git remote rename` to give a different name.
+            ",
+            name = git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.as_symbol(),
+        )?;
+    }
+
     Ok(())
 }
 
@@ -477,37 +498,29 @@ impl RateEstimateState {
 }
 
 struct RefStatus {
-    ref_kind: RefKind,
-    ref_name: String,
+    ref_kind: GitRefKind,
+    symbol: String,
     tracking_status: TrackingStatus,
     import_status: ImportStatus,
 }
 
 impl RefStatus {
     fn new(
-        ref_name: &RefName,
+        ref_kind: GitRefKind,
+        symbol: RemoteRefSymbol<'_>,
         remote_ref: &RemoteRef,
         ref_target: &RefTarget,
         repo: &dyn Repo,
     ) -> Self {
-        let (ref_name, ref_kind, tracking_status) = match ref_name {
-            RefName::RemoteBranch(symbol) => (
-                format!("{symbol}"),
-                RefKind::Branch,
-                if repo
-                    .view()
-                    .get_remote_bookmark(symbol.as_ref())
-                    .is_tracking()
-                {
+        let tracking_status = match ref_kind {
+            GitRefKind::Bookmark => {
+                if repo.view().get_remote_bookmark(symbol).is_tracked() {
                     TrackingStatus::Tracked
                 } else {
                     TrackingStatus::Untracked
-                },
-            ),
-            RefName::Tag(tag) => (tag.clone(), RefKind::Tag, TrackingStatus::NotApplicable),
-            RefName::LocalBranch(branch) => {
-                (branch.clone(), RefKind::Branch, TrackingStatus::Tracked)
+                }
             }
+            GitRefKind::Tag => TrackingStatus::NotApplicable,
         };
 
         let import_status = match (remote_ref.target.is_absent(), ref_target.is_absent()) {
@@ -517,7 +530,7 @@ impl RefStatus {
         };
 
         Self {
-            ref_name,
+            symbol: symbol.to_string(),
             tracking_status,
             import_status,
             ref_kind,
@@ -526,7 +539,7 @@ impl RefStatus {
 
     fn output(
         &self,
-        max_ref_name_width: usize,
+        max_symbol_width: usize,
         has_both_ref_kinds: bool,
         out: &mut dyn Formatter,
     ) -> std::io::Result<()> {
@@ -542,27 +555,22 @@ impl RefStatus {
             ImportStatus::Updated => "updated",
         };
 
-        let ref_name_display_width = self.ref_name.width();
-        let pad_width = max_ref_name_width.saturating_sub(ref_name_display_width);
-        let padded_ref_name = format!("{}{:>pad_width$}", self.ref_name, "", pad_width = pad_width);
+        let symbol_width = self.symbol.width();
+        let pad_width = max_symbol_width.saturating_sub(symbol_width);
+        let padded_symbol = format!("{}{:>pad_width$}", self.symbol, "", pad_width = pad_width);
 
         let ref_kind = match self.ref_kind {
-            RefKind::Branch => "bookmark: ",
-            RefKind::Tag if !has_both_ref_kinds => "tag: ",
-            RefKind::Tag => "tag:    ",
+            GitRefKind::Bookmark => "bookmark: ",
+            GitRefKind::Tag if !has_both_ref_kinds => "tag: ",
+            GitRefKind::Tag => "tag:    ",
         };
 
         write!(out, "{ref_kind}")?;
-        write!(out.labeled("bookmark"), "{padded_ref_name}")?;
+        write!(out.labeled("bookmark"), "{padded_symbol}")?;
         writeln!(out, " [{import_status}] {tracking_status}")
     }
 }
 
-enum RefKind {
-    Branch,
-    Tag,
-}
-
 enum TrackingStatus {
     Tracked,
     Untracked,
@@ -575,25 +583,23 @@ enum ImportStatus {
     Updated,
 }
 
-pub fn print_failed_git_export(
-    ui: &Ui,
-    failed_refs: &[FailedRefExport],
-) -> Result<(), std::io::Error> {
-    if !failed_refs.is_empty() {
+pub fn print_git_export_stats(ui: &Ui, stats: &GitExportStats) -> Result<(), std::io::Error> {
+    if !stats.failed_bookmarks.is_empty() {
         writeln!(ui.warning_default(), "Failed to export some bookmarks:")?;
         let mut formatter = ui.stderr_formatter();
-        for FailedRefExport { name, reason } in failed_refs {
+        for (symbol, reason) in &stats.failed_bookmarks {
             write!(formatter, "  ")?;
-            write!(formatter.labeled("bookmark"), "{name}")?;
+            write!(formatter.labeled("bookmark"), "{symbol}")?;
             for err in iter::successors(Some(reason as &dyn error::Error), |err| err.source()) {
                 write!(formatter, ": {err}")?;
             }
             writeln!(formatter)?;
         }
         drop(formatter);
-        if failed_refs
+        if stats
+            .failed_bookmarks
             .iter()
-            .any(|failed| matches!(failed.reason, FailedRefExportReason::FailedToSet(_)))
+            .any(|(_, reason)| matches!(reason, FailedRefExportReason::FailedToSet(_)))
         {
             writeln!(
                 ui.hint_default(),
@@ -702,7 +708,7 @@ mod tests {
         };
         // First output is after the initial delay
         assert_snapshot!(update(crate::progress::INITIAL_DELAY - Duration::from_millis(1), 0.1), @"");
-        assert_snapshot!(update(Duration::from_millis(1), 0.10), @"[?25l\r 10% [█▊                ][K");
+        assert_snapshot!(update(Duration::from_millis(1), 0.10), @"\u{1b}[?25l\r 10% [█▊                ]\u{1b}[K");
         // No updates for the next 30 milliseconds
         assert_snapshot!(update(Duration::from_millis(10), 0.11), @"");
         assert_snapshot!(update(Duration::from_millis(10), 0.12), @"");
