@@ -2,12 +2,11 @@ use std::borrow::Cow;
 use std::path::Path;
 use std::sync::Arc;
 
-use futures::StreamExt;
-use futures::TryFutureExt;
-use futures::TryStreamExt;
-use itertools::Itertools;
+use futures::StreamExt as _;
+use futures::TryFutureExt as _;
+use futures::TryStreamExt as _;
+use itertools::Itertools as _;
 use jj_lib::backend::BackendResult;
-use jj_lib::backend::FileId;
 use jj_lib::backend::MergedTreeId;
 use jj_lib::backend::TreeValue;
 use jj_lib::conflicts::materialize_merge_result_to_bytes;
@@ -23,11 +22,11 @@ use jj_lib::merge::Merge;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::merged_tree::MergedTreeBuilder;
 use jj_lib::merged_tree::TreeDiffEntry;
-use jj_lib::object_id::ObjectId;
+use jj_lib::object_id::ObjectId as _;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::store::Store;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use thiserror::Error;
 
 use super::MergeToolFile;
@@ -36,12 +35,6 @@ use super::MergeToolFile;
 pub enum BuiltinToolError {
     #[error("Failed to record changes")]
     Record(#[from] scm_record::RecordError),
-    #[error("Failed to read file {path:?} with ID {id}")]
-    ReadFileIo {
-        path: RepoPathBuf,
-        id: FileId,
-        source: std::io::Error,
-    },
     #[error("Failed to decode UTF-8 text for item {item} (this should not happen)")]
     DecodeUtf8 {
         source: std::str::Utf8Error,
@@ -75,27 +68,6 @@ pub struct FileInfo {
     contents: FileContents,
 }
 
-impl FileInfo {
-    // Returns `Some(true)` if the file exists and is empty.
-    // Returns `None` if the file is absent and does not exist.
-    fn is_empty(&self) -> Option<bool> {
-        match self.contents {
-            FileContents::Absent => {
-                // If the contents are `Absent` the mode should be `Absent` as
-                // well since the file doesn't exist.
-                debug_assert!(self.file_mode == scm_record::FileMode::absent());
-                None
-            }
-            FileContents::Text {
-                contents: _,
-                hash: _,
-                num_bytes,
-            }
-            | FileContents::Binary { hash: _, num_bytes } => Some(num_bytes == 0),
-        }
-    }
-}
-
 /// File modes according to the Git file mode conventions. used for display
 /// purposes and equality comparison.
 ///
@@ -103,9 +75,10 @@ impl FileInfo {
 /// figure out some other way to represent file mode changes in a jj-compatible
 /// manner?
 mod mode {
-    pub const NORMAL: usize = 0o100644;
-    pub const EXECUTABLE: usize = 0o100755;
-    pub const SYMLINK: usize = 0o120000;
+    pub const ABSENT: scm_record::FileMode = scm_record::FileMode::Absent;
+    pub const NORMAL: scm_record::FileMode = scm_record::FileMode::Unix(0o100644);
+    pub const EXECUTABLE: scm_record::FileMode = scm_record::FileMode::Unix(0o100755);
+    pub const SYMLINK: scm_record::FileMode = scm_record::FileMode::Unix(0o120000);
 }
 
 fn describe_binary(hash: Option<&str>, num_bytes: u64) -> String {
@@ -146,11 +119,11 @@ fn read_file_contents(
         .block_on()?;
     match materialized_value {
         MaterializedTreeValue::Absent => Ok(FileInfo {
-            file_mode: scm_record::FileMode::absent(),
+            file_mode: mode::ABSENT,
             contents: FileContents::Absent,
         }),
         MaterializedTreeValue::AccessDenied(err) => Ok(FileInfo {
-            file_mode: scm_record::FileMode(mode::NORMAL),
+            file_mode: mode::NORMAL,
             contents: FileContents::Text {
                 contents: format!("Access denied: {err}"),
                 hash: None,
@@ -158,26 +131,14 @@ fn read_file_contents(
             },
         }),
 
-        MaterializedTreeValue::File {
-            id,
-            executable,
-            mut reader,
-        } => {
-            let mut buf = Vec::new();
-            reader
-                .read_to_end(&mut buf)
-                .map_err(|err| BuiltinToolError::ReadFileIo {
-                    path: path.to_owned(),
-                    id: id.clone(),
-                    source: err,
-                })?;
-
-            let file_mode = if executable {
-                scm_record::FileMode(mode::EXECUTABLE)
+        MaterializedTreeValue::File(mut file) => {
+            let buf = file.read_all(path)?;
+            let file_mode = if file.executable {
+                mode::EXECUTABLE
             } else {
-                scm_record::FileMode(mode::NORMAL)
+                mode::NORMAL
             };
-            let contents = buf_to_file_contents(Some(id.hex()), buf);
+            let contents = buf_to_file_contents(Some(file.id.hex()), buf);
             Ok(FileInfo {
                 file_mode,
                 contents,
@@ -185,7 +146,7 @@ fn read_file_contents(
         }
 
         MaterializedTreeValue::Symlink { id, target } => {
-            let file_mode = scm_record::FileMode(mode::SYMLINK);
+            let file_mode = mode::SYMLINK;
             let num_bytes = target.len().try_into().unwrap();
             Ok(FileInfo {
                 file_mode,
@@ -213,7 +174,7 @@ fn read_file_contents(
             // TODO: Render the ID somehow?
             let contents = buf_to_file_contents(None, buf);
             Ok(FileInfo {
-                file_mode: scm_record::FileMode(mode::NORMAL),
+                file_mode: mode::NORMAL,
                 contents,
             })
         }
@@ -221,7 +182,7 @@ fn read_file_contents(
             // TODO: Render the ID somehow?
             let contents = buf_to_file_contents(None, id.describe().into_bytes());
             Ok(FileInfo {
-                file_mode: scm_record::FileMode(mode::NORMAL),
+                file_mode: mode::NORMAL,
                 contents,
             })
         }
@@ -291,17 +252,6 @@ fn make_diff_sections(
     Ok(sections)
 }
 
-fn should_render_mode_section(left: &FileInfo, right: &FileInfo) -> bool {
-    match (left.is_empty(), right.is_empty()) {
-        // The file only exists on one side, but it's not empty on the other
-        // side. We'll render a section for the change in content, so the mode
-        // change is implied and mandatory if the user selects the change in
-        // content.
-        (Some(false), None) | (None, Some(false)) => false,
-        _ => left.file_mode != right.file_mode,
-    }
-}
-
 pub fn make_diff_files(
     store: &Arc<Store>,
     left_tree: &MergedTree,
@@ -316,11 +266,10 @@ pub fn make_diff_files(
             read_file_contents(store, right_tree, changed_path, conflict_marker_style)?;
         let mut sections = Vec::new();
 
-        if should_render_mode_section(&left_info, &right_info) {
+        if left_info.file_mode != right_info.file_mode {
             sections.push(scm_record::Section::FileMode {
                 is_checked: false,
-                before: left_info.file_mode,
-                after: right_info.file_mode,
+                mode: right_info.file_mode,
             });
         }
 
@@ -435,7 +384,7 @@ pub fn make_diff_files(
             old_path: None,
             // Path for displaying purposes, not for file access.
             path: Cow::Owned(changed_path.to_fs_path_unchecked(Path::new(""))),
-            file_mode: Some(left_info.file_mode),
+            file_mode: left_info.file_mode,
             sections,
         });
     }
@@ -457,37 +406,22 @@ pub fn apply_diff_builtin(
     );
     // TODO: Write files concurrently
     for (path, file) in changed_files.into_iter().zip(files) {
-        let (selected, _unselected) = file.get_selected_contents();
-        match selected {
-            scm_record::SelectedContents::Absent => {
-                // TODO(https://github.com/arxanas/scm-record/issues/26): This
-                // is probably an upstream bug in scm-record. If the file exists
-                // but is empty, `get_selected_contents` should probably return
-                // `Unchanged` or `Present`. When this is fixed upstream we can
-                // simplify this logic.
+        let (
+            scm_record::SelectedChanges {
+                contents,
+                file_mode,
+            },
+            _unselected,
+        ) = file.get_selected_contents();
 
-                // Currently, `Absent` means the file is either empty or
-                // deleted. We need to disambiguate three cases:
-                // 1. The file is new and empty.
-                // 2. The file existed before, is empty, and nothing changed.
-                // 3. The file does not exist (it's been deleted).
-                let old_mode = file.file_mode;
-                let new_mode = file.get_file_mode();
-                let file_existed_previously =
-                    old_mode.is_some() && old_mode != Some(scm_record::FileMode::absent());
-                let file_exists_now =
-                    new_mode.is_some() && new_mode != Some(scm_record::FileMode::absent());
-                let new_empty_file = !file_existed_previously && file_exists_now;
-                let file_deleted = file_existed_previously && !file_exists_now;
+        // If a file was not present in the selected changes (i.e. split out of a
+        // change, deleted, etc.) remove it from the tree.
+        if file_mode == scm_record::FileMode::Absent {
+            tree_builder.set_or_remove(path, Merge::absent());
+            continue;
+        }
 
-                if new_empty_file {
-                    let value = right_tree.path_value(&path)?;
-                    tree_builder.set_or_remove(path, value);
-                } else if file_deleted {
-                    tree_builder.set_or_remove(path, Merge::absent());
-                }
-                // Else: the file is empty and nothing changed.
-            }
+        match contents {
             scm_record::SelectedContents::Unchanged => {
                 // Do nothing.
             }
@@ -498,16 +432,16 @@ pub fn apply_diff_builtin(
                 let value = right_tree.path_value(&path)?;
                 tree_builder.set_or_remove(path, value);
             }
-            scm_record::SelectedContents::Present { contents } => {
+            scm_record::SelectedContents::Text { contents } => {
                 let file_id = store
                     .write_file(&path, &mut contents.as_bytes())
                     .block_on()?;
+
                 tree_builder.set_or_remove(
                     path,
                     Merge::normal(TreeValue::File {
                         id: file_id,
-                        executable: file.get_file_mode()
-                            == Some(scm_record::FileMode(mode::EXECUTABLE)),
+                        executable: file_mode == mode::EXECUTABLE,
                     }),
                 );
             }
@@ -646,7 +580,7 @@ fn make_merge_file(
                 .repo_path
                 .to_fs_path_unchecked(Path::new("")),
         ),
-        file_mode: None,
+        file_mode: mode::NORMAL,
         sections,
     })
 }
@@ -681,9 +615,10 @@ pub fn edit_merge_builtin(
 
 #[cfg(test)]
 mod tests {
+    use jj_lib::backend::FileId;
     use jj_lib::conflicts::extract_as_single_hunk;
     use jj_lib::merge::MergedTreeValue;
-    use jj_lib::repo::Repo;
+    use jj_lib::repo::Repo as _;
     use testutils::TestRepo;
 
     use super::*;
@@ -728,15 +663,13 @@ mod tests {
             ConflictMarkerStyle::Diff,
         )
         .unwrap();
-        insta::assert_debug_snapshot!(files, @r###"
+        insta::assert_debug_snapshot!(files, @r#"
         [
             File {
                 old_path: None,
                 path: "unchanged",
-                file_mode: Some(
-                    FileMode(
-                        33188,
-                    ),
+                file_mode: Unix(
+                    33188,
                 ),
                 sections: [
                     Unchanged {
@@ -749,10 +682,8 @@ mod tests {
             File {
                 old_path: None,
                 path: "changed",
-                file_mode: Some(
-                    FileMode(
-                        33188,
-                    ),
+                file_mode: Unix(
+                    33188,
                 ),
                 sections: [
                     Unchanged {
@@ -798,12 +729,14 @@ mod tests {
             File {
                 old_path: None,
                 path: "added",
-                file_mode: Some(
-                    FileMode(
-                        0,
-                    ),
-                ),
+                file_mode: Absent,
                 sections: [
+                    FileMode {
+                        is_checked: false,
+                        mode: Unix(
+                            33188,
+                        ),
+                    },
                     Changed {
                         lines: [
                             SectionChangedLine {
@@ -816,7 +749,7 @@ mod tests {
                 ],
             },
         ]
-        "###);
+        "#);
 
         let no_changes_tree_id = apply_diff_builtin(
             store,
@@ -865,25 +798,174 @@ mod tests {
             ConflictMarkerStyle::Diff,
         )
         .unwrap();
-        insta::assert_debug_snapshot!(files, @r###"
+        insta::assert_debug_snapshot!(files, @r#"
         [
             File {
                 old_path: None,
                 path: "empty_file",
-                file_mode: Some(
-                    FileMode(
-                        0,
-                    ),
+                file_mode: Absent,
+                sections: [
+                    FileMode {
+                        is_checked: false,
+                        mode: Unix(
+                            33188,
+                        ),
+                    },
+                ],
+            },
+        ]
+        "#);
+        let no_changes_tree_id = apply_diff_builtin(
+            store,
+            &left_tree,
+            &right_tree,
+            changed_files.clone(),
+            &files,
+        )
+        .unwrap();
+        let no_changes_tree = store.get_root_tree(&no_changes_tree_id).unwrap();
+        assert_eq!(
+            no_changes_tree.id(),
+            left_tree.id(),
+            "no-changes tree was different",
+        );
+
+        let mut files = files;
+        for file in &mut files {
+            file.toggle_all();
+        }
+        let all_changes_tree_id =
+            apply_diff_builtin(store, &left_tree, &right_tree, changed_files, &files).unwrap();
+        let all_changes_tree = store.get_root_tree(&all_changes_tree_id).unwrap();
+        assert_eq!(
+            all_changes_tree.id(),
+            right_tree.id(),
+            "all-changes tree was different",
+        );
+    }
+
+    #[test]
+    fn test_edit_diff_builtin_add_executable_file() {
+        let test_repo = TestRepo::init();
+        let store = test_repo.repo.store();
+
+        let added_executable_file_path = RepoPath::from_internal_string("executable_file");
+        let left_tree = testutils::create_tree(&test_repo.repo, &[]);
+        let right_tree = {
+            // let store = test_repo.repo.store();
+            let mut tree_builder = store.tree_builder(store.empty_tree_id().clone());
+            testutils::write_executable_file(
+                &mut tree_builder,
+                added_executable_file_path,
+                "executable",
+            );
+            let id = tree_builder.write_tree().unwrap();
+            MergedTree::resolved(store.get_tree(RepoPathBuf::root(), &id).unwrap())
+        };
+
+        let changed_files = vec![added_executable_file_path.to_owned()];
+        let files = make_diff_files(
+            store,
+            &left_tree,
+            &right_tree,
+            &changed_files,
+            ConflictMarkerStyle::Diff,
+        )
+        .unwrap();
+        insta::assert_debug_snapshot!(files, @r###"
+        [
+            File {
+                old_path: None,
+                path: "executable_file",
+                file_mode: Absent,
+                sections: [
+                    FileMode {
+                        is_checked: false,
+                        mode: Unix(
+                            33261,
+                        ),
+                    },
+                    Changed {
+                        lines: [
+                            SectionChangedLine {
+                                is_checked: false,
+                                change_type: Added,
+                                line: "executable",
+                            },
+                        ],
+                    },
+                ],
+            },
+        ]
+        "###);
+        let no_changes_tree_id = apply_diff_builtin(
+            store,
+            &left_tree,
+            &right_tree,
+            changed_files.clone(),
+            &files,
+        )
+        .unwrap();
+        let no_changes_tree = store.get_root_tree(&no_changes_tree_id).unwrap();
+        assert_eq!(
+            no_changes_tree.id(),
+            left_tree.id(),
+            "no-changes tree was different",
+        );
+
+        let mut files = files;
+        for file in &mut files {
+            file.toggle_all();
+        }
+        let all_changes_tree_id =
+            apply_diff_builtin(store, &left_tree, &right_tree, changed_files, &files).unwrap();
+        let all_changes_tree = store.get_root_tree(&all_changes_tree_id).unwrap();
+        assert_eq!(
+            all_changes_tree.id(),
+            right_tree.id(),
+            "all-changes tree was different",
+        );
+    }
+
+    #[test]
+    fn test_edit_diff_builtin_delete_file() {
+        let test_repo = TestRepo::init();
+        let store = test_repo.repo.store();
+
+        let file_path = RepoPath::from_internal_string("file_with_content");
+        let left_tree = testutils::create_tree(&test_repo.repo, &[(file_path, "content\n")]);
+        let right_tree = testutils::create_tree(&test_repo.repo, &[]);
+
+        let changed_files = vec![file_path.to_owned()];
+        let files = make_diff_files(
+            store,
+            &left_tree,
+            &right_tree,
+            &changed_files,
+            ConflictMarkerStyle::Diff,
+        )
+        .unwrap();
+        insta::assert_debug_snapshot!(files, @r###"
+        [
+            File {
+                old_path: None,
+                path: "file_with_content",
+                file_mode: Unix(
+                    33188,
                 ),
                 sections: [
                     FileMode {
                         is_checked: false,
-                        before: FileMode(
-                            0,
-                        ),
-                        after: FileMode(
-                            33188,
-                        ),
+                        mode: Absent,
+                    },
+                    Changed {
+                        lines: [
+                            SectionChangedLine {
+                                is_checked: false,
+                                change_type: Removed,
+                                line: "content\n",
+                            },
+                        ],
                     },
                 ],
             },
@@ -936,30 +1018,23 @@ mod tests {
             ConflictMarkerStyle::Diff,
         )
         .unwrap();
-        insta::assert_debug_snapshot!(files, @r###"
+        insta::assert_debug_snapshot!(files, @r#"
         [
             File {
                 old_path: None,
                 path: "empty_file",
-                file_mode: Some(
-                    FileMode(
-                        33188,
-                    ),
+                file_mode: Unix(
+                    33188,
                 ),
                 sections: [
                     FileMode {
                         is_checked: false,
-                        before: FileMode(
-                            33188,
-                        ),
-                        after: FileMode(
-                            0,
-                        ),
+                        mode: Absent,
                     },
                 ],
             },
         ]
-        "###);
+        "#);
         let no_changes_tree_id = apply_diff_builtin(
             store,
             &left_tree,
@@ -1008,15 +1083,13 @@ mod tests {
             ConflictMarkerStyle::Diff,
         )
         .unwrap();
-        insta::assert_debug_snapshot!(files, @r###"
+        insta::assert_debug_snapshot!(files, @r#"
         [
             File {
                 old_path: None,
                 path: "empty_file",
-                file_mode: Some(
-                    FileMode(
-                        33188,
-                    ),
+                file_mode: Unix(
+                    33188,
                 ),
                 sections: [
                     Changed {
@@ -1031,6 +1104,75 @@ mod tests {
                 ],
             },
         ]
+        "#);
+        let no_changes_tree_id = apply_diff_builtin(
+            store,
+            &left_tree,
+            &right_tree,
+            changed_files.clone(),
+            &files,
+        )
+        .unwrap();
+        let no_changes_tree = store.get_root_tree(&no_changes_tree_id).unwrap();
+        assert_eq!(
+            no_changes_tree.id(),
+            left_tree.id(),
+            "no-changes tree was different",
+        );
+
+        let mut files = files;
+        for file in &mut files {
+            file.toggle_all();
+        }
+        let all_changes_tree_id =
+            apply_diff_builtin(store, &left_tree, &right_tree, changed_files, &files).unwrap();
+        let all_changes_tree = store.get_root_tree(&all_changes_tree_id).unwrap();
+        assert_eq!(
+            all_changes_tree.id(),
+            right_tree.id(),
+            "all-changes tree was different",
+        );
+    }
+
+    #[test]
+    fn test_edit_diff_builtin_make_file_empty() {
+        let test_repo = TestRepo::init();
+        let store = test_repo.repo.store();
+
+        let file_path = RepoPath::from_internal_string("file_with_content");
+        let left_tree = testutils::create_tree(&test_repo.repo, &[(file_path, "content\n")]);
+        let right_tree = testutils::create_tree(&test_repo.repo, &[(file_path, "")]);
+
+        let changed_files = vec![file_path.to_owned()];
+        let files = make_diff_files(
+            store,
+            &left_tree,
+            &right_tree,
+            &changed_files,
+            ConflictMarkerStyle::Diff,
+        )
+        .unwrap();
+        insta::assert_debug_snapshot!(files, @r###"
+        [
+            File {
+                old_path: None,
+                path: "file_with_content",
+                file_mode: Unix(
+                    33188,
+                ),
+                sections: [
+                    Changed {
+                        lines: [
+                            SectionChangedLine {
+                                is_checked: false,
+                                change_type: Removed,
+                                line: "content\n",
+                            },
+                        ],
+                    },
+                ],
+            },
+        ]
         "###);
         let no_changes_tree_id = apply_diff_builtin(
             store,
@@ -1098,7 +1240,7 @@ mod tests {
             .unwrap();
         let merge_result = files::merge(&content);
         let sections = make_merge_sections(merge_result).unwrap();
-        insta::assert_debug_snapshot!(sections, @r###"
+        insta::assert_debug_snapshot!(sections, @r#"
         [
             Changed {
                 lines: [
@@ -1146,6 +1288,6 @@ mod tests {
                 ],
             },
         ]
-        "###);
+        "#);
     }
 }
