@@ -7,9 +7,8 @@ use std::process::Stdio;
 use std::sync::Arc;
 
 use bstr::BString;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::backend::MergedTreeId;
-use jj_lib::backend::TreeValue;
 use jj_lib::conflicts;
 use jj_lib::conflicts::choose_materialized_conflict_marker_len;
 use jj_lib::conflicts::materialize_merge_result_to_bytes_with_marker_len;
@@ -23,7 +22,7 @@ use jj_lib::merged_tree::MergedTreeBuilder;
 use jj_lib::repo_path::RepoPathUiConverter;
 use jj_lib::store::Store;
 use jj_lib::working_copy::CheckoutOptions;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use thiserror::Error;
 
 use super::diff_working_copies::check_out_trees;
@@ -303,19 +302,13 @@ fn run_mergetool_external_single_file(
             ExternalToolError::InvalidConflictMarkers { exit_status },
         ));
     }
-
-    let new_tree_value = match new_file_ids.into_resolved() {
-        Ok(new_file_id) => Merge::normal(TreeValue::File {
-            id: new_file_id.unwrap(),
-            executable: conflict
-                .to_executable_merge()
-                .as_ref()
-                .and_then(Merge::resolve_trivial)
-                .copied()
-                .unwrap_or_default(),
-        }),
-        Err(new_file_ids) => conflict.with_new_file_ids(&new_file_ids),
+    // Update the file ids only, leaving the executable flags unchanged
+    let new_file_ids = if let Some(resolved) = new_file_ids.as_resolved() {
+        Merge::from_vec(vec![resolved.clone(); conflict.as_slice().len()])
+    } else {
+        new_file_ids
     };
+    let new_tree_value = conflict.with_new_file_ids(&new_file_ids);
     tree_builder.set_or_remove(repo_path.to_owned(), new_tree_value);
     Ok(())
 }
