@@ -19,7 +19,8 @@ mod external;
 use std::sync::Arc;
 
 use bstr::BString;
-use itertools::Itertools;
+use itertools::Itertools as _;
+use jj_lib::backend::BackendError;
 use jj_lib::backend::FileId;
 use jj_lib::backend::MergedTreeId;
 use jj_lib::config::ConfigGetError;
@@ -32,13 +33,14 @@ use jj_lib::matchers::Matcher;
 use jj_lib::merge::Merge;
 use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::MergedTree;
+use jj_lib::merged_tree::MergedTreeBuilder;
 use jj_lib::repo_path::InvalidRepoPathError;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::repo_path::RepoPathUiConverter;
 use jj_lib::settings::UserSettings;
 use jj_lib::working_copy::SnapshotError;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use thiserror::Error;
 
 use self::builtin::edit_diff_builtin;
@@ -56,6 +58,8 @@ use crate::config::CommandNameAndArgs;
 use crate::ui::Ui;
 
 const BUILTIN_EDITOR_NAME: &str = ":builtin";
+const OURS_TOOL_NAME: &str = ":ours";
+const THEIRS_TOOL_NAME: &str = ":theirs";
 
 #[derive(Debug, Error)]
 pub enum DiffEditError {
@@ -127,6 +131,8 @@ pub enum MergeToolConfigError {
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub enum MergeTool {
     Builtin,
+    Ours,
+    Theirs,
     // Boxed because ExternalMergeTool is big compared to the Builtin variant.
     External(Box<ExternalMergeTool>),
 }
@@ -135,6 +141,45 @@ impl MergeTool {
     fn external(tool: ExternalMergeTool) -> Self {
         MergeTool::External(Box::new(tool))
     }
+
+    /// Resolves builtin merge tool names or loads external tool options from
+    /// `[merge-tools.<name>]`.
+    fn get_tool_config(
+        settings: &UserSettings,
+        name: &str,
+    ) -> Result<Option<Self>, MergeToolConfigError> {
+        match name {
+            BUILTIN_EDITOR_NAME => Ok(Some(MergeTool::Builtin)),
+            OURS_TOOL_NAME => Ok(Some(MergeTool::Ours)),
+            THEIRS_TOOL_NAME => Ok(Some(MergeTool::Theirs)),
+            _ => Ok(get_external_tool_config(settings, name)?.map(MergeTool::external)),
+        }
+    }
+}
+
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub enum DiffTool {
+    Builtin,
+    // Boxed because ExternalMergeTool is big compared to the Builtin variant.
+    External(Box<ExternalMergeTool>),
+}
+
+impl DiffTool {
+    fn external(tool: ExternalMergeTool) -> Self {
+        DiffTool::External(Box::new(tool))
+    }
+
+    /// Resolves builtin merge tool name or loads external tool options from
+    /// `[merge-tools.<name>]`.
+    fn get_tool_config(
+        settings: &UserSettings,
+        name: &str,
+    ) -> Result<Option<Self>, MergeToolConfigError> {
+        match name {
+            BUILTIN_EDITOR_NAME => Ok(Some(DiffTool::Builtin)),
+            _ => Ok(get_external_tool_config(settings, name)?.map(DiffTool::external)),
+        }
+    }
 }
 
 /// Finds the appropriate tool for diff editing or merges
@@ -159,19 +204,6 @@ fn editor_args_from_settings(
     }
 }
 
-/// Resolves builtin merge tool name or loads external tool options from
-/// `[merge-tools.<name>]`.
-fn get_tool_config(
-    settings: &UserSettings,
-    name: &str,
-) -> Result<Option<MergeTool>, ConfigGetError> {
-    if name == BUILTIN_EDITOR_NAME {
-        Ok(Some(MergeTool::Builtin))
-    } else {
-        Ok(get_external_tool_config(settings, name)?.map(MergeTool::external))
-    }
-}
-
 /// Loads external diff/merge tool options from `[merge-tools.<name>]`.
 pub fn get_external_tool_config(
     settings: &UserSettings,
@@ -190,7 +222,7 @@ pub fn get_external_tool_config(
 /// Configured diff editor.
 #[derive(Clone, Debug)]
 pub struct DiffEditor {
-    tool: MergeTool,
+    tool: DiffTool,
     base_ignores: Arc<GitIgnoreFile>,
     use_instructions: bool,
     conflict_marker_style: ConflictMarkerStyle,
@@ -205,8 +237,8 @@ impl DiffEditor {
         base_ignores: Arc<GitIgnoreFile>,
         conflict_marker_style: ConflictMarkerStyle,
     ) -> Result<Self, MergeToolConfigError> {
-        let tool = get_tool_config(settings, name)?
-            .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_program(name)));
+        let tool = DiffTool::get_tool_config(settings, name)?
+            .unwrap_or_else(|| DiffTool::external(ExternalMergeTool::with_program(name)));
         Self::new_inner(tool, settings, base_ignores, conflict_marker_style)
     }
 
@@ -219,16 +251,16 @@ impl DiffEditor {
     ) -> Result<Self, MergeToolConfigError> {
         let args = editor_args_from_settings(ui, settings, "ui.diff-editor")?;
         let tool = if let CommandNameAndArgs::String(name) = &args {
-            get_tool_config(settings, name)?
+            DiffTool::get_tool_config(settings, name)?
         } else {
             None
         }
-        .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_edit_args(&args)));
+        .unwrap_or_else(|| DiffTool::external(ExternalMergeTool::with_edit_args(&args)));
         Self::new_inner(tool, settings, base_ignores, conflict_marker_style)
     }
 
     fn new_inner(
-        tool: MergeTool,
+        tool: DiffTool,
         settings: &UserSettings,
         base_ignores: Arc<GitIgnoreFile>,
         conflict_marker_style: ConflictMarkerStyle,
@@ -257,13 +289,13 @@ impl DiffEditor {
         format_instructions: impl FnOnce() -> String,
     ) -> Result<MergedTreeId, DiffEditError> {
         match &self.tool {
-            MergeTool::Builtin => {
+            DiffTool::Builtin => {
                 Ok(
                     edit_diff_builtin(left_tree, right_tree, matcher, self.conflict_marker_style)
                         .map_err(Box::new)?,
                 )
             }
-            MergeTool::External(editor) => {
+            DiffTool::External(editor) => {
                 let instructions = self.use_instructions.then(format_instructions);
                 edit_diff_external(
                     editor,
@@ -337,7 +369,7 @@ impl MergeEditor {
         path_converter: RepoPathUiConverter,
         conflict_marker_style: ConflictMarkerStyle,
     ) -> Result<Self, MergeToolConfigError> {
-        let tool = get_tool_config(settings, name)?
+        let tool = MergeTool::get_tool_config(settings, name)?
             .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_program(name)));
         Self::new_inner(name, tool, path_converter, conflict_marker_style)
     }
@@ -351,7 +383,7 @@ impl MergeEditor {
     ) -> Result<Self, MergeToolConfigError> {
         let args = editor_args_from_settings(ui, settings, "ui.merge-editor")?;
         let tool = if let CommandNameAndArgs::String(name) = &args {
-            get_tool_config(settings, name)?
+            MergeTool::get_tool_config(settings, name)?
         } else {
             None
         }
@@ -394,6 +426,14 @@ impl MergeEditor {
                 let tree_id = edit_merge_builtin(tree, &merge_tool_files).map_err(Box::new)?;
                 Ok((tree_id, None))
             }
+            MergeTool::Ours => {
+                let tree_id = pick_conflict_side(tree, &merge_tool_files, 0)?;
+                Ok((tree_id, None))
+            }
+            MergeTool::Theirs => {
+                let tree_id = pick_conflict_side(tree, &merge_tool_files, 1)?;
+                Ok((tree_id, None))
+            }
             MergeTool::External(editor) => external::run_mergetool_external(
                 ui,
                 &self.path_converter,
@@ -406,6 +446,21 @@ impl MergeEditor {
     }
 }
 
+fn pick_conflict_side(
+    tree: &MergedTree,
+    merge_tool_files: &[MergeToolFile],
+    add_index: usize,
+) -> Result<MergedTreeId, BackendError> {
+    let mut tree_builder = MergedTreeBuilder::new(tree.id());
+    for merge_tool_file in merge_tool_files {
+        tree_builder.set_or_remove(
+            merge_tool_file.repo_path.clone(),
+            Merge::resolved(merge_tool_file.conflict.get_add(add_index).unwrap().clone()),
+        );
+    }
+    tree_builder.write_tree(tree.store())
+}
+
 #[cfg(test)]
 mod tests {
     use jj_lib::config::ConfigLayer;
@@ -703,11 +758,11 @@ mod tests {
         insta::assert_debug_snapshot!(get(":builtin", "").unwrap(), @"Builtin");
 
         // Just program name
-        insta::assert_debug_snapshot!(get("my diff", "").unwrap_err(), @r###"
+        insta::assert_debug_snapshot!(get("my diff", "").unwrap_err(), @r#"
         MergeArgsNotConfigured {
             tool_name: "my diff",
         }
-        "###);
+        "#);
 
         // Pick from merge-tools
         insta::assert_debug_snapshot!(get(
@@ -763,11 +818,11 @@ mod tests {
         insta::assert_debug_snapshot!(get("").unwrap(), @"Builtin");
 
         // Just program name
-        insta::assert_debug_snapshot!(get(r#"ui.merge-editor = "my-merge""#).unwrap_err(), @r###"
+        insta::assert_debug_snapshot!(get(r#"ui.merge-editor = "my-merge""#).unwrap_err(), @r#"
         MergeArgsNotConfigured {
             tool_name: "my-merge",
         }
-        "###);
+        "#);
 
         // String args
         insta::assert_debug_snapshot!(
@@ -871,11 +926,11 @@ mod tests {
 
         // List args should never be a merge-tools key
         insta::assert_debug_snapshot!(
-            get(r#"ui.merge-editor = ["meld"]"#).unwrap_err(), @r###"
+            get(r#"ui.merge-editor = ["meld"]"#).unwrap_err(), @r#"
         MergeArgsNotConfigured {
             tool_name: "meld",
         }
-        "###);
+        "#);
 
         // Invalid type
         assert!(get(r#"ui.merge-editor.k = 0"#).is_err());
