@@ -747,7 +747,7 @@ impl<'a> TemplateFormatter<'a> {
     ///
     /// This does not borrow `self` so the underlying formatter can be mutably
     /// borrowed.
-    pub fn rewrap_fn(&self) -> impl Fn(&mut dyn Formatter) -> TemplateFormatter<'_> {
+    pub fn rewrap_fn(&self) -> impl Fn(&mut dyn Formatter) -> TemplateFormatter<'_> + use<> {
         let error_handler = self.error_handler;
         move |formatter| TemplateFormatter::new(formatter, error_handler)
     }
@@ -879,7 +879,10 @@ fn propagate_property_error(
 /// This inherits the error handling strategy from the given `formatter`.
 fn record_non_empty_fn<T: Template + ?Sized>(
     formatter: &TemplateFormatter,
-) -> impl Fn(&T) -> Option<io::Result<FormatRecorder>> {
+    // TODO: T doesn't have to be captured, but "currently, all type parameters
+    // are required to be mentioned in the precise captures list" as of rustc
+    // 1.85.0.
+) -> impl Fn(&T) -> Option<io::Result<FormatRecorder>> + use<T> {
     let rewrap = formatter.rewrap_fn();
     move |template| {
         let mut recorder = FormatRecorder::new();
