@@ -15,21 +15,19 @@
 use std::cell::RefCell;
 use std::collections::HashMap;
 use std::ffi::OsStr;
-use std::fmt;
-use std::fmt::Debug;
-use std::fmt::Display;
 use std::path::Path;
 use std::path::PathBuf;
-use std::process::ExitStatus;
 
+use bstr::BString;
 use indoc::formatdoc;
 use regex::Captures;
 use regex::Regex;
 use tempfile::TempDir;
 
+use super::command_output::CommandOutput;
+use super::command_output::CommandOutputString;
 use super::fake_diff_editor_path;
 use super::fake_editor_path;
-use super::strip_last_line;
 use super::to_toml_value;
 
 pub struct TestEnvironment {
@@ -44,7 +42,7 @@ pub struct TestEnvironment {
 
 impl Default for TestEnvironment {
     fn default() -> Self {
-        testutils::hermetic_libgit2();
+        testutils::hermetic_git();
 
         let tmp_dir = testutils::new_temp_dir();
         let env_root = dunce::canonicalize(tmp_dir.path()).unwrap();
@@ -76,6 +74,16 @@ impl Default for TestEnvironment {
 }
 
 impl TestEnvironment {
+    /// Returns test helper for the specified directory.
+    ///
+    /// The `root` path usually points to the workspace root, but it may be
+    /// arbitrary path including non-existent directory.
+    #[must_use]
+    pub fn work_dir(&self, root: impl AsRef<Path>) -> TestWorkDir<'_> {
+        let root = self.env_root.join(root);
+        TestWorkDir { env: self, root }
+    }
+
     /// Runs `jj args..` in the `current_dir`, returns the output.
     #[must_use = "either snapshot the output or assert the exit status with .success()"]
     pub fn run_jj_in<I>(&self, current_dir: impl AsRef<Path>, args: I) -> CommandOutput
@@ -83,8 +91,7 @@ impl TestEnvironment {
         I: IntoIterator,
         I::Item: AsRef<OsStr>,
     {
-        let current_dir = self.env_root.join(current_dir);
-        self.run_jj_with(|cmd| cmd.current_dir(current_dir).args(args))
+        self.work_dir(current_dir).run_jj(args)
     }
 
     /// Runs `jj` command with additional configuration, returns the output.
@@ -93,13 +100,7 @@ impl TestEnvironment {
         &self,
         configure: impl FnOnce(&mut assert_cmd::Command) -> &mut assert_cmd::Command,
     ) -> CommandOutput {
-        let mut cmd = self.new_jj_cmd();
-        let output = configure(&mut cmd).output().unwrap();
-        CommandOutput {
-            stdout: self.normalize_output(String::from_utf8(output.stdout).unwrap()),
-            stderr: self.normalize_output(String::from_utf8(output.stderr).unwrap()),
-            status: output.status,
-        }
+        self.work_dir("").run_jj_with(configure)
     }
 
     /// Returns command builder to run `jj` in the test environment.
@@ -119,6 +120,9 @@ impl TestEnvironment {
         // executables like `git` from the PATH.
         cmd.env("PATH", std::env::var_os("PATH").unwrap_or_default());
         cmd.env("HOME", self.home_dir.to_str().unwrap());
+        // Prevent git.subprocess from reading outside git config
+        cmd.env("GIT_CONFIG_SYSTEM", "/dev/null");
+        cmd.env("GIT_CONFIG_GLOBAL", "/dev/null");
         cmd.env("JJ_CONFIG", self.config_path.to_str().unwrap());
         cmd.env("JJ_USER", "Test User");
         cmd.env("JJ_EMAIL", "test.user@example.com");
@@ -134,14 +138,6 @@ impl TestEnvironment {
         cmd.env("JJ_TIMESTAMP", timestamp.to_rfc3339());
         cmd.env("JJ_OP_TIMESTAMP", timestamp.to_rfc3339());
 
-        // libgit2 always initializes OpenSSL, and it takes a few tens of milliseconds
-        // to load the system CA certificates in X509_load_cert_crl_file_ex(). As we
-        // don't use HTTPS in our tests, we can disable the cert loading to speed up the
-        // CLI tests. If we migrated to gitoxide, maybe we can remove this hack.
-        if cfg!(unix) {
-            cmd.env("SSL_CERT_FILE", "/dev/null");
-        }
-
         if cfg!(windows) {
             // Windows uses `TEMP` to create temporary directories, which we need for some
             // tests.
@@ -165,6 +161,12 @@ impl TestEnvironment {
         &self.config_path
     }
 
+    pub fn first_config_file_path(&self) -> PathBuf {
+        let config_file_number = 1;
+        self.config_path
+            .join(format!("config{config_file_number:04}.toml"))
+    }
+
     pub fn last_config_file_path(&self) -> PathBuf {
         let config_file_number = self.config_file_number.borrow();
         self.config_path
@@ -196,13 +198,6 @@ impl TestEnvironment {
         self.env_vars.insert(key.into(), val.into());
     }
 
-    pub fn current_operation_id(&self, repo_path: &Path) -> String {
-        let output = self
-            .run_jj_in(repo_path, ["debug", "operation", "--display=id"])
-            .success();
-        output.stdout.raw().trim_end().to_owned()
-    }
-
     /// Sets up the fake editor to read an edit script from the returned path
     /// Also sets up the fake editor as a merge tool named "fake-editor"
     pub fn set_up_fake_editor(&mut self) -> PathBuf {
@@ -255,184 +250,96 @@ impl TestEnvironment {
     }
 }
 
-/// Command output and exit status to be displayed in normalized form.
-#[derive(Clone, Debug, Eq, PartialEq)]
-pub struct CommandOutput {
-    pub stdout: CommandOutputString,
-    pub stderr: CommandOutputString,
-    pub status: ExitStatus,
+/// Helper to execute `jj` or file operation in sub directory.
+pub struct TestWorkDir<'a> {
+    env: &'a TestEnvironment,
+    root: PathBuf,
 }
 
-impl CommandOutput {
-    /// Normalizes Windows directory separator to slash.
-    #[must_use]
-    pub fn normalize_backslash(self) -> Self {
+impl TestWorkDir<'_> {
+    /// Path to the working directory.
+    pub fn root(&self) -> &Path {
+        &self.root
+    }
+
+    /// Runs `jj args..` in the working directory, returns the output.
+    #[must_use = "either snapshot the output or assert the exit status with .success()"]
+    pub fn run_jj<I>(&self, args: I) -> CommandOutput
+    where
+        I: IntoIterator,
+        I::Item: AsRef<OsStr>,
+    {
+        self.run_jj_with(|cmd| cmd.args(args))
+    }
+
+    /// Runs `jj` command with additional configuration, returns the output.
+    #[must_use = "either snapshot the output or assert the exit status with .success()"]
+    pub fn run_jj_with(
+        &self,
+        configure: impl FnOnce(&mut assert_cmd::Command) -> &mut assert_cmd::Command,
+    ) -> CommandOutput {
+        let env = &self.env;
+        let mut cmd = env.new_jj_cmd();
+        let output = configure(cmd.current_dir(&self.root)).output().unwrap();
         CommandOutput {
-            stdout: self.stdout.normalize_backslash(),
-            stderr: self.stderr.normalize_backslash(),
-            status: self.status,
+            stdout: env.normalize_output(String::from_utf8(output.stdout).unwrap()),
+            stderr: env.normalize_output(String::from_utf8(output.stderr).unwrap()),
+            status: output.status,
         }
     }
 
-    /// Normalizes [`ExitStatus`] message in stderr text.
-    #[must_use]
-    pub fn normalize_stderr_exit_status(self) -> Self {
-        CommandOutput {
-            stdout: self.stdout,
-            stderr: self.stderr.normalize_exit_status(),
-            status: self.status,
-        }
-    }
-
-    /// Removes the last line (such as platform-specific error message) from the
-    /// normalized stderr text.
-    #[must_use]
-    pub fn strip_stderr_last_line(self) -> Self {
-        CommandOutput {
-            stdout: self.stdout,
-            stderr: self.stderr.strip_last_line(),
-            status: self.status,
-        }
-    }
-
-    #[must_use]
-    pub fn normalize_stdout_with(self, f: impl FnOnce(String) -> String) -> Self {
-        CommandOutput {
-            stdout: self.stdout.normalize_with(f),
-            stderr: self.stderr,
-            status: self.status,
-        }
-    }
-
-    #[must_use]
-    pub fn normalize_stderr_with(self, f: impl FnOnce(String) -> String) -> Self {
-        CommandOutput {
-            stdout: self.stdout,
-            stderr: self.stderr.normalize_with(f),
-            status: self.status,
-        }
-    }
-
-    /// Ensures that the command exits with success status.
     #[track_caller]
-    pub fn success(self) -> Self {
-        assert!(self.status.success(), "{self}");
-        self
+    pub fn current_operation_id(&self) -> String {
+        let output = self
+            .run_jj(["debug", "operation", "--display=id"])
+            .success();
+        output.stdout.raw().trim_end().to_owned()
     }
-}
 
-impl Display for CommandOutput {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let CommandOutput {
-            stdout,
-            stderr,
-            status,
-        } = self;
-        write!(f, "{stdout}")?;
-        if !stderr.is_empty() {
-            writeln!(f, "------- stderr -------")?;
-            write!(f, "{stderr}")?;
+    /// Returns test helper for the specified sub directory.
+    #[must_use]
+    pub fn dir(&self, path: impl AsRef<Path>) -> Self {
+        let env = self.env;
+        let root = self.root.join(path);
+        TestWorkDir { env, root }
+    }
+
+    #[track_caller]
+    pub fn create_dir(&self, path: impl AsRef<Path>) -> Self {
+        let dir = self.dir(path);
+        std::fs::create_dir(&dir.root).unwrap();
+        dir
+    }
+
+    #[track_caller]
+    pub fn create_dir_all(&self, path: impl AsRef<Path>) -> Self {
+        let dir = self.dir(path);
+        std::fs::create_dir_all(&dir.root).unwrap();
+        dir
+    }
+
+    #[track_caller]
+    pub fn remove_dir_all(&self, path: impl AsRef<Path>) {
+        std::fs::remove_dir_all(self.root.join(path)).unwrap();
+    }
+
+    #[track_caller]
+    pub fn remove_file(&self, path: impl AsRef<Path>) {
+        std::fs::remove_file(self.root.join(path)).unwrap();
+    }
+
+    #[track_caller]
+    pub fn read_file(&self, path: impl AsRef<Path>) -> BString {
+        std::fs::read(self.root.join(path)).unwrap().into()
+    }
+
+    #[track_caller]
+    pub fn write_file(&self, path: impl AsRef<Path>, contents: impl AsRef<[u8]>) {
+        let path = path.as_ref();
+        if let Some(dir) = path.parent() {
+            self.create_dir_all(dir);
         }
-        if !status.success() {
-            // If there is an exit code, `{status}` would get rendered as "exit
-            // code: N" on Windows, so we render it ourselves for compatibility.
-            if let Some(code) = status.code() {
-                writeln!(f, "[exit status: {code}]")?;
-            } else {
-                writeln!(f, "[{status}]")?;
-            }
-        }
-        Ok(())
-    }
-}
-
-/// Command output data to be displayed in normalized form.
-#[derive(Clone)]
-pub struct CommandOutputString {
-    // TODO: use BString?
-    raw: String,
-    normalized: String,
-}
-
-impl CommandOutputString {
-    /// Normalizes Windows directory separator to slash.
-    #[must_use]
-    pub fn normalize_backslash(self) -> Self {
-        self.normalize_with(|s| s.replace('\\', "/"))
-    }
-
-    /// Normalizes [`ExitStatus`] message.
-    ///
-    /// On Windows, it prints "exit code" instead of "exit status".
-    #[must_use]
-    pub fn normalize_exit_status(self) -> Self {
-        self.normalize_with(|s| s.replace("exit code:", "exit status:"))
-    }
-
-    /// Removes the last line (such as platform-specific error message) from the
-    /// normalized text.
-    #[must_use]
-    pub fn strip_last_line(self) -> Self {
-        self.normalize_with(|mut s| {
-            s.truncate(strip_last_line(&s).len());
-            s
-        })
-    }
-
-    #[must_use]
-    pub fn normalize_with(mut self, f: impl FnOnce(String) -> String) -> Self {
-        self.normalized = f(self.normalized);
-        self
-    }
-
-    #[must_use]
-    pub fn is_empty(&self) -> bool {
-        self.raw.is_empty()
-    }
-
-    /// Raw output data.
-    #[must_use]
-    pub fn raw(&self) -> &str {
-        &self.raw
-    }
-
-    /// Normalized text for snapshot testing.
-    #[must_use]
-    pub fn normalized(&self) -> &str {
-        &self.normalized
-    }
-
-    /// Extracts raw output data.
-    #[must_use]
-    pub fn into_raw(self) -> String {
-        self.raw
-    }
-}
-
-impl Debug for CommandOutputString {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        // Print only raw data. Normalized string should be nearly identical.
-        Debug::fmt(&self.raw, f)
-    }
-}
-
-impl Display for CommandOutputString {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        if self.is_empty() {
-            return Ok(());
-        }
-        // Append "[EOF]" marker to test line ending
-        // https://github.com/mitsuhiko/insta/issues/384
-        writeln!(f, "{}[EOF]", self.normalized)
-    }
-}
-
-impl Eq for CommandOutputString {}
-
-impl PartialEq for CommandOutputString {
-    fn eq(&self, other: &Self) -> bool {
-        // Compare only raw data. Normalized string is for displaying purpose.
-        self.raw == other.raw
+        std::fs::write(self.root.join(path), contents).unwrap();
     }
 }
 
