@@ -12,38 +12,32 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::PathBuf;
-
 use crate::common::TestEnvironment;
 
-fn set_up(trunk_name: &str) -> (TestEnvironment, PathBuf) {
+fn set_up(trunk_name: &str) -> TestEnvironment {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "origin"]).success();
-    let origin_path = test_env.env_root().join("origin");
-    let origin_git_repo_path = origin_path
+    let origin_dir = test_env.work_dir("origin");
+    let origin_git_repo_path = origin_dir
+        .root()
         .join(".jj")
         .join("repo")
         .join("store")
         .join("git");
 
-    test_env
-        .run_jj_in(&origin_path, ["describe", "-m=description 1"])
+    origin_dir
+        .run_jj(["describe", "-m=description 1"])
         .success();
-    test_env
-        .run_jj_in(&origin_path, ["bookmark", "create", "-r@", trunk_name])
+    origin_dir
+        .run_jj(["bookmark", "create", "-r@", trunk_name])
         .success();
-    test_env
-        .run_jj_in(&origin_path, ["new", "root()", "-m=description 2"])
+    origin_dir
+        .run_jj(["new", "root()", "-m=description 2"])
         .success();
-    test_env
-        .run_jj_in(
-            &origin_path,
-            ["bookmark", "create", "-r@", "unrelated_bookmark"],
-        )
-        .success();
-    test_env
-        .run_jj_in(&origin_path, ["git", "export"])
+    origin_dir
+        .run_jj(["bookmark", "create", "-r@", "unrelated_bookmark"])
         .success();
+    origin_dir.run_jj(["git", "export"]).success();
 
     test_env
         .run_jj_in(
@@ -57,15 +51,15 @@ fn set_up(trunk_name: &str) -> (TestEnvironment, PathBuf) {
             ],
         )
         .success();
-    let workspace_root = test_env.env_root().join("local");
-    (test_env, workspace_root)
+    test_env
 }
 
 #[test]
 fn test_builtin_alias_trunk_matches_main() {
-    let (test_env, workspace_root) = set_up("main");
+    let test_env = set_up("main");
+    let work_dir = test_env.work_dir("local");
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 main d13ecdbd
     │  (empty) description 1
@@ -76,9 +70,10 @@ fn test_builtin_alias_trunk_matches_main() {
 
 #[test]
 fn test_builtin_alias_trunk_matches_master() {
-    let (test_env, workspace_root) = set_up("master");
+    let test_env = set_up("master");
+    let work_dir = test_env.work_dir("local");
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 master d13ecdbd
     │  (empty) description 1
@@ -89,9 +84,10 @@ fn test_builtin_alias_trunk_matches_master() {
 
 #[test]
 fn test_builtin_alias_trunk_matches_trunk() {
-    let (test_env, workspace_root) = set_up("trunk");
+    let test_env = set_up("trunk");
+    let work_dir = test_env.work_dir("local");
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 trunk d13ecdbd
     │  (empty) description 1
@@ -102,16 +98,17 @@ fn test_builtin_alias_trunk_matches_trunk() {
 
 #[test]
 fn test_builtin_alias_trunk_matches_exactly_one_commit() {
-    let (test_env, workspace_root) = set_up("main");
-    let origin_path = test_env.env_root().join("origin");
-    test_env
-        .run_jj_in(&origin_path, ["new", "root()", "-m=description 3"])
+    let test_env = set_up("main");
+    let work_dir = test_env.work_dir("local");
+    let origin_dir = test_env.work_dir("origin");
+    origin_dir
+        .run_jj(["new", "root()", "-m=description 3"])
         .success();
-    test_env
-        .run_jj_in(&origin_path, ["bookmark", "create", "-r@", "master"])
+    origin_dir
+        .run_jj(["bookmark", "create", "-r@", "master"])
         .success();
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 main d13ecdbd
     │  (empty) description 1
@@ -122,13 +119,14 @@ fn test_builtin_alias_trunk_matches_exactly_one_commit() {
 
 #[test]
 fn test_builtin_alias_trunk_override_alias() {
-    let (test_env, workspace_root) = set_up("override-trunk");
+    let test_env = set_up("override-trunk");
+    let work_dir = test_env.work_dir("local");
 
     test_env.add_config(
         r#"revset-aliases.'trunk()' = 'latest(remote_bookmarks(exact:"override-trunk", exact:"origin"))'"#,
     );
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 override-trunk d13ecdbd
     │  (empty) description 1
@@ -139,9 +137,10 @@ fn test_builtin_alias_trunk_override_alias() {
 
 #[test]
 fn test_builtin_alias_trunk_no_match() {
-    let (test_env, workspace_root) = set_up("no-match-trunk");
+    let test_env = set_up("no-match-trunk");
+    let work_dir = test_env.work_dir("local");
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  zzzzzzzz root() 00000000
     [EOF]
@@ -150,9 +149,10 @@ fn test_builtin_alias_trunk_no_match() {
 
 #[test]
 fn test_builtin_alias_trunk_no_match_only_exact() {
-    let (test_env, workspace_root) = set_up("maint");
+    let test_env = set_up("maint");
+    let work_dir = test_env.work_dir("local");
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ◆  zzzzzzzz root() 00000000
     [EOF]
@@ -161,13 +161,14 @@ fn test_builtin_alias_trunk_no_match_only_exact() {
 
 #[test]
 fn test_builtin_user_redefines_builtin_immutable_heads() {
-    let (test_env, workspace_root) = set_up("main");
+    let test_env = set_up("main");
+    let work_dir = test_env.work_dir("local");
 
     test_env.add_config(r#"revset-aliases.'builtin_immutable_heads()' = '@'"#);
     test_env.add_config(r#"revset-aliases.'mutable()' = '@'"#);
     test_env.add_config(r#"revset-aliases.'immutable()' = '@'"#);
 
-    let output = test_env.run_jj_in(&workspace_root, ["log", "-r", "trunk()"]);
+    let output = work_dir.run_jj(["log", "-r", "trunk()"]);
     insta::assert_snapshot!(output, @r"
     ○  xtvrqkyv test.user@example.com 2001-02-03 08:05:08 main d13ecdbd
     │  (empty) description 1
