@@ -18,21 +18,17 @@ use crate::common::TestEnvironment;
 fn test_rewrite_immutable_generic() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    std::fs::write(repo_path.join("file"), "a").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m=a"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.write_file("file", "a");
+    work_dir.run_jj(["describe", "-m=a"]).success();
+    work_dir.run_jj(["new", "-m=b"]).success();
+    work_dir.write_file("file", "b");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
-    test_env.run_jj_in(&repo_path, ["new", "-m=b"]).success();
-    std::fs::write(repo_path.join("file"), "b").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "main"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "main-", "-m=c"])
-        .success();
-    std::fs::write(repo_path.join("file"), "c").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["log"]);
+    work_dir.run_jj(["new", "main-", "-m=c"]).success();
+    work_dir.write_file("file", "c");
+    let output = work_dir.run_jj(["log"]);
     insta::assert_snapshot!(output, @r"
     @  mzvwutvl test.user@example.com 2001-02-03 08:05:12 7adb43e8
     │  c
@@ -46,8 +42,8 @@ fn test_rewrite_immutable_generic() {
 
     // Cannot rewrite a commit in the configured set
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
-    let output = test_env.run_jj_in(&repo_path, ["edit", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["edit", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit 72e1b68cbcf2 is immutable
     Hint: Could not modify commit: kkmpptxz 72e1b68c main | b
@@ -58,10 +54,10 @@ fn test_rewrite_immutable_generic() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // Cannot rewrite an ancestor of the configured set
-    let output = test_env.run_jj_in(&repo_path, ["edit", "main-"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["edit", "main-"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit b84b821b8a2b is immutable
     Hint: Could not modify commit: qpvuntsm b84b821b a
@@ -72,10 +68,10 @@ fn test_rewrite_immutable_generic() {
     Hint: This operation would rewrite 2 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // Cannot rewrite the root commit even with an empty set of immutable commits
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
-    let output = test_env.run_jj_in(&repo_path, ["edit", "root()"]);
+    let output = work_dir.run_jj(["edit", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The root commit 000000000000 is immutable
@@ -88,7 +84,7 @@ fn test_rewrite_immutable_generic() {
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "bookmark_that_does_not_exist""#);
     // Suppress warning in the commit summary template
     test_env.add_config("template-aliases.'format_short_id(id)' = 'id.short(8)'");
-    let output = test_env.run_jj_in(&repo_path, ["new", "main"]);
+    let output = work_dir.run_jj(["new", "main"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Config error: Invalid `revset-aliases.immutable_heads()`
@@ -100,16 +96,16 @@ fn test_rewrite_immutable_generic() {
 
     // Can use --ignore-immutable to override
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
-    let output = test_env.run_jj_in(&repo_path, ["--ignore-immutable", "edit", "main"]);
+    let output = work_dir.run_jj(["--ignore-immutable", "edit", "main"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy now at: kkmpptxz 72e1b68c main | b
-    Parent commit      : qpvuntsm b84b821b a
+    Working copy  (@) now at: kkmpptxz 72e1b68c main | b
+    Parent commit (@-)      : qpvuntsm b84b821b a
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
     // ... but not the root commit
-    let output = test_env.run_jj_in(&repo_path, ["--ignore-immutable", "edit", "root()"]);
+    let output = work_dir.run_jj(["--ignore-immutable", "edit", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The root commit 000000000000 is immutable
@@ -121,17 +117,17 @@ fn test_rewrite_immutable_generic() {
     test_env.add_config(
         r#"revset-aliases."immutable_heads()" = "present(bookmark_that_does_not_exist)""#,
     );
-    let output = test_env.run_jj_in(&repo_path, ["new", "main"]);
+    let output = work_dir.run_jj(["new", "main"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy now at: wqnwkozp fc921593 (empty) (no description set)
-    Parent commit      : kkmpptxz 72e1b68c main | b
+    Working copy  (@) now at: wqnwkozp fc921593 (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz 72e1b68c main | b
     [EOF]
     ");
 
     // immutable_heads() of different arity doesn't shadow the 0-ary one
     test_env.add_config(r#"revset-aliases."immutable_heads(foo)" = "none()""#);
-    let output = test_env.run_jj_in(&repo_path, ["edit", "root()"]);
+    let output = work_dir.run_jj(["edit", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The root commit 000000000000 is immutable
@@ -144,18 +140,19 @@ fn test_rewrite_immutable_generic() {
 fn test_new_wc_commit_when_wc_immutable() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init"]).success();
-    test_env
-        .run_jj_in(".", ["bookmark", "create", "-r@", "main"])
+    let work_dir = test_env.work_dir("");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
-    test_env.run_jj_in(".", ["new", "-m=a"]).success();
-    let output = test_env.run_jj_in(".", ["bookmark", "set", "main", "-r@"]);
+    work_dir.run_jj(["new", "-m=a"]).success();
+    let output = work_dir.run_jj(["bookmark", "set", "main", "-r@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Moved 1 bookmarks to kkmpptxz a164195b main | (empty) a
     Warning: The working-copy commit in workspace 'default' became immutable, so a new commit has been created on top of it.
-    Working copy now at: zsuskuln ef5fa85b (empty) (no description set)
-    Parent commit      : kkmpptxz a164195b main | (empty) a
+    Working copy  (@) now at: zsuskuln ef5fa85b (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz a164195b main | (empty) a
     [EOF]
     ");
 }
@@ -164,16 +161,17 @@ fn test_new_wc_commit_when_wc_immutable() {
 fn test_immutable_heads_set_to_working_copy() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init"]).success();
-    test_env
-        .run_jj_in(".", ["bookmark", "create", "-r@", "main"])
+    let work_dir = test_env.work_dir("");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "@""#);
-    let output = test_env.run_jj_in(".", ["new", "-m=a"]);
+    let output = work_dir.run_jj(["new", "-m=a"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: The working-copy commit in workspace 'default' became immutable, so a new commit has been created on top of it.
-    Working copy now at: pmmvwywv 7278b2d8 (empty) (no description set)
-    Parent commit      : kkmpptxz a713ef56 (empty) a
+    Working copy  (@) now at: pmmvwywv 7278b2d8 (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz a713ef56 (empty) a
     [EOF]
     ");
 }
@@ -182,33 +180,31 @@ fn test_immutable_heads_set_to_working_copy() {
 fn test_new_wc_commit_when_wc_immutable_multi_workspace() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "main"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
-    test_env.run_jj_in(&repo_path, ["new", "-m=a"]).success();
-    test_env
-        .run_jj_in(&repo_path, ["workspace", "add", "../workspace1"])
+    work_dir.run_jj(["new", "-m=a"]).success();
+    work_dir
+        .run_jj(["workspace", "add", "../workspace1"])
         .success();
-    let workspace1_envroot = test_env.env_root().join("workspace1");
-    test_env
-        .run_jj_in(&workspace1_envroot, ["edit", "default@"])
-        .success();
-    let output = test_env.run_jj_in(&repo_path, ["bookmark", "set", "main", "-r@"]);
+    let workspace1_dir = test_env.work_dir("workspace1");
+    workspace1_dir.run_jj(["edit", "default@"]).success();
+    let output = work_dir.run_jj(["bookmark", "set", "main", "-r@"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Moved 1 bookmarks to kkmpptxz 7796c4df main | (empty) a
     Warning: The working-copy commit in workspace 'default' became immutable, so a new commit has been created on top of it.
     Warning: The working-copy commit in workspace 'workspace1' became immutable, so a new commit has been created on top of it.
-    Working copy now at: royxmykx 896465c4 (empty) (no description set)
-    Parent commit      : kkmpptxz 7796c4df main | (empty) a
+    Working copy  (@) now at: royxmykx 896465c4 (empty) (no description set)
+    Parent commit (@-)      : kkmpptxz 7796c4df main | (empty) a
     [EOF]
     ");
-    test_env
-        .run_jj_in(&workspace1_envroot, ["workspace", "update-stale"])
+    workspace1_dir
+        .run_jj(["workspace", "update-stale"])
         .success();
-    let output = test_env.run_jj_in(&workspace1_envroot, ["log", "--no-graph"]);
+    let output = workspace1_dir.run_jj(["log", "--no-graph"]);
     insta::assert_snapshot!(output, @r"
     nppvrztz test.user@example.com 2001-02-03 08:05:11 workspace1@ ee0671fd
     (empty) (no description set)
@@ -225,35 +221,29 @@ fn test_new_wc_commit_when_wc_immutable_multi_workspace() {
 fn test_rewrite_immutable_commands() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    std::fs::write(repo_path.join("file"), "a").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m=a"])
-        .success();
-    test_env.run_jj_in(&repo_path, ["new", "-m=b"]).success();
-    std::fs::write(repo_path.join("file"), "b").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["new", "@-", "-m=c"])
-        .success();
-    std::fs::write(repo_path.join("file"), "c").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["new", "all:visible_heads()", "-m=merge"])
+    let work_dir = test_env.work_dir("repo");
+    work_dir.write_file("file", "a");
+    work_dir.run_jj(["describe", "-m=a"]).success();
+    work_dir.run_jj(["new", "-m=b"]).success();
+    work_dir.write_file("file", "b");
+    work_dir.run_jj(["new", "@-", "-m=c"]).success();
+    work_dir.write_file("file", "c");
+    work_dir
+        .run_jj(["new", "all:visible_heads()", "-m=merge"])
         .success();
     // Create another file to make sure the merge commit isn't empty (to satisfy `jj
     // split`) and still has a conflict (to satisfy `jj resolve`).
-    std::fs::write(repo_path.join("file2"), "merged").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "main"])
+    work_dir.write_file("file2", "merged");
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "description(b)"])
-        .success();
-    std::fs::write(repo_path.join("file"), "w").unwrap();
+    work_dir.run_jj(["new", "description(b)"]).success();
+    work_dir.write_file("file", "w");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
     test_env.add_config(r#"revset-aliases."trunk()" = "main""#);
 
     // Log shows mutable commits, their parents, and trunk() by default
-    let output = test_env.run_jj_in(&repo_path, ["log"]);
+    let output = work_dir.run_jj(["log"]);
     insta::assert_snapshot!(output, @r"
     @  yqosqzyt test.user@example.com 2001-02-03 08:05:14 55641cc5
     │  (no description set)
@@ -269,8 +259,8 @@ fn test_rewrite_immutable_commands() {
     ");
 
     // abandon
-    let output = test_env.run_jj_in(&repo_path, ["abandon", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["abandon", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -281,10 +271,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // absorb
-    let output = test_env.run_jj_in(&repo_path, ["absorb", "--into=::@-"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["absorb", "--into=::@-"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit 72e1b68cbcf2 is immutable
     Hint: Could not modify commit: kkmpptxz 72e1b68c b
@@ -295,10 +285,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 2 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // chmod
-    let output = test_env.run_jj_in(&repo_path, ["file", "chmod", "-r=main", "x", "file"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["file", "chmod", "-r=main", "x", "file"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -309,10 +299,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // describe
-    let output = test_env.run_jj_in(&repo_path, ["describe", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["describe", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -323,10 +313,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // diffedit
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "-r=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["diffedit", "-r=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -337,10 +327,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // edit
-    let output = test_env.run_jj_in(&repo_path, ["edit", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["edit", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -351,10 +341,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // new --insert-before
-    let output = test_env.run_jj_in(&repo_path, ["new", "--insert-before", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["new", "--insert-before", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -365,10 +355,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // new --insert-after parent_of_main
-    let output = test_env.run_jj_in(&repo_path, ["new", "--insert-after", "description(b)"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["new", "--insert-after", "description(b)"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -379,10 +369,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // parallelize
-    let output = test_env.run_jj_in(&repo_path, ["parallelize", "description(b)", "main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["parallelize", "description(b)", "main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -393,10 +383,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 2 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // rebase -s
-    let output = test_env.run_jj_in(&repo_path, ["rebase", "-s=main", "-d=@"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["rebase", "-s=main", "-d=@"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -407,10 +397,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // rebase -b
-    let output = test_env.run_jj_in(&repo_path, ["rebase", "-b=main", "-d=@"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["rebase", "-b=main", "-d=@"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit 77cee210cbf5 is immutable
     Hint: Could not modify commit: zsuskuln 77cee210 c
@@ -421,10 +411,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 2 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // rebase -r
-    let output = test_env.run_jj_in(&repo_path, ["rebase", "-r=main", "-d=@"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["rebase", "-r=main", "-d=@"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -435,10 +425,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // resolve
-    let output = test_env.run_jj_in(&repo_path, ["resolve", "-r=description(merge)", "file"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["resolve", "-r=description(merge)", "file"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -449,10 +439,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // restore -c
-    let output = test_env.run_jj_in(&repo_path, ["restore", "-c=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["restore", "-c=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -463,10 +453,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // restore --into
-    let output = test_env.run_jj_in(&repo_path, ["restore", "--into=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["restore", "--into=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -477,10 +467,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // split
-    let output = test_env.run_jj_in(&repo_path, ["split", "-r=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["split", "-r=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -491,10 +481,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // squash -r
-    let output = test_env.run_jj_in(&repo_path, ["squash", "-r=description(b)"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["squash", "-r=description(b)"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit 72e1b68cbcf2 is immutable
     Hint: Could not modify commit: kkmpptxz 72e1b68c b
@@ -505,10 +495,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 4 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // squash --from
-    let output = test_env.run_jj_in(&repo_path, ["squash", "--from=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["squash", "--from=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -519,10 +509,10 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
+    "#);
     // squash --into
-    let output = test_env.run_jj_in(&repo_path, ["squash", "--into=main"]);
-    insta::assert_snapshot!(output, @r##"
+    let output = work_dir.run_jj(["squash", "--into=main"]);
+    insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
     Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
@@ -533,28 +523,9 @@ fn test_rewrite_immutable_commands() {
     Hint: This operation would rewrite 1 immutable commits.
     [EOF]
     [exit status: 1]
-    "##);
-    // unsquash
-    let output = test_env.run_jj_in(&repo_path, ["unsquash", "-r=main"]);
-    insta::assert_snapshot!(output, @r##"
-    ------- stderr -------
-    Warning: `jj unsquash` is deprecated; use `jj diffedit --restore-descendants` or `jj squash` instead
-    Warning: `jj unsquash` will be removed in a future version, and this will be a hard error
-    Error: Commit bcab555fc80e is immutable
-    Hint: Could not modify commit: mzvwutvl bcab555f main | (conflict) merge
-    Hint: Immutable commits are used to protect shared history.
-    Hint: For more information, see:
-          - https://jj-vcs.github.io/jj/latest/config/#set-of-immutable-commits
-          - `jj help -k config`, "Set of immutable commits"
-    Hint: This operation would rewrite 1 immutable commits.
-    [EOF]
-    [exit status: 1]
-    "##);
+    "#);
     // sign
-    let output = test_env.run_jj_in(
-        &repo_path,
-        &["sign", "-r=main", "--config=signing.backend=test"],
-    );
+    let output = work_dir.run_jj(["sign", "-r=main", "--config=signing.backend=test"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
@@ -568,7 +539,7 @@ fn test_rewrite_immutable_commands() {
     [exit status: 1]
     "#);
     // unsign
-    let output = test_env.run_jj_in(&repo_path, &["unsign", "-r=main"]);
+    let output = work_dir.run_jj(["unsign", "-r=main"]);
     insta::assert_snapshot!(output, @r#"
     ------- stderr -------
     Error: Commit bcab555fc80e is immutable
