@@ -12,11 +12,11 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
 use std::path::PathBuf;
 
 use test_case::test_case;
 
+use crate::common::create_commit;
 use crate::common::TestEnvironment;
 
 fn create_repo() -> (TestEnvironment, PathBuf) {
@@ -27,20 +27,6 @@ fn create_repo() -> (TestEnvironment, PathBuf) {
     (test_env, repo_path)
 }
 
-fn create_commit(test_env: &TestEnvironment, repo_path: &Path, name: &str, parents: &[&str]) {
-    test_env
-        .run_jj_with(|cmd| {
-            cmd.current_dir(repo_path)
-                .args(["new", "-m", name])
-                .args(parents)
-        })
-        .success();
-    std::fs::write(repo_path.join(name), format!("{name}\n")).unwrap();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", name])
-        .success();
-}
-
 #[test]
 fn test_simplify_parents_no_commits() {
     let (test_env, repo_path) = create_repo();
@@ -70,8 +56,8 @@ fn test_simplify_parents_immutable() {
 fn test_simplify_parents_no_change() {
     let (test_env, repo_path) = create_repo();
 
-    create_commit(&test_env, &repo_path, "a", &["root()"]);
-    create_commit(&test_env, &repo_path, "b", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "a", &["root()"]);
+    create_commit(&test_env.work_dir(&repo_path), "b", &["a"]);
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "all()", "-T", "description"]);
     insta::assert_snapshot!(output, @r"
     @  b
@@ -100,10 +86,10 @@ fn test_simplify_parents_no_change() {
 fn test_simplify_parents_no_change_diamond() {
     let (test_env, repo_path) = create_repo();
 
-    create_commit(&test_env, &repo_path, "a", &["root()"]);
-    create_commit(&test_env, &repo_path, "b", &["a"]);
-    create_commit(&test_env, &repo_path, "c", &["a"]);
-    create_commit(&test_env, &repo_path, "d", &["b", "c"]);
+    create_commit(&test_env.work_dir(&repo_path), "a", &["root()"]);
+    create_commit(&test_env.work_dir(&repo_path), "b", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "c", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "d", &["b", "c"]);
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "all()", "-T", "description"]);
     insta::assert_snapshot!(output, @r"
     @    d
@@ -141,9 +127,9 @@ fn test_simplify_parents_no_change_diamond() {
 fn test_simplify_parents_redundant_parent(args: &[&str]) {
     let (test_env, repo_path) = create_repo();
 
-    create_commit(&test_env, &repo_path, "a", &["root()"]);
-    create_commit(&test_env, &repo_path, "b", &["a"]);
-    create_commit(&test_env, &repo_path, "c", &["a", "b"]);
+    create_commit(&test_env.work_dir(&repo_path), "a", &["root()"]);
+    create_commit(&test_env.work_dir(&repo_path), "b", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "c", &["a", "b"]);
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "all()", "-T", "description"]);
     insta::allow_duplicates! {
         insta::assert_snapshot!(output, @r"
@@ -161,8 +147,8 @@ fn test_simplify_parents_redundant_parent(args: &[&str]) {
         insta::assert_snapshot!(output, @r"
         ------- stderr -------
         Removed 1 edges from 1 out of 3 commits.
-        Working copy now at: royxmykx 0ac2063b c | c
-        Parent commit      : zsuskuln 1394f625 b | b
+        Working copy  (@) now at: royxmykx 0ac2063b c | c
+        Parent commit (@-)      : zsuskuln 1394f625 b | b
         [EOF]
         ");
     }
@@ -183,12 +169,12 @@ fn test_simplify_parents_redundant_parent(args: &[&str]) {
 fn test_simplify_parents_multiple_redundant_parents() {
     let (test_env, repo_path) = create_repo();
 
-    create_commit(&test_env, &repo_path, "a", &["root()"]);
-    create_commit(&test_env, &repo_path, "b", &["a"]);
-    create_commit(&test_env, &repo_path, "c", &["a", "b"]);
-    create_commit(&test_env, &repo_path, "d", &["c"]);
-    create_commit(&test_env, &repo_path, "e", &["d"]);
-    create_commit(&test_env, &repo_path, "f", &["d", "e"]);
+    create_commit(&test_env.work_dir(&repo_path), "a", &["root()"]);
+    create_commit(&test_env.work_dir(&repo_path), "b", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "c", &["a", "b"]);
+    create_commit(&test_env.work_dir(&repo_path), "d", &["c"]);
+    create_commit(&test_env.work_dir(&repo_path), "e", &["d"]);
+    create_commit(&test_env.work_dir(&repo_path), "f", &["d", "e"]);
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "all()", "-T", "description"]);
     insta::assert_snapshot!(output, @r"
     @    f
@@ -204,7 +190,7 @@ fn test_simplify_parents_multiple_redundant_parents() {
     ◆
     [EOF]
     ");
-    let setup_opid = test_env.current_operation_id(&repo_path);
+    let setup_opid = test_env.work_dir(&repo_path).current_operation_id();
 
     // Test with `-r`.
     let output = test_env.run_jj_in(&repo_path, ["simplify-parents", "-r", "c", "-r", "f"]);
@@ -212,8 +198,8 @@ fn test_simplify_parents_multiple_redundant_parents() {
     ------- stderr -------
     Removed 2 edges from 2 out of 2 commits.
     Rebased 2 descendant commits
-    Working copy now at: kmkuslsw 8cc01e1b f | f
-    Parent commit      : znkkpsqq 040ae3a6 e | e
+    Working copy  (@) now at: kmkuslsw 8cc01e1b f | f
+    Parent commit (@-)      : znkkpsqq 040ae3a6 e | e
     [EOF]
     ");
 
@@ -238,8 +224,8 @@ fn test_simplify_parents_multiple_redundant_parents() {
     ------- stderr -------
     Removed 2 edges from 2 out of 4 commits.
     Rebased 2 descendant commits
-    Working copy now at: kmkuslsw 70a39dff f | f
-    Parent commit      : znkkpsqq a021fee9 e | e
+    Working copy  (@) now at: kmkuslsw 70a39dff f | f
+    Parent commit (@-)      : znkkpsqq a021fee9 e | e
     [EOF]
     ");
 
@@ -260,12 +246,12 @@ fn test_simplify_parents_multiple_redundant_parents() {
 fn test_simplify_parents_no_args() {
     let (test_env, repo_path) = create_repo();
 
-    create_commit(&test_env, &repo_path, "a", &["root()"]);
-    create_commit(&test_env, &repo_path, "b", &["a"]);
-    create_commit(&test_env, &repo_path, "c", &["a", "b"]);
-    create_commit(&test_env, &repo_path, "d", &["c"]);
-    create_commit(&test_env, &repo_path, "e", &["d"]);
-    create_commit(&test_env, &repo_path, "f", &["d", "e"]);
+    create_commit(&test_env.work_dir(&repo_path), "a", &["root()"]);
+    create_commit(&test_env.work_dir(&repo_path), "b", &["a"]);
+    create_commit(&test_env.work_dir(&repo_path), "c", &["a", "b"]);
+    create_commit(&test_env.work_dir(&repo_path), "d", &["c"]);
+    create_commit(&test_env.work_dir(&repo_path), "e", &["d"]);
+    create_commit(&test_env.work_dir(&repo_path), "f", &["d", "e"]);
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "all()", "-T", "description"]);
     insta::assert_snapshot!(output, @r"
     @    f
@@ -281,15 +267,15 @@ fn test_simplify_parents_no_args() {
     ◆
     [EOF]
     ");
-    let setup_opid = test_env.current_operation_id(&repo_path);
+    let setup_opid = test_env.work_dir(&repo_path).current_operation_id();
 
     let output = test_env.run_jj_in(&repo_path, ["simplify-parents"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Removed 2 edges from 2 out of 6 commits.
     Rebased 2 descendant commits
-    Working copy now at: kmkuslsw 8cc01e1b f | f
-    Parent commit      : znkkpsqq 040ae3a6 e | e
+    Working copy  (@) now at: kmkuslsw 8cc01e1b f | f
+    Parent commit (@-)      : znkkpsqq 040ae3a6 e | e
     [EOF]
     ");
 
@@ -314,8 +300,8 @@ fn test_simplify_parents_no_args() {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Removed 1 edges from 1 out of 3 commits.
-    Working copy now at: kmkuslsw 0c6b4c43 f | f
-    Parent commit      : znkkpsqq 6a679611 e | e
+    Working copy  (@) now at: kmkuslsw 0c6b4c43 f | f
+    Parent commit (@-)      : znkkpsqq 6a679611 e | e
     [EOF]
     ");
 
