@@ -69,19 +69,20 @@ fn test_split_by_paths() {
     ------- stderr -------
     First part: qpvuntsm 65569ca7 (no description set)
     Second part: zsuskuln 709756f0 (no description set)
-    Working copy now at: zsuskuln 709756f0 (no description set)
-    Parent commit      : qpvuntsm 65569ca7 (no description set)
+    Working copy  (@) now at: zsuskuln 709756f0 (no description set)
+    Parent commit (@-)      : qpvuntsm 65569ca7 (no description set)
     [EOF]
     ");
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor0")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
 
+
     JJ: This commit contains the following changes:
     JJ:     A file2
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     assert!(!test_env.env_root().join("editor1").exists());
 
     insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
@@ -123,8 +124,8 @@ fn test_split_by_paths() {
     Rebased 1 descendant commits
     First part: qpvuntsm 9da0eea0 (no description set)
     Second part: znkkpsqq 5b5714a3 (empty) (no description set)
-    Working copy now at: zsuskuln 0c798ee7 (no description set)
-    Parent commit      : znkkpsqq 5b5714a3 (empty) (no description set)
+    Working copy  (@) now at: zsuskuln 0c798ee7 (no description set)
+    Parent commit (@-)      : znkkpsqq 5b5714a3 (empty) (no description set)
     [EOF]
     ");
 
@@ -154,8 +155,8 @@ fn test_split_by_paths() {
     Rebased 1 descendant commits
     First part: qpvuntsm bd42f95a (empty) (no description set)
     Second part: lylxulpl ed55c86b (no description set)
-    Working copy now at: zsuskuln 1e1ed741 (no description set)
-    Parent commit      : lylxulpl ed55c86b (no description set)
+    Working copy  (@) now at: zsuskuln 1e1ed741 (no description set)
+    Parent commit (@-)      : lylxulpl ed55c86b (no description set)
     [EOF]
     ");
 
@@ -204,31 +205,31 @@ fn test_split_with_non_empty_description() {
     ------- stderr -------
     First part: qpvuntsm 231a3c00 part 1
     Second part: kkmpptxz e96291aa part 2
-    Working copy now at: kkmpptxz e96291aa part 2
-    Parent commit      : qpvuntsm 231a3c00 part 1
+    Working copy  (@) now at: kkmpptxz e96291aa part 2
+    Parent commit (@-)      : qpvuntsm 231a3c00 part 1
     [EOF]
     ");
 
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
     test
 
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r#"
     JJ: Enter a description for the second commit.
     test
 
     JJ: This commit contains the following changes:
     JJ:     A file2
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
     @  kkmpptxzrspx false part 2
     ○  qpvuntsmwlqt false part 1
@@ -258,8 +259,8 @@ fn test_split_with_default_description() {
     ------- stderr -------
     First part: qpvuntsm 02ee5d60 TESTED=TODO
     Second part: rlvkpnrz 33cd046b (no description set)
-    Working copy now at: rlvkpnrz 33cd046b (no description set)
-    Parent commit      : qpvuntsm 02ee5d60 TESTED=TODO
+    Working copy  (@) now at: rlvkpnrz 33cd046b (no description set)
+    Parent commit (@-)      : qpvuntsm 02ee5d60 TESTED=TODO
     [EOF]
     ");
 
@@ -268,7 +269,7 @@ fn test_split_with_default_description() {
     // default value we set. The second commit will inherit the empty
     // description from the commit being split.
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
 
 
@@ -276,9 +277,9 @@ fn test_split_with_default_description() {
 
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     assert!(!test_env.env_root().join("editor2").exists());
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
     @  rlvkpnrzqnoo false
@@ -288,6 +289,127 @@ fn test_split_with_default_description() {
     ");
 }
 
+#[test]
+fn test_split_with_descendants() {
+    // Configure the environment and make the initial commits.
+    let mut test_env = TestEnvironment::default();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let workspace_path = test_env.env_root().join("repo");
+
+    // First commit. This is the one we will split later.
+    std::fs::write(workspace_path.join("file1"), "foo\n").unwrap();
+    std::fs::write(workspace_path.join("file2"), "bar\n").unwrap();
+    test_env
+        .run_jj_in(&workspace_path, ["commit", "-m", "Add file1 & file2"])
+        .success();
+    // Second commit.
+    std::fs::write(workspace_path.join("file3"), "baz\n").unwrap();
+    test_env
+        .run_jj_in(&workspace_path, ["commit", "-m", "Add file3"])
+        .success();
+    // Third commit.
+    std::fs::write(workspace_path.join("file4"), "foobarbaz\n").unwrap();
+    test_env
+        .run_jj_in(&workspace_path, ["describe", "-m", "Add file4"])
+        .success();
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
+    @  kkmpptxzrspx false Add file4
+    ○  rlvkpnrzqnoo false Add file3
+    ○  qpvuntsmwlqt false Add file1 & file2
+    ◆  zzzzzzzzzzzz true
+    [EOF]
+    "###);
+
+    // Set up the editor and do the split.
+    let edit_script = test_env.set_up_fake_editor();
+    std::fs::write(
+        edit_script,
+        [
+            "dump editor1",
+            "write\nAdd file1",
+            "next invocation\n",
+            "dump editor2",
+            "write\nAdd file2",
+        ]
+        .join("\0"),
+    )
+    .unwrap();
+    let output = test_env.run_jj_in(&workspace_path, ["split", "file1", "-r", "qpvu"]);
+    insta::assert_snapshot!(output, @r"
+    ------- stderr -------
+    Rebased 2 descendant commits
+    First part: qpvuntsm 34dd141b Add file1
+    Second part: royxmykx 465e03d0 Add file2
+    Working copy  (@) now at: kkmpptxz 2d5d641f Add file4
+    Parent commit (@-)      : rlvkpnrz b3bd9eb7 Add file3
+    [EOF]
+    ");
+    insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r###"
+    @  kkmpptxzrspx false Add file4
+    ○  rlvkpnrzqnoo false Add file3
+    ○  royxmykxtrkr false Add file2
+    ○  qpvuntsmwlqt false Add file1
+    ◆  zzzzzzzzzzzz true
+    [EOF]
+    "###);
+
+    // The commit we're splitting has a description, so the user will be
+    // prompted to enter a description for each of the commits.
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
+    JJ: Enter a description for the first commit.
+    Add file1 & file2
+
+    JJ: This commit contains the following changes:
+    JJ:     A file1
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+    insta::assert_snapshot!(
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r#"
+    JJ: Enter a description for the second commit.
+    Add file1 & file2
+
+    JJ: This commit contains the following changes:
+    JJ:     A file2
+    JJ:
+    JJ: Lines starting with "JJ:" (like this one) will be removed.
+    "#);
+
+    // Check the evolog for the first commit. It shows four entries:
+    // - The initial empty commit.
+    // - The rewritten commit from the snapshot after the files were added.
+    // - The rewritten commit once the description is added during `jj commit`.
+    // - The rewritten commit after the split.
+    let evolog_1 = test_env.run_jj_in(&workspace_path, ["evolog", "-r", "qpvun"]);
+    insta::assert_snapshot!(evolog_1, @r###"
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:12 34dd141b
+    │  Add file1
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 764d46f1
+    │  Add file1 & file2
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 44af2155
+    │  (no description set)
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+       (empty) (no description set)
+    [EOF]
+    "###);
+
+    // The evolog for the second commit is the same, except that the change id
+    // changes after the split.
+    let evolog_2 = test_env.run_jj_in(&workspace_path, ["evolog", "-r", "royxm"]);
+    insta::assert_snapshot!(evolog_2, @r###"
+    ○  royxmykx test.user@example.com 2001-02-03 08:05:12 465e03d0
+    │  Add file2
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 764d46f1
+    │  Add file1 & file2
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 44af2155
+    │  (no description set)
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+       (empty) (no description set)
+    [EOF]
+    "###);
+}
+
 // This test makes sure that the children of the commit being split retain any
 // other parents which weren't involved in the split.
 #[test]
@@ -332,9 +454,9 @@ fn test_split_with_merge_child() {
     Rebased 1 descendant commits
     First part: kkmpptxz e8006b47 Add file1
     Second part: royxmykx 5e1b793d Add file2
-    Working copy now at: zsuskuln 696935af (empty) 2
-    Parent commit      : qpvuntsm 8b64ddff (empty) 1
-    Parent commit      : royxmykx 5e1b793d Add file2
+    Working copy  (@) now at: zsuskuln 696935af (empty) 2
+    Parent commit (@-)      : qpvuntsm 8b64ddff (empty) 1
+    Parent commit (@-)      : royxmykx 5e1b793d Add file2
     [EOF]
     ");
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
@@ -352,7 +474,7 @@ fn test_split_with_merge_child() {
 #[test]
 // Split a commit with no descendants into siblings. Also tests that the default
 // description is set correctly on the first commit.
-fn test_split_siblings_no_descendants() {
+fn test_split_parallel_no_descendants() {
     let mut test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     test_env.add_config(r#"ui.default-description = "\n\nTESTED=TODO""#);
@@ -378,8 +500,8 @@ fn test_split_siblings_no_descendants() {
     ------- stderr -------
     First part: qpvuntsm 48018df6 TESTED=TODO
     Second part: kkmpptxz 7eddbf93 (no description set)
-    Working copy now at: kkmpptxz 7eddbf93 (no description set)
-    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) now at: kkmpptxz 7eddbf93 (no description set)
+    Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 0 files, removed 1 files
     [EOF]
     ");
@@ -396,7 +518,7 @@ fn test_split_siblings_no_descendants() {
     // default value we set. The second commit will inherit the empty
     // description from the commit being split.
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
 
 
@@ -404,18 +526,45 @@ fn test_split_siblings_no_descendants() {
 
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     assert!(!test_env.env_root().join("editor2").exists());
+
+    // Check the evolog for the first commit. It shows three entries:
+    // - The initial empty commit.
+    // - The rewritten commit from the snapshot after the files were added.
+    // - The rewritten commit after the split.
+    let evolog_1 = test_env.run_jj_in(&workspace_path, ["evolog", "-r", "qpvun"]);
+    insta::assert_snapshot!(evolog_1, @r###"
+    ○  qpvuntsm test.user@example.com 2001-02-03 08:05:09 48018df6
+    │  TESTED=TODO
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 44af2155
+    │  (no description set)
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+       (empty) (no description set)
+    [EOF]
+    "###);
+
+    // The evolog for the second commit is the same, except that the change id
+    // changes after the split.
+    let evolog_2 = test_env.run_jj_in(&workspace_path, ["evolog", "-r", "kkmpp"]);
+    insta::assert_snapshot!(evolog_2, @r###"
+    @  kkmpptxz test.user@example.com 2001-02-03 08:05:09 7eddbf93
+    │  (no description set)
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 44af2155
+    │  (no description set)
+    ○  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
+       (empty) (no description set)
+    [EOF]
+    "###);
 }
 
 #[test]
-fn test_split_siblings_with_descendants() {
+fn test_split_parallel_with_descendants() {
     // Configure the environment and make the initial commits.
     let mut test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    // test_env.add_config(r#"ui.default-description = "\n\nTESTED=TODO""#);
     let workspace_path = test_env.env_root().join("repo");
 
     // First commit. This is the one we will split later.
@@ -470,8 +619,8 @@ fn test_split_siblings_with_descendants() {
     Rebased 2 descendant commits
     First part: qpvuntsm 84df941d Add file1
     Second part: vruxwmqv 94753be3 Add file2
-    Working copy now at: vruxwmqv 94753be3 Add file2
-    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) now at: vruxwmqv 94753be3 Add file2
+    Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     Added 0 files, modified 0 files, removed 1 files
     [EOF]
     ");
@@ -489,31 +638,31 @@ fn test_split_siblings_with_descendants() {
     // The commit we're splitting has a description, so the user will be
     // prompted to enter a description for each of the sibling commits.
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor1")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
     Add file1 & file2
 
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor2")).unwrap(), @r#"
     JJ: Enter a description for the second commit.
     Add file1 & file2
 
     JJ: This commit contains the following changes:
     JJ:     A file2
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
 }
 
 // This test makes sure that the children of the commit being split retain any
 // other parents which weren't involved in the split.
 #[test]
-fn test_split_siblings_with_merge_child() {
+fn test_split_parallel_with_merge_child() {
     let mut test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let workspace_path = test_env.env_root().join("repo");
@@ -557,10 +706,10 @@ fn test_split_siblings_with_merge_child() {
     Rebased 1 descendant commits
     First part: kkmpptxz e8006b47 Add file1
     Second part: royxmykx 2cc60f3d Add file2
-    Working copy now at: zsuskuln 35b5d7eb (empty) 2
-    Parent commit      : qpvuntsm 8b64ddff (empty) 1
-    Parent commit      : kkmpptxz e8006b47 Add file1
-    Parent commit      : royxmykx 2cc60f3d Add file2
+    Working copy  (@) now at: zsuskuln 35b5d7eb (empty) 2
+    Parent commit (@-)      : qpvuntsm 8b64ddff (empty) 1
+    Parent commit (@-)      : kkmpptxz e8006b47 Add file1
+    Parent commit (@-)      : royxmykx 2cc60f3d Add file2
     [EOF]
     ");
     insta::assert_snapshot!(get_log_output(&test_env, &workspace_path), @r"
@@ -638,30 +787,31 @@ fn test_split_interactive() {
     ------- stderr -------
     First part: qpvuntsm 0e15949e (no description set)
     Second part: rlvkpnrz 9ed12e4c (no description set)
-    Working copy now at: rlvkpnrz 9ed12e4c (no description set)
-    Parent commit      : qpvuntsm 0e15949e (no description set)
+    Working copy  (@) now at: rlvkpnrz 9ed12e4c (no description set)
+    Parent commit (@-)      : qpvuntsm 0e15949e (no description set)
     [EOF]
     ");
 
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r#"
+        std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r"
     You are splitting a commit into two: qpvuntsm 44af2155 (no description set)
 
     The diff initially shows the changes in the commit you're splitting.
 
     Adjust the right side until it shows the contents you want for the first commit.
     The remainder will be in the second commit.
-    "#);
+    ");
 
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
 
+
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
 
     let output = test_env.run_jj_in(&workspace_path, ["log", "--summary"]);
     insta::assert_snapshot!(output, @r"
@@ -692,6 +842,11 @@ fn test_split_interactive_with_paths() {
     let edit_script = test_env.set_up_fake_editor();
     std::fs::write(edit_script, ["dump editor"].join("\0")).unwrap();
     let diff_editor = test_env.set_up_fake_diff_editor();
+    // On the before side, file2 is empty. On the after side, it contains "bar".
+    // The "reset file2" copies the empty version from the before side to the
+    // after side, effectively "unselecting" the changes and leaving only the
+    // changes made to file1. file3 doesn't appear on either side since it isn't
+    // in the filesets passed to `jj split`.
     let diff_script = [
         "files-before file2",
         "files-after JJ-INSTRUCTIONS file1 file2",
@@ -700,26 +855,28 @@ fn test_split_interactive_with_paths() {
     .join("\0");
     std::fs::write(diff_editor, diff_script).unwrap();
 
-    // Select file1 and file2 by args, then select file1 interactively
+    // Select file1 and file2 by args, then select file1 interactively via the diff
+    // script.
     let output = test_env.run_jj_in(&workspace_path, ["split", "-i", "file1", "file2"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     First part: rlvkpnrz e3d766b8 (no description set)
     Second part: kkmpptxz 4cf22d3b (no description set)
-    Working copy now at: kkmpptxz 4cf22d3b (no description set)
-    Parent commit      : rlvkpnrz e3d766b8 (no description set)
+    Working copy  (@) now at: kkmpptxz 4cf22d3b (no description set)
+    Parent commit (@-)      : rlvkpnrz e3d766b8 (no description set)
     [EOF]
     ");
 
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("editor")).unwrap(), @r#"
     JJ: Enter a description for the first commit.
 
+
     JJ: This commit contains the following changes:
     JJ:     A file1
-
+    JJ:
     JJ: Lines starting with "JJ:" (like this one) will be removed.
-    "###);
+    "#);
 
     let output = test_env.run_jj_in(&workspace_path, ["log", "--summary"]);
     insta::assert_snapshot!(output, @r"
@@ -935,8 +1092,8 @@ fn test_split_with_bookmarks(bookmark_behavior: BookmarkBehavior) {
             ------- stderr -------
             First part: qpvuntsm 63d0c5ed *le-signet* | first-commit
             Second part: mzvwutvl a9f5665f second-commit
-            Working copy now at: mzvwutvl a9f5665f second-commit
-            Parent commit      : qpvuntsm 63d0c5ed *le-signet* | first-commit
+            Working copy  (@) now at: mzvwutvl a9f5665f second-commit
+            Parent commit (@-)      : qpvuntsm 63d0c5ed *le-signet* | first-commit
             [EOF]
             ");
             }
@@ -951,14 +1108,14 @@ fn test_split_with_bookmarks(bookmark_behavior: BookmarkBehavior) {
         }
         BookmarkBehavior::Default | BookmarkBehavior::MoveBookmarkToChild => {
             insta::allow_duplicates! {
-            insta::assert_snapshot!(output, @r###"
+            insta::assert_snapshot!(output, @r"
             ------- stderr -------
             First part: qpvuntsm 63d0c5ed first-commit
             Second part: mzvwutvl a9f5665f *le-signet* | second-commit
-            Working copy now at: mzvwutvl a9f5665f *le-signet* | second-commit
-            Parent commit      : qpvuntsm 63d0c5ed first-commit
+            Working copy  (@) now at: mzvwutvl a9f5665f *le-signet* | second-commit
+            Parent commit (@-)      : qpvuntsm 63d0c5ed first-commit
             [EOF]
-            "###);
+            ");
             }
             insta::allow_duplicates! {
             insta::assert_snapshot!(get_log_output(&test_env, &main_path), @r"
