@@ -12,36 +12,9 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
-
+use crate::common::create_commit_with_files;
 use crate::common::TestEnvironment;
 
-fn create_commit(
-    test_env: &TestEnvironment,
-    repo_path: &Path,
-    name: &str,
-    parents: &[&str],
-    files: &[(&str, &str)],
-) {
-    let parents = match parents {
-        [] => &["root()"],
-        parents => parents,
-    };
-    test_env
-        .run_jj_with(|cmd| {
-            cmd.current_dir(repo_path)
-                .args(["new", "-m", name])
-                .args(parents)
-        })
-        .success();
-    for (name, content) in files {
-        std::fs::write(repo_path.join(name), content).unwrap();
-    }
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", name])
-        .success();
-}
-
 #[test]
 fn test_status_copies() {
     let test_env = TestEnvironment::default();
@@ -70,8 +43,8 @@ fn test_status_copies() {
     M copy-source
     C {copy-source => copy-target}
     R {rename-source => rename-target}
-    Working copy : rlvkpnrz a96c3086 (no description set)
-    Parent commit: qpvuntsm e3e2c703 (no description set)
+    Working copy  (@) : rlvkpnrz a96c3086 (no description set)
+    Parent commit (@-): qpvuntsm e3e2c703 (no description set)
     [EOF]
     ");
 }
@@ -100,9 +73,9 @@ fn test_status_merge() {
     let output = test_env.run_jj_in(&repo_path, ["status"]);
     insta::assert_snapshot!(output, @r"
     The working copy has no changes.
-    Working copy : mzvwutvl a538c72d (empty) (no description set)
-    Parent commit: rlvkpnrz d3dd19f1 left | (empty) left
-    Parent commit: zsuskuln 705a356d right
+    Working copy  (@) : mzvwutvl a538c72d (empty) (no description set)
+    Parent commit (@-): rlvkpnrz d3dd19f1 left | (empty) left
+    Parent commit (@-): zsuskuln 705a356d right
     [EOF]
     ");
 }
@@ -127,8 +100,8 @@ fn test_status_ignored_gitignore() {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     A .gitignore
-    Working copy : qpvuntsm 3cef2183 (no description set)
-    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) : qpvuntsm 3cef2183 (no description set)
+    Parent commit (@-): zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
 }
@@ -147,8 +120,8 @@ fn test_status_filtered() {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     A file_1
-    Working copy : qpvuntsm c8fb8395 (no description set)
-    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) : qpvuntsm c8fb8395 (no description set)
+    Parent commit (@-): zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
 }
@@ -227,8 +200,8 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     let output = test_env.run_jj_in(&repo_path, ["status"]);
     insta::assert_snapshot!(output, @r"
     The working copy has no changes.
-    Working copy : yqosqzyt dcb25635 (conflict) (empty) boom-cont-2
-    Parent commit: royxmykx 664a4c6c (conflict) (empty) boom-cont
+    Working copy  (@) : yqosqzyt dcb25635 (conflict) (empty) boom-cont-2
+    Parent commit (@-): royxmykx 664a4c6c (conflict) (empty) boom-cont
     Warning: There are unresolved conflicts at these paths:
     conflicted.txt    2-sided conflict
     Hint: To resolve the conflicts, start by updating to the first one:
@@ -242,8 +215,8 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     let output = test_env.run_jj_in(&repo_path, ["status", "--color=always"]);
     insta::assert_snapshot!(output, @r"
     The working copy has no changes.
-    Working copy : [1m[38;5;13my[38;5;8mqosqzyt[39m [38;5;12md[38;5;8mcb25635[39m [38;5;9m(conflict)[39m [38;5;10m(empty)[39m boom-cont-2[0m
-    Parent commit: [1m[38;5;5mr[0m[38;5;8moyxmykx[39m [1m[38;5;4m6[0m[38;5;8m64a4c6c[39m [38;5;1m(conflict)[39m [38;5;2m(empty)[39m boom-cont
+    Working copy  (@) : [1m[38;5;13my[38;5;8mqosqzyt[39m [38;5;12md[38;5;8mcb25635[39m [38;5;9m(conflict)[39m [38;5;10m(empty)[39m boom-cont-2[0m
+    Parent commit (@-): [1m[38;5;5mr[0m[38;5;8moyxmykx[39m [1m[38;5;4m6[0m[38;5;8m64a4c6c[39m [38;5;1m(conflict)[39m [38;5;2m(empty)[39m boom-cont
     [1m[38;5;3mWarning: [39mThere are unresolved conflicts at these paths:[0m
     conflicted.txt    [38;5;3m2-sided conflict[39m
     [1m[38;5;6mHint: [0m[39mTo resolve the conflicts, start by updating to the first one:[39m
@@ -254,6 +227,19 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     [EOF]
     ");
 
+    let output = test_env.run_jj_in(
+        &repo_path,
+        ["status", "--config=hints.resolving-conflicts=false"],
+    );
+    insta::assert_snapshot!(output, @r"
+    The working copy has no changes.
+    Working copy  (@) : yqosqzyt dcb25635 (conflict) (empty) boom-cont-2
+    Parent commit (@-): royxmykx 664a4c6c (conflict) (empty) boom-cont
+    Warning: There are unresolved conflicts at these paths:
+    conflicted.txt    2-sided conflict
+    [EOF]
+    ");
+
     // Resolve conflict
     test_env
         .run_jj_in(&repo_path, ["new", "--message", "fixed 1"])
@@ -270,9 +256,9 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "::"]);
 
     insta::assert_snapshot!(output, @r"
-    @  kmkuslsw test.user@example.com 2001-02-03 08:05:19 caa7e9d5
+    @  wqnwkozp test.user@example.com 2001-02-03 08:05:20 c4a6dbc2
     │  fixed 2
-    ○  kpqxywon test.user@example.com 2001-02-03 08:05:18 26bf6863
+    ○  kmkuslsw test.user@example.com 2001-02-03 08:05:19 fcebf6ee
     │  fixed 1
     ×  yqosqzyt test.user@example.com 2001-02-03 08:05:13 dcb25635 conflict
     │  (empty) boom-cont-2
@@ -295,8 +281,8 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     M conflicted.txt
-    Working copy : kmkuslsw caa7e9d5 fixed 2
-    Parent commit: kpqxywon 26bf6863 fixed 1
+    Working copy  (@) : wqnwkozp c4a6dbc2 fixed 2
+    Parent commit (@-): kmkuslsw fcebf6ee fixed 1
     [EOF]
     ");
 
@@ -306,9 +292,9 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "::"]);
 
     insta::assert_snapshot!(output, @r"
-    ○  kmkuslsw test.user@example.com 2001-02-03 08:05:19 caa7e9d5
+    ○  wqnwkozp test.user@example.com 2001-02-03 08:05:20 c4a6dbc2
     │  fixed 2
-    @  kpqxywon test.user@example.com 2001-02-03 08:05:18 26bf6863
+    @  kmkuslsw test.user@example.com 2001-02-03 08:05:19 fcebf6ee
     │  fixed 1
     ×  yqosqzyt test.user@example.com 2001-02-03 08:05:13 dcb25635 conflict
     │  (empty) boom-cont-2
@@ -331,8 +317,8 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     M conflicted.txt
-    Working copy : kpqxywon 26bf6863 fixed 1
-    Parent commit: yqosqzyt dcb25635 (conflict) (empty) boom-cont-2
+    Working copy  (@) : kmkuslsw fcebf6ee fixed 1
+    Parent commit (@-): yqosqzyt dcb25635 (conflict) (empty) boom-cont-2
     Hint: Conflict in parent commit has been resolved in working copy
     [EOF]
     ");
@@ -346,9 +332,9 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     let output = test_env.run_jj_in(&repo_path, ["log", "-r", "::"]);
 
     insta::assert_snapshot!(output, @r"
-    ○  kmkuslsw test.user@example.com 2001-02-03 08:05:19 caa7e9d5
+    ○  wqnwkozp test.user@example.com 2001-02-03 08:05:20 c4a6dbc2
     │  fixed 2
-    ○  kpqxywon test.user@example.com 2001-02-03 08:05:18 26bf6863
+    ○  kmkuslsw test.user@example.com 2001-02-03 08:05:19 fcebf6ee
     │  fixed 1
     ×  yqosqzyt test.user@example.com 2001-02-03 08:05:13 dcb25635 conflict
     │  (empty) boom-cont-2
@@ -371,8 +357,8 @@ fn test_status_display_relevant_working_commit_conflict_hints() {
     insta::assert_snapshot!(output, @r"
     Working copy changes:
     A conflicted.txt
-    Working copy : qpvuntsm aade7195 Initial contents
-    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) : qpvuntsm aade7195 Initial contents
+    Parent commit (@-): zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
 }
@@ -386,22 +372,50 @@ fn test_status_simplify_conflict_sides() {
     // Creates a 4-sided conflict, with fileA and fileB having different conflicts:
     // fileA: A - B + C - B + B - B + B
     // fileB: A - A + A - A + B - C + D
-    create_commit(
-        &test_env,
-        &repo_path,
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
         "base",
         &[],
         &[("fileA", "base\n"), ("fileB", "base\n")],
     );
-    create_commit(&test_env, &repo_path, "a1", &["base"], &[("fileA", "1\n")]);
-    create_commit(&test_env, &repo_path, "a2", &["base"], &[("fileA", "2\n")]);
-    create_commit(&test_env, &repo_path, "b1", &["base"], &[("fileB", "1\n")]);
-    create_commit(&test_env, &repo_path, "b2", &["base"], &[("fileB", "2\n")]);
-    create_commit(&test_env, &repo_path, "conflictA", &["a1", "a2"], &[]);
-    create_commit(&test_env, &repo_path, "conflictB", &["b1", "b2"], &[]);
-    create_commit(
-        &test_env,
-        &repo_path,
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "a1",
+        &["base"],
+        &[("fileA", "1\n")],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "a2",
+        &["base"],
+        &[("fileA", "2\n")],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "b1",
+        &["base"],
+        &[("fileB", "1\n")],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "b2",
+        &["base"],
+        &[("fileB", "2\n")],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "conflictA",
+        &["a1", "a2"],
+        &[],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
+        "conflictB",
+        &["b1", "b2"],
+        &[],
+    );
+    create_commit_with_files(
+        &test_env.work_dir(&repo_path),
         "conflict",
         &["conflictA", "conflictB"],
         &[],
@@ -410,9 +424,9 @@ fn test_status_simplify_conflict_sides() {
     insta::assert_snapshot!(test_env.run_jj_in(&repo_path, ["status"]),
     @r"
     The working copy has no changes.
-    Working copy : nkmrtpmo 83c4b9e7 conflict | (conflict) (empty) conflict
-    Parent commit: kmkuslsw 4601566f conflictA | (conflict) (empty) conflictA
-    Parent commit: lylxulpl 6f8d8381 conflictB | (conflict) (empty) conflictB
+    Working copy  (@) : nkmrtpmo 83c4b9e7 conflict | (conflict) (empty) conflict
+    Parent commit (@-): kmkuslsw 4601566f conflictA | (conflict) (empty) conflictA
+    Parent commit (@-): lylxulpl 6f8d8381 conflictB | (conflict) (empty) conflictB
     Warning: There are unresolved conflicts at these paths:
     fileA    2-sided conflict
     fileB    2-sided conflict
@@ -447,8 +461,8 @@ fn test_status_untracked_files() {
     ? initially-untracked-file
     ? sub/always-untracked
     ? sub/initially-untracked
-    Working copy : qpvuntsm 230dd059 (empty) (no description set)
-    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) : qpvuntsm 230dd059 (empty) (no description set)
+    Parent commit (@-): zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
 
@@ -472,8 +486,8 @@ fn test_status_untracked_files() {
     Untracked paths:
     ? always-untracked-file
     ? sub/always-untracked
-    Working copy : qpvuntsm 99798fcd (no description set)
-    Parent commit: zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) : qpvuntsm 99798fcd (no description set)
+    Parent commit (@-): zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
 
@@ -484,8 +498,8 @@ fn test_status_untracked_files() {
     Untracked paths:
     ? always-untracked-file
     ? sub/always-untracked
-    Working copy : mzvwutvl 30e53c74 (empty) (no description set)
-    Parent commit: qpvuntsm 99798fcd (no description set)
+    Working copy  (@) : mzvwutvl 30e53c74 (empty) (no description set)
+    Parent commit (@-): qpvuntsm 99798fcd (no description set)
     [EOF]
     ");
 
@@ -510,8 +524,8 @@ fn test_status_untracked_files() {
     ? initially-untracked-file
     ? sub/always-untracked
     ? sub/initially-untracked
-    Working copy : mzvwutvl bb362aaf (no description set)
-    Parent commit: qpvuntsm 99798fcd (no description set)
+    Working copy  (@) : mzvwutvl bb362aaf (no description set)
+    Parent commit (@-): qpvuntsm 99798fcd (no description set)
     [EOF]
     ");
 
@@ -524,8 +538,8 @@ fn test_status_untracked_files() {
     ? initially-untracked-file
     ? sub/always-untracked
     ? sub/initially-untracked
-    Working copy : yostqsxw 8e8c02fe (empty) (no description set)
-    Parent commit: mzvwutvl bb362aaf (no description set)
+    Working copy  (@) : yostqsxw 8e8c02fe (empty) (no description set)
+    Parent commit (@-): mzvwutvl bb362aaf (no description set)
     [EOF]
     ");
 }
