@@ -122,6 +122,20 @@ fn test_templater_parse_error() {
     [exit status: 1]
     ");
 
+    // "at least N arguments"
+    insta::assert_snapshot!(render("separate()"), @r"
+    ------- stderr -------
+    Error: Failed to parse template: Function `separate`: Expected at least 1 arguments
+    Caused by:  --> 1:10
+      |
+    1 | separate()
+      |          ^
+      |
+      = Function `separate`: Expected at least 1 arguments
+    [EOF]
+    [exit status: 1]
+    ");
+
     // -Tbuiltin shows the predefined builtin_* aliases. This isn't 100%
     // guaranteed, but is nice.
     insta::assert_snapshot!(render(r#"builtin"#), @r"
@@ -133,7 +147,7 @@ fn test_templater_parse_error() {
       | ^-----^
       |
       = Keyword `builtin` doesn't exist
-    Hint: Did you mean `builtin_log_comfortable`, `builtin_log_compact`, `builtin_log_compact_full_description`, `builtin_log_detailed`, `builtin_log_node`, `builtin_log_node_ascii`, `builtin_log_oneline`, `builtin_op_log_comfortable`, `builtin_op_log_compact`, `builtin_op_log_node`, `builtin_op_log_node_ascii`, `builtin_op_log_oneline`?
+    Hint: Did you mean `builtin_config_list`, `builtin_config_list_detailed`, `builtin_draft_commit_description`, `builtin_log_comfortable`, `builtin_log_compact`, `builtin_log_compact_full_description`, `builtin_log_detailed`, `builtin_log_node`, `builtin_log_node_ascii`, `builtin_log_oneline`, `builtin_op_log_comfortable`, `builtin_op_log_compact`, `builtin_op_log_node`, `builtin_op_log_node_ascii`, `builtin_op_log_oneline`?
     [EOF]
     [exit status: 1]
     ");
@@ -147,58 +161,20 @@ fn test_template_parse_warning() {
 
     let template = indoc! {r#"
         separate(' ',
-          branches,
-          local_branches,
-          remote_branches,
-          self.contained_in('branches()'),
           author.username(),
         )
     "#};
     let output = test_env.run_jj_in(&repo_path, ["log", "-r@", "-T", template]);
     insta::assert_snapshot!(output, @r"
-    @  false test.user
+    @  test.user
     │
     ~
     [EOF]
     ------- stderr -------
     Warning: In template expression
-     --> 2:3
+     --> 2:10
       |
-    2 |   branches,
-      |   ^------^
-      |
-      = branches() is deprecated; use bookmarks() instead
-    Warning: In template expression
-     --> 3:3
-      |
-    3 |   local_branches,
-      |   ^------------^
-      |
-      = local_branches() is deprecated; use local_bookmarks() instead
-    Warning: In template expression
-     --> 4:3
-      |
-    4 |   remote_branches,
-      |   ^-------------^
-      |
-      = remote_branches() is deprecated; use remote_bookmarks() instead
-    Warning: In template expression
-     --> 5:21
-      |
-    5 |   self.contained_in('branches()'),
-      |                     ^----------^
-      |
-      = In revset expression
-     --> 1:1
-      |
-    1 | branches()
-      | ^------^
-      |
-      = branches() is deprecated; use bookmarks() instead
-    Warning: In template expression
-     --> 6:10
-      |
-    6 |   author.username(),
+    2 |   author.username(),
       |          ^------^
       |
       = username() is deprecated; use email().local() instead
@@ -238,7 +214,7 @@ fn test_templater_alias() {
     'recurse2()' = 'recurse'
     'identity(x)' = 'x'
     'coalesce(x, y)' = 'if(x, x, y)'
-    'deprecated()' = 'branches ++ self.contained_in("branches()")'
+    'deprecated()' = 'author.username()'
     'builtin_log_node' = '"#"'
     'builtin_op_log_node' = '"#"'
     "###,
@@ -429,8 +405,8 @@ fn test_templater_alias() {
     ");
 
     let output = test_env.run_jj_in(&repo_path, ["log", "-r@", "-Tdeprecated()"]);
-    insta::assert_snapshot!(output, @r##"
-    #  false
+    insta::assert_snapshot!(output, @r"
+    #  test.user
     │
     ~
     [EOF]
@@ -442,33 +418,14 @@ fn test_templater_alias() {
       | ^----------^
       |
       = In alias `deprecated()`
-     --> 1:1
+     --> 1:8
       |
-    1 | branches ++ self.contained_in("branches()")
-      | ^------^
+    1 | author.username()
+      |        ^------^
       |
-      = branches() is deprecated; use bookmarks() instead
-    Warning: In template expression
-     --> 1:1
-      |
-    1 | deprecated()
-      | ^----------^
-      |
-      = In alias `deprecated()`
-     --> 1:31
-      |
-    1 | branches ++ self.contained_in("branches()")
-      |                               ^----------^
-      |
-      = In revset expression
-     --> 1:1
-      |
-    1 | branches()
-      | ^------^
-      |
-      = branches() is deprecated; use bookmarks() instead
+      = username() is deprecated; use email().local() instead
     [EOF]
-    "##);
+    ");
 }
 
 #[test]
