@@ -38,10 +38,10 @@ use futures::stream::BoxStream;
 use gix::bstr::BString;
 use gix::objs::CommitRef;
 use gix::objs::CommitRefIter;
-use gix::objs::WriteTo;
-use itertools::Itertools;
-use pollster::FutureExt;
-use prost::Message;
+use gix::objs::WriteTo as _;
+use itertools::Itertools as _;
+use pollster::FutureExt as _;
+use prost::Message as _;
 use smallvec::SmallVec;
 use thiserror::Error;
 
@@ -82,7 +82,7 @@ use crate::repo_path::RepoPathComponentBuf;
 use crate::settings::UserSettings;
 use crate::stacked_table::MutableTable;
 use crate::stacked_table::ReadonlyTable;
-use crate::stacked_table::TableSegment;
+use crate::stacked_table::TableSegment as _;
 use crate::stacked_table::TableStore;
 use crate::stacked_table::TableStoreError;
 
@@ -92,7 +92,7 @@ const CHANGE_ID_LENGTH: usize = 16;
 const NO_GC_REF_NAMESPACE: &str = "refs/jj/keep/";
 const CONFLICT_SUFFIX: &str = ".jjconflict";
 
-const JJ_TREES_COMMIT_HEADER: &[u8] = b"jj:trees";
+pub const JJ_TREES_COMMIT_HEADER: &[u8] = b"jj:trees";
 
 #[derive(Debug, Error)]
 pub enum GitBackendInitError {
@@ -306,11 +306,6 @@ impl GitBackend {
         self.base_repo.to_thread_local()
     }
 
-    /// Creates new owned git repository instance.
-    pub fn open_git_repo(&self) -> Result<git2::Repository, git2::Error> {
-        git2::Repository::open(self.git_repo_path())
-    }
-
     /// Path to the `.git` directory or the repository itself if it's bare.
     pub fn git_repo_path(&self) -> &Path {
         self.base_repo.path()
@@ -367,14 +362,6 @@ impl GitBackend {
     pub fn import_head_commits<'a>(
         &self,
         head_ids: impl IntoIterator<Item = &'a CommitId>,
-    ) -> BackendResult<()> {
-        self.import_head_commits_with_tree_conflicts(head_ids, true)
-    }
-
-    fn import_head_commits_with_tree_conflicts<'a>(
-        &self,
-        head_ids: impl IntoIterator<Item = &'a CommitId>,
-        uses_tree_conflict_format: bool,
     ) -> BackendResult<()> {
         let head_ids: HashSet<&CommitId> = head_ids
             .into_iter()
@@ -404,7 +391,6 @@ impl GitBackend {
             &mut mut_table,
             &table_lock,
             &head_ids,
-            uses_tree_conflict_format,
         )?;
         self.save_extra_metadata_table(mut_table, &table_lock)
     }
@@ -492,6 +478,10 @@ fn gix_open_opts_from_settings(settings: &UserSettings) -> gix::open::Options {
         ])
         // The git_target path should point the repository, not the working directory.
         .open_path_as_is(true)
+        // Gitoxide recommends this when correctness is preferred
+        .strict_config(true)
+        // This breaks tests and generally seems undesirable
+        .lossy_config(false)
 }
 
 /// Reads the `jj:trees` header from the commit.
@@ -506,7 +496,10 @@ fn root_tree_from_header(git_commit: &CommitRef) -> Result<Option<MergedTreeId>,
                 }
                 tree_ids.push(tree_id);
             }
-            if tree_ids.len() % 2 == 0 {
+            // It is invalid to use `jj:trees` with a non-conflicted tree. If this were
+            // allowed, it would be possible to construct a commit which appears to have
+            // different contents depending on whether it is viewed using `jj` or `git`.
+            if tree_ids.len() == 1 || tree_ids.len() % 2 == 0 {
                 return Err(());
             }
             return Ok(Some(MergedTreeId::Merge(Merge::from_vec(tree_ids))));
@@ -659,6 +652,9 @@ fn serialize_extras(commit: &Commit) -> Vec<u8> {
     if let MergedTreeId::Merge(tree_ids) = &commit.root_tree {
         proto.uses_tree_conflict_format = true;
         if !tree_ids.is_resolved() {
+            // This is done for the sake of jj versions <0.28 (before commit
+            // f7b14be) being able to read the repo. At some point in the
+            // future, we can stop doing it.
             proto.root_tree = tree_ids.iter().map(|r| r.to_bytes()).collect();
         }
     }
@@ -671,28 +667,21 @@ fn serialize_extras(commit: &Commit) -> Vec<u8> {
 fn deserialize_extras(commit: &mut Commit, bytes: &[u8]) {
     let proto = crate::protos::git_store::Commit::decode(bytes).unwrap();
     commit.change_id = ChangeId::new(proto.change_id);
-    if proto.uses_tree_conflict_format {
-        if !proto.root_tree.is_empty() {
-            let merge_builder: MergeBuilder<_> = proto
-                .root_tree
-                .iter()
-                .map(|id_bytes| TreeId::from_bytes(id_bytes))
-                .collect();
-            let merge = merge_builder.build();
-            // Check that the trees from the extras match the one we found in the jj:trees
-            // header
-            if let MergedTreeId::Merge(existing_merge) = &commit.root_tree {
-                assert!(existing_merge.is_resolved() || *existing_merge == merge);
+    if let MergedTreeId::Legacy(legacy_tree_id) = &commit.root_tree {
+        if proto.uses_tree_conflict_format {
+            if !proto.root_tree.is_empty() {
+                let merge_builder: MergeBuilder<_> = proto
+                    .root_tree
+                    .iter()
+                    .map(|id_bytes| TreeId::from_bytes(id_bytes))
+                    .collect();
+                commit.root_tree = MergedTreeId::Merge(merge_builder.build());
+            } else {
+                // uses_tree_conflict_format was set but there was no root_tree override in the
+                // proto, which means we should just promote the tree id from the
+                // git commit to be a known-conflict-free tree
+                commit.root_tree = MergedTreeId::resolved(legacy_tree_id.clone());
             }
-            commit.root_tree = MergedTreeId::Merge(merge);
-        } else {
-            // uses_tree_conflict_format was set but there was no root_tree override in the
-            // proto, which means we should just promote the tree id from the
-            // git commit to be a known-conflict-free tree
-            let MergedTreeId::Legacy(legacy_tree_id) = &commit.root_tree else {
-                panic!("root tree should have been initialized to a legacy id");
-            };
-            commit.root_tree = MergedTreeId::resolved(legacy_tree_id.clone());
         }
     }
     for predecessor in &proto.predecessors {
@@ -865,7 +854,6 @@ fn import_extra_metadata_entries_from_heads(
     mut_table: &mut MutableTable,
     _table_lock: &FileLock,
     head_ids: &HashSet<&CommitId>,
-    uses_tree_conflict_format: bool,
 ) -> BackendResult<()> {
     let shallow_commits = git_repo
         .shallow_commits()
@@ -886,12 +874,7 @@ fn import_extra_metadata_entries_from_heads(
         // TODO(#1624): Should we read the root tree here and check if it has a
         // `.jjconflict-...` entries? That could happen if the user used `git` to e.g.
         // change the description of a commit with tree-level conflicts.
-        let commit = commit_from_git_without_root_parent(
-            &id,
-            &git_object,
-            uses_tree_conflict_format,
-            is_shallow,
-        )?;
+        let commit = commit_from_git_without_root_parent(&id, &git_object, true, is_shallow)?;
         mut_table.add_entry(id.to_bytes(), serialize_extras(&commit));
         work_ids.extend(
             commit
@@ -1335,7 +1318,7 @@ impl Backend for GitBackend {
                     .map_err(|err| to_invalid_utf8_err(err, head_id))?;
 
                 let target = RepoPathBuf::from_internal_string(dest);
-                if !paths.map_or(true, |paths| paths.contains(&target)) {
+                if !paths.is_none_or(|paths| paths.contains(&target)) {
                     return Ok(None);
                 }
 
@@ -1531,9 +1514,8 @@ fn bytes_vec_from_json(value: &serde_json::Value) -> Vec<u8> {
 #[cfg(test)]
 mod tests {
     use assert_matches::assert_matches;
-    use hex::ToHex;
-    use pollster::FutureExt;
-    use test_case::test_case;
+    use hex::ToHex as _;
+    use pollster::FutureExt as _;
 
     use super::*;
     use crate::config::StackedConfig;
@@ -1552,7 +1534,10 @@ mod tests {
     }
 
     fn open_options() -> gix::open::Options {
-        gix::open::Options::isolated().config_overrides(git_config())
+        gix::open::Options::isolated()
+            .config_overrides(git_config())
+            .strict_config(true)
+            .lossy_config(false)
     }
 
     fn git_init(directory: impl AsRef<Path>) -> gix::Repository {
@@ -1566,9 +1551,8 @@ mod tests {
         .to_thread_local()
     }
 
-    #[test_case(false; "legacy tree format")]
-    #[test_case(true; "tree-level conflict format")]
-    fn read_plain_git_commit(uses_tree_conflict_format: bool) {
+    #[test]
+    fn read_plain_git_commit() {
         let settings = user_settings();
         let temp_dir = new_temp_dir();
         let store_path = temp_dir.path();
@@ -1649,9 +1633,7 @@ mod tests {
         let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
 
         // Import the head commit and its ancestors
-        backend
-            .import_head_commits_with_tree_conflicts([&commit_id2], uses_tree_conflict_format)
-            .unwrap();
+        backend.import_head_commits([&commit_id2]).unwrap();
         // Ref should be created only for the head commit
         let git_refs = backend
             .git_repo()
@@ -1671,11 +1653,7 @@ mod tests {
             commit.root_tree.to_merge(),
             Merge::resolved(TreeId::from_bytes(root_tree_id.as_bytes()))
         );
-        if uses_tree_conflict_format {
-            assert_matches!(commit.root_tree, MergedTreeId::Merge(_));
-        } else {
-            assert_matches!(commit.root_tree, MergedTreeId::Legacy(_));
-        }
+        assert_matches!(commit.root_tree, MergedTreeId::Merge(_));
         assert_eq!(commit.description, "git commit message");
         assert_eq!(commit.author.name, "git author");
         assert_eq!(commit.author.email, "git.author@example.com");
@@ -1740,11 +1718,7 @@ mod tests {
             commit.root_tree.to_merge(),
             Merge::resolved(TreeId::from_bytes(root_tree_id.as_bytes()))
         );
-        if uses_tree_conflict_format {
-            assert_matches!(commit.root_tree, MergedTreeId::Merge(_));
-        } else {
-            assert_matches!(commit.root_tree, MergedTreeId::Legacy(_));
-        }
+        assert_matches!(commit.root_tree, MergedTreeId::Merge(_));
     }
 
     #[test]
@@ -2162,7 +2136,7 @@ mod tests {
         let settings = user_settings();
         let temp_dir = new_temp_dir();
         let backend = GitBackend::init_internal(&settings, temp_dir.path()).unwrap();
-        let mut commit1 = Commit {
+        let commit1 = Commit {
             parents: vec![backend.root_commit_id().clone()],
             predecessors: vec![],
             root_tree: MergedTreeId::Legacy(backend.empty_tree_id().clone()),
@@ -2177,10 +2151,6 @@ mod tests {
             backend.write_commit(commit, None).block_on()
         };
 
-        // libgit2 doesn't seem to preserve negative timestamps, so set it to at least 1
-        // second after the epoch, so the timestamp adjustment can remove 1
-        // second and it will still be nonnegative
-        commit1.committer.timestamp.timestamp = MillisSinceEpoch(1000);
         let (commit_id1, mut commit2) = write_commit(commit1).unwrap();
         commit2.predecessors.push(commit_id1.clone());
         // `write_commit` should prevent the ids from being the same by changing the
@@ -2250,17 +2220,17 @@ mod tests {
         let sig = commit.secure_sig.expect("failed to read the signature");
         assert_eq!(&sig, &returned_sig);
 
-        insta::assert_snapshot!(std::str::from_utf8(&sig.sig).unwrap(), @r###"
+        insta::assert_snapshot!(std::str::from_utf8(&sig.sig).unwrap(), @r"
         test sig
         hash=9ad9526c3b2103c41a229f2f3c82d107a0ecd902f476a855f0e1dd5f7bef1430663de12749b73e293a877113895a8a2a0f29da4bbc5a5f9a19c3523fb0e53518
-        "###);
-        insta::assert_snapshot!(std::str::from_utf8(&sig.data).unwrap(), @r###"
+        ");
+        insta::assert_snapshot!(std::str::from_utf8(&sig.data).unwrap(), @r"
         tree 4b825dc642cb6eb9a060e54bf8d69288fbee4904
         author Someone <someone@example.com> 0 +0000
         committer Someone <someone@example.com> 0 +0000
 
         initial
-        "###);
+        ");
     }
 
     fn git_id(commit_id: &CommitId) -> gix::ObjectId {
