@@ -24,14 +24,27 @@ use std::process::Output;
 use std::process::Stdio;
 use std::thread;
 
-use bstr::ByteSlice;
+use bstr::ByteSlice as _;
+use itertools::Itertools as _;
 use thiserror::Error;
 
+use crate::git::GitPushStats;
 use crate::git::Progress;
 use crate::git::RefSpec;
 use crate::git::RefToPush;
 use crate::git::RemoteCallbacks;
 use crate::git_backend::GitBackend;
+use crate::ref_name::GitRefNameBuf;
+use crate::ref_name::RefNameBuf;
+use crate::ref_name::RemoteName;
+
+// This is not the minimum required version, that would be 2.29.0, which
+// introduced the `--no-write-fetch-head` option. However, that by itself
+// is quite old and unsupported, so we don't want to encourage users to
+// update to that.
+//
+// 2.40 still receives security patches (latest one was in Jan/2025)
+const MINIMUM_GIT_VERSION: &str = "2.40.4";
 
 /// Error originating by a Git subprocess
 #[derive(Error, Debug)]
@@ -52,6 +65,11 @@ pub enum GitSubprocessError {
     },
     #[error("Failed to wait for the git process")]
     Wait(std::io::Error),
+    #[error(
+        "Git does not recognize required option: {0} (note: supported version is \
+         {MINIMUM_GIT_VERSION})"
+    )]
+    UnsupportedGitOption(String),
     #[error("Git process failed: {0}")]
     External(String),
 }
@@ -80,6 +98,14 @@ impl<'a> GitSubprocessContext<'a> {
     /// Create the Git command
     fn create_command(&self) -> Command {
         let mut git_cmd = Command::new(self.git_executable_path);
+        // Hide console window on Windows (https://stackoverflow.com/a/60958956)
+        #[cfg(windows)]
+        {
+            use std::os::windows::process::CommandExt;
+            const CREATE_NO_WINDOW: u32 = 0x08000000;
+            git_cmd.creation_flags(CREATE_NO_WINDOW);
+        }
+
         // TODO: here we are passing the full path to the git_dir, which can lead to UNC
         // bugs in Windows. The ideal way to do this is to pass the workspace
         // root to Command::current_dir and then pass a relative path to the git
@@ -120,7 +146,7 @@ impl<'a> GitSubprocessContext<'a> {
     /// Note that git only returns one failed ref at a time
     pub(crate) fn spawn_fetch(
         &self,
-        remote_name: &str,
+        remote_name: &RemoteName,
         refspecs: &[RefSpec],
         callbacks: &mut RemoteCallbacks<'_>,
         depth: Option<NonZeroU32>,
@@ -139,7 +165,7 @@ impl<'a> GitSubprocessContext<'a> {
         if let Some(d) = depth {
             command.arg(format!("--depth={d}"));
         }
-        command.arg("--").arg(remote_name);
+        command.arg("--").arg(remote_name.as_str());
         command.args(refspecs.iter().map(|x| x.to_git_format()));
 
         let output = wait_with_progress(self.spawn_cmd(command)?, callbacks)?;
@@ -155,6 +181,7 @@ impl<'a> GitSubprocessContext<'a> {
         if branches_to_prune.is_empty() {
             return Ok(());
         }
+        tracing::debug!(?branches_to_prune, "pruning branches");
         let mut command = self.create_command();
         command.stdout(Stdio::null());
         command.args(["branch", "--remotes", "--delete", "--"]);
@@ -176,17 +203,18 @@ impl<'a> GitSubprocessContext<'a> {
     /// `  HEAD branch: <default_branch>`
     pub(crate) fn spawn_remote_show(
         &self,
-        remote_name: &str,
-    ) -> Result<Option<String>, GitSubprocessError> {
+        remote_name: &RemoteName,
+    ) -> Result<Option<RefNameBuf>, GitSubprocessError> {
         let mut command = self.create_command();
         command.stdout(Stdio::piped());
-        command.args(["remote", "show", "--", remote_name]);
+        command.args(["remote", "show", "--", remote_name.as_str()]);
         let output = wait_with_output(self.spawn_cmd(command)?)?;
 
         let output = parse_git_remote_show_output(output)?;
 
         // find the HEAD branch line in the output
-        parse_git_remote_show_default_branch(&output.stdout)
+        let maybe_branch = parse_git_remote_show_default_branch(&output.stdout)?;
+        Ok(maybe_branch.map(Into::into))
     }
 
     /// Push references to git
@@ -199,10 +227,10 @@ impl<'a> GitSubprocessContext<'a> {
     ///     2. refs that succeeded to push
     pub(crate) fn spawn_push(
         &self,
-        remote_name: &str,
+        remote_name: &RemoteName,
         references: &[RefToPush],
         callbacks: &mut RemoteCallbacks<'_>,
-    ) -> Result<(Vec<String>, Vec<String>), GitSubprocessError> {
+    ) -> Result<GitPushStats, GitSubprocessError> {
         let mut command = self.create_command();
         command.stdout(Stdio::piped());
         // Currently jj does not support commit hooks, so we prevent git from running
@@ -219,7 +247,7 @@ impl<'a> GitSubprocessContext<'a> {
                 .iter()
                 .map(|reference| format!("--force-with-lease={}", reference.to_git_lease())),
         );
-        command.args(["--", remote_name]);
+        command.args(["--", remote_name.as_str()]);
         // with --force-with-lease we cannot have the forced refspec,
         // as it ignores the lease
         command.args(
@@ -305,6 +333,22 @@ fn parse_no_remote_tracking_branch(stderr: &[u8]) -> Option<String> {
         .map(|branch| branch.to_str_lossy().into_owned())
 }
 
+/// Parse unknown options
+///
+/// Return the unknown option
+///
+/// If a user is running a very old git version, our commands may fail
+/// We want to give a good error in this case
+fn parse_unknown_option(stderr: &[u8]) -> Option<String> {
+    let first_line = stderr.lines().next()?;
+    first_line
+        .strip_prefix(b"unknown option: --")
+        .or(first_line
+            .strip_prefix(b"error: unknown option `")
+            .and_then(|s| s.strip_suffix(b"'")))
+        .map(|s| s.to_str_lossy().into())
+}
+
 // return the fully qualified ref that failed to fetch
 //
 // note that git fetch only returns one error at a time
@@ -314,6 +358,10 @@ fn parse_git_fetch_output(output: Output) -> Result<Option<String>, GitSubproces
     }
 
     // There are some git errors we want to parse out
+    if let Some(option) = parse_unknown_option(&output.stderr) {
+        return Err(GitSubprocessError::UnsupportedGitOption(option));
+    }
+
     if let Some(remote) = parse_no_such_remote(&output.stderr) {
         return Err(GitSubprocessError::NoSuchRepository(remote));
     }
@@ -335,6 +383,10 @@ fn parse_git_branch_prune_output(output: Output) -> Result<(), GitSubprocessErro
     }
 
     // There are some git errors we want to parse out
+    if let Some(option) = parse_unknown_option(&output.stderr) {
+        return Err(GitSubprocessError::UnsupportedGitOption(option));
+    }
+
     if parse_no_remote_tracking_branch(&output.stderr).is_some() {
         return Ok(());
     }
@@ -348,6 +400,10 @@ fn parse_git_remote_show_output(output: Output) -> Result<Output, GitSubprocessE
     }
 
     // There are some git errors we want to parse out
+    if let Some(option) = parse_unknown_option(&output.stderr) {
+        return Err(GitSubprocessError::UnsupportedGitOption(option));
+    }
+
     if let Some(remote) = parse_no_such_remote(&output.stderr) {
         return Err(GitSubprocessError::NoSuchRepository(remote));
     }
@@ -362,6 +418,7 @@ fn parse_git_remote_show_default_branch(
         .lines()
         .map(|x| x.trim())
         .find(|x| x.starts_with_str("HEAD branch:"))
+        .inspect(|x| tracing::debug!(line = ?x.to_str_lossy(), "default branch"))
         .and_then(|x| x.split_str(" ").last().map(|y| y.trim()))
         .filter(|branch_name| branch_name != b"(unknown)")
         .map(|branch_name| branch_name.to_str())
@@ -371,7 +428,7 @@ fn parse_git_remote_show_default_branch(
 }
 
 // git-push porcelain has the following format (per line)
-// `<flag>\t<from>:<to>\t<summary>\t(<reason>)`
+// `<flag>\t<from>:<to>\t<summary> (<reason>)`
 //
 // <flag> is one of:
 //     ' ' for a successfully pushed fast-forward;
@@ -384,10 +441,9 @@ fn parse_git_remote_show_default_branch(
 // <from>:<to> is the refspec
 //
 // <summary> is extra info (commit ranges or reason for rejected)
-// at times the summary is omitted
 //
 // <reason> is a human-readable explanation
-fn parse_ref_pushes(stdout: &[u8]) -> Result<(Vec<String>, Vec<String>), GitSubprocessError> {
+fn parse_ref_pushes(stdout: &[u8]) -> Result<GitPushStats, GitSubprocessError> {
     if !stdout.starts_with(b"To ") {
         return Err(GitSubprocessError::External(format!(
             "Git push output unfamiliar:\n{}",
@@ -395,33 +451,20 @@ fn parse_ref_pushes(stdout: &[u8]) -> Result<(Vec<String>, Vec<String>), GitSubp
         )));
     }
 
-    let mut pushed_refs = Vec::new();
-    let mut rejected_refs = Vec::new();
+    let mut push_stats = GitPushStats::default();
     for (idx, line) in stdout
         .lines()
         .skip(1)
         .take_while(|line| line != b"Done")
         .enumerate()
     {
-        // format:
-        // <flag>\t<ref>\t<summary>\t<comment>
-        // sometimes the summary is omitted
-        let create_error = || {
+        tracing::debug!("response #{idx}: {}", line.to_str_lossy());
+        let (flag, reference, summary) = line.split_str("\t").collect_tuple().ok_or_else(|| {
             GitSubprocessError::External(format!(
                 "Line #{idx} of git-push has unknown format: {}",
                 line.to_str_lossy()
             ))
-        };
-        let mut it = line.split_str("\t").fuse();
-        let flag = it.next().ok_or_else(create_error)?;
-        let reference = it.next().ok_or_else(create_error)?;
-        // we capture the remaining elements to ensure the line is well formed
-        let _summary_or_comment = it.next().ok_or_else(create_error)?;
-        let _comment_opt = it.next();
-        if it.next().is_some() {
-            return Err(create_error());
-        }
-
+        })?;
         let full_refspec = reference
             .to_str()
             .map_err(|e| {
@@ -434,9 +477,9 @@ fn parse_ref_pushes(stdout: &[u8]) -> Result<(Vec<String>, Vec<String>), GitSubp
             })
             .map_err(GitSubprocessError::External)?;
 
-        let reference = full_refspec
+        let reference: GitRefNameBuf = full_refspec
             .split_once(':')
-            .map(|(_refname, reference)| reference.to_string())
+            .map(|(_refname, reference)| reference.into())
             .ok_or_else(|| {
                 GitSubprocessError::External(format!(
                     "Line #{idx} of git-push has full refspec without named ref: {full_refspec}"
@@ -450,11 +493,24 @@ fn parse_ref_pushes(stdout: &[u8]) -> Result<(Vec<String>, Vec<String>), GitSubp
             //  * for a successfully pushed new ref
             //  =  for a ref that was up to date and did not need pushing.
             b"+" | b"-" | b"*" | b"=" | b" " => {
-                pushed_refs.push(reference);
+                push_stats.pushed.push(reference);
             }
             // ! for a ref that was rejected or failed to push; and
             b"!" => {
-                rejected_refs.push(reference);
+                if let Some(reason) = summary.strip_prefix(b"[remote rejected]") {
+                    let reason = reason
+                        .strip_prefix(b" (")
+                        .and_then(|r| r.strip_suffix(b")"))
+                        .map(|x| x.to_str_lossy().into_owned());
+                    push_stats.remote_rejected.push((reference, reason));
+                } else {
+                    let reason = summary
+                        .split_once_str("]")
+                        .and_then(|(_, reason)| reason.strip_prefix(b" ("))
+                        .and_then(|r| r.strip_suffix(b")"))
+                        .map(|x| x.to_str_lossy().into_owned());
+                    push_stats.rejected.push((reference, reason));
+                }
             }
             unknown => {
                 return Err(GitSubprocessError::External(format!(
@@ -467,18 +523,22 @@ fn parse_ref_pushes(stdout: &[u8]) -> Result<(Vec<String>, Vec<String>), GitSubp
         }
     }
 
-    Ok((rejected_refs, pushed_refs))
+    Ok(push_stats)
 }
 
 // on Ok, return a tuple with
 //  1. list of failed references from test and set
 //  2. list of successful references pushed
-fn parse_git_push_output(output: Output) -> Result<(Vec<String>, Vec<String>), GitSubprocessError> {
+fn parse_git_push_output(output: Output) -> Result<GitPushStats, GitSubprocessError> {
     if output.status.success() {
         let ref_pushes = parse_ref_pushes(&output.stdout)?;
         return Ok(ref_pushes);
     }
 
+    if let Some(option) = parse_unknown_option(&output.stderr) {
+        return Err(GitSubprocessError::UnsupportedGitOption(option));
+    }
+
     if let Some(remote) = parse_no_such_remote(&output.stderr) {
         return Err(GitSubprocessError::NoSuchRepository(remote));
     }
@@ -700,12 +760,15 @@ and the repository exists. "###;
     const SAMPLE_NO_REMOTE_TRACKING_BRANCH_ERROR: &[u8] =
         b"error: remote-tracking branch 'bookmark' not found";
     const SAMPLE_PUSH_REFS_PORCELAIN_OUTPUT: &[u8] = b"To origin
-*\tdeadbeef:refs/heads/bookmark1\tdeadbeef\t[new branch]
-+\tdeadbeef:refs/heads/bookmark2\tdeadbeef\t[new branch]
--\tdeadbeef:refs/heads/bookmark3\tdeadbeef\t[new branch]
- \tdeadbeef:refs/heads/bookmark4\tdeadbeef\t[new branch]
-=\tdeadbeef:refs/heads/bookmark5\tdeadbeef\t[new branch]
-!\tdeadbeef:refs/heads/bookmark6\tdeadbeef\t[new branch]
+*\tdeadbeef:refs/heads/bookmark1\t[new branch]
++\tdeadbeef:refs/heads/bookmark2\tabcd..dead
+-\tdeadbeef:refs/heads/bookmark3\t[deleted branch]
+ \tdeadbeef:refs/heads/bookmark4\tabcd..dead
+=\tdeadbeef:refs/heads/bookmark5\tabcd..abcd
+!\tdeadbeef:refs/heads/bookmark6\t[rejected] (failure lease)
+!\tdeadbeef:refs/heads/bookmark7\t[rejected]
+!\tdeadbeef:refs/heads/bookmark8\t[remote rejected] (hook failure)
+!\tdeadbeef:refs/heads/bookmark9\t[remote rejected]
 Done";
     const SAMPLE_OK_STDERR: &[u8] = b"";
 
@@ -778,16 +841,40 @@ Done";
         assert!(parse_ref_pushes(SAMPLE_NO_SUCH_REMOTE_ERROR).is_err());
         assert!(parse_ref_pushes(SAMPLE_NO_REMOTE_REF_ERROR).is_err());
         assert!(parse_ref_pushes(SAMPLE_NO_REMOTE_TRACKING_BRANCH_ERROR).is_err());
-        let (failed, success) = parse_ref_pushes(SAMPLE_PUSH_REFS_PORCELAIN_OUTPUT).unwrap();
-        assert_eq!(failed, vec!["refs/heads/bookmark6".to_string()]);
+        let GitPushStats {
+            pushed,
+            rejected,
+            remote_rejected,
+        } = parse_ref_pushes(SAMPLE_PUSH_REFS_PORCELAIN_OUTPUT).unwrap();
         assert_eq!(
-            success,
+            pushed,
+            [
+                "refs/heads/bookmark1",
+                "refs/heads/bookmark2",
+                "refs/heads/bookmark3",
+                "refs/heads/bookmark4",
+                "refs/heads/bookmark5",
+            ]
+            .map(GitRefNameBuf::from)
+        );
+        assert_eq!(
+            rejected,
             vec![
-                "refs/heads/bookmark1".to_string(),
-                "refs/heads/bookmark2".to_string(),
-                "refs/heads/bookmark3".to_string(),
-                "refs/heads/bookmark4".to_string(),
-                "refs/heads/bookmark5".to_string(),
+                (
+                    "refs/heads/bookmark6".into(),
+                    Some("failure lease".to_string())
+                ),
+                ("refs/heads/bookmark7".into(), None),
+            ]
+        );
+        assert_eq!(
+            remote_rejected,
+            vec![
+                (
+                    "refs/heads/bookmark8".into(),
+                    Some("hook failure".to_string())
+                ),
+                ("refs/heads/bookmark9".into(), None)
             ]
         );
         assert!(parse_ref_pushes(SAMPLE_OK_STDERR).is_err());
@@ -859,4 +946,17 @@ Done";
         );
         assert_eq!(read_progress_line(b"fatal: this is a git error\n"), None);
     }
+
+    #[test]
+    fn test_parse_unknown_option() {
+        assert_eq!(
+            parse_unknown_option(b"unknown option: --abc").unwrap(),
+            "abc".to_string()
+        );
+        assert_eq!(
+            parse_unknown_option(b"error: unknown option `abc'").unwrap(),
+            "abc".to_string()
+        );
+        assert!(parse_unknown_option(b"error: unknown option: 'abc'").is_none());
+    }
 }
