@@ -14,9 +14,6 @@
 
 #![allow(missing_docs)]
 
-use std::fmt;
-use std::fmt::Display;
-
 use itertools::EitherOrBoth;
 
 use crate::backend::CommitId;
@@ -25,7 +22,6 @@ use crate::merge::trivial_merge;
 use crate::merge::Merge;
 use crate::op_store::RefTarget;
 use crate::op_store::RemoteRef;
-use crate::revset;
 
 /// Compares `refs1` and `refs2` targets, yields entry if they differ.
 ///
@@ -74,6 +70,22 @@ pub fn iter_named_local_remote_refs<'a, 'b, K: Ord>(
     )
 }
 
+/// Compares `ids1` and `ids2` commit ids, yields entry if they differ.
+///
+/// `ids1` and `ids2` must be sorted by `K`.
+pub fn diff_named_commit_ids<'a, 'b, K: Ord>(
+    ids1: impl IntoIterator<Item = (K, &'a CommitId)>,
+    ids2: impl IntoIterator<Item = (K, &'b CommitId)>,
+) -> impl Iterator<Item = (K, (Option<&'a CommitId>, Option<&'b CommitId>))> {
+    iter_named_pairs(
+        ids1.into_iter().map(|(k, v)| (k, Some(v))),
+        ids2.into_iter().map(|(k, v)| (k, Some(v))),
+        || None,
+        || None,
+    )
+    .filter(|(_, (target1, target2))| target1 != target2)
+}
+
 fn iter_named_pairs<K: Ord, V1, V2>(
     refs1: impl IntoIterator<Item = (K, V1)>,
     refs2: impl IntoIterator<Item = (K, V2)>,
@@ -171,93 +183,6 @@ fn find_pair_to_remove(
     None
 }
 
-/// Owned remote bookmark or tag name.
-///
-/// This type can be displayed in `{name}@{remote}` form, with quoting and
-/// escaping if necessary.
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct RemoteRefSymbolBuf {
-    /// Local name.
-    pub name: String,
-    /// Remote name.
-    pub remote: String,
-}
-
-impl RemoteRefSymbolBuf {
-    /// Converts to reference type.
-    pub fn as_ref(&self) -> RemoteRefSymbol<'_> {
-        RemoteRefSymbol {
-            name: &self.name,
-            remote: &self.remote,
-        }
-    }
-}
-
-/// Borrowed remote bookmark or tag name.
-///
-/// This type can be displayed in `{name}@{remote}` form, with quoting and
-/// escaping if necessary.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct RemoteRefSymbol<'a> {
-    /// Local name.
-    pub name: &'a str,
-    /// Remote name.
-    pub remote: &'a str,
-}
-
-impl RemoteRefSymbol<'_> {
-    /// Converts to owned type.
-    pub fn to_owned(self) -> RemoteRefSymbolBuf {
-        RemoteRefSymbolBuf {
-            name: self.name.to_owned(),
-            remote: self.remote.to_owned(),
-        }
-    }
-}
-
-impl From<RemoteRefSymbol<'_>> for RemoteRefSymbolBuf {
-    fn from(value: RemoteRefSymbol<'_>) -> Self {
-        value.to_owned()
-    }
-}
-
-impl PartialEq<RemoteRefSymbol<'_>> for RemoteRefSymbolBuf {
-    fn eq(&self, other: &RemoteRefSymbol) -> bool {
-        self.as_ref() == *other
-    }
-}
-
-impl PartialEq<RemoteRefSymbol<'_>> for &RemoteRefSymbolBuf {
-    fn eq(&self, other: &RemoteRefSymbol) -> bool {
-        self.as_ref() == *other
-    }
-}
-
-impl PartialEq<RemoteRefSymbolBuf> for RemoteRefSymbol<'_> {
-    fn eq(&self, other: &RemoteRefSymbolBuf) -> bool {
-        *self == other.as_ref()
-    }
-}
-
-impl PartialEq<&RemoteRefSymbolBuf> for RemoteRefSymbol<'_> {
-    fn eq(&self, other: &&RemoteRefSymbolBuf) -> bool {
-        *self == other.as_ref()
-    }
-}
-
-impl Display for RemoteRefSymbolBuf {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        Display::fmt(&self.as_ref(), f)
-    }
-}
-
-impl Display for RemoteRefSymbol<'_> {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        let RemoteRefSymbol { name, remote } = self;
-        f.pad(&revset::format_remote_symbol(name, remote))
-    }
-}
-
 /// Pair of local and remote targets.
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub struct LocalAndRemoteRef<'a> {
@@ -284,14 +209,14 @@ pub enum BookmarkPushAction {
 /// this bookmark.
 pub fn classify_bookmark_push_action(targets: LocalAndRemoteRef) -> BookmarkPushAction {
     let local_target = targets.local_target;
-    let remote_target = targets.remote_ref.tracking_target();
+    let remote_target = targets.remote_ref.tracked_target();
     if local_target == remote_target {
         BookmarkPushAction::AlreadyMatches
     } else if local_target.has_conflict() {
         BookmarkPushAction::LocalConflicted
     } else if remote_target.has_conflict() {
         BookmarkPushAction::RemoteConflicted
-    } else if targets.remote_ref.is_present() && !targets.remote_ref.is_tracking() {
+    } else if targets.remote_ref.is_present() && !targets.remote_ref.is_tracked() {
         BookmarkPushAction::RemoteUntracked
     } else {
         BookmarkPushAction::Update(BookmarkPushUpdate {
@@ -313,10 +238,10 @@ mod tests {
         }
     }
 
-    fn tracking_remote_ref(target: RefTarget) -> RemoteRef {
+    fn tracked_remote_ref(target: RefTarget) -> RemoteRef {
         RemoteRef {
             target,
-            state: RemoteRefState::Tracking,
+            state: RemoteRefState::Tracked,
         }
     }
 
@@ -325,7 +250,7 @@ mod tests {
         let commit_id1 = CommitId::from_hex("11");
         let targets = LocalAndRemoteRef {
             local_target: &RefTarget::normal(commit_id1.clone()),
-            remote_ref: &tracking_remote_ref(RefTarget::normal(commit_id1)),
+            remote_ref: &tracked_remote_ref(RefTarget::normal(commit_id1)),
         };
         assert_eq!(
             classify_bookmark_push_action(targets),
@@ -354,7 +279,7 @@ mod tests {
         let commit_id1 = CommitId::from_hex("11");
         let targets = LocalAndRemoteRef {
             local_target: RefTarget::absent_ref(),
-            remote_ref: &tracking_remote_ref(RefTarget::normal(commit_id1.clone())),
+            remote_ref: &tracked_remote_ref(RefTarget::normal(commit_id1.clone())),
         };
         assert_eq!(
             classify_bookmark_push_action(targets),
@@ -371,7 +296,7 @@ mod tests {
         let commit_id2 = CommitId::from_hex("22");
         let targets = LocalAndRemoteRef {
             local_target: &RefTarget::normal(commit_id2.clone()),
-            remote_ref: &tracking_remote_ref(RefTarget::normal(commit_id1.clone())),
+            remote_ref: &tracked_remote_ref(RefTarget::normal(commit_id1.clone())),
         };
         assert_eq!(
             classify_bookmark_push_action(targets),
@@ -417,7 +342,7 @@ mod tests {
         let commit_id2 = CommitId::from_hex("22");
         let targets = LocalAndRemoteRef {
             local_target: &RefTarget::from_legacy_form([], [commit_id1.clone(), commit_id2]),
-            remote_ref: &tracking_remote_ref(RefTarget::normal(commit_id1)),
+            remote_ref: &tracked_remote_ref(RefTarget::normal(commit_id1)),
         };
         assert_eq!(
             classify_bookmark_push_action(targets),
@@ -431,7 +356,7 @@ mod tests {
         let commit_id2 = CommitId::from_hex("22");
         let targets = LocalAndRemoteRef {
             local_target: &RefTarget::normal(commit_id1.clone()),
-            remote_ref: &tracking_remote_ref(RefTarget::from_legacy_form(
+            remote_ref: &tracked_remote_ref(RefTarget::from_legacy_form(
                 [],
                 [commit_id1, commit_id2],
             )),
