@@ -24,7 +24,7 @@ use std::path::Path;
 use std::slice;
 use std::sync::Arc;
 
-use itertools::Itertools;
+use itertools::Itertools as _;
 use once_cell::sync::OnceCell;
 use thiserror::Error;
 use tracing::instrument;
@@ -54,10 +54,9 @@ use crate::index::IndexReadError;
 use crate::index::IndexStore;
 use crate::index::MutableIndex;
 use crate::index::ReadonlyIndex;
-use crate::local_backend::LocalBackend;
 use crate::merge::MergeBuilder;
 use crate::object_id::HexPrefix;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::object_id::PrefixResolution;
 use crate::op_heads_store;
 use crate::op_heads_store::OpHeadResolutionError;
@@ -72,16 +71,20 @@ use crate::op_store::RefTarget;
 use crate::op_store::RemoteRef;
 use crate::op_store::RemoteRefState;
 use crate::op_store::RootOperationData;
-use crate::op_store::WorkspaceId;
 use crate::operation::Operation;
+use crate::ref_name::GitRefName;
+use crate::ref_name::RefName;
+use crate::ref_name::RemoteName;
+use crate::ref_name::RemoteRefSymbol;
+use crate::ref_name::WorkspaceName;
+use crate::ref_name::WorkspaceNameBuf;
 use crate::refs::diff_named_ref_targets;
 use crate::refs::diff_named_remote_refs;
 use crate::refs::merge_ref_targets;
 use crate::refs::merge_remote_refs;
-use crate::refs::RemoteRefSymbol;
 use crate::revset;
 use crate::revset::RevsetExpression;
-use crate::revset::RevsetIteratorExt;
+use crate::revset::RevsetIteratorExt as _;
 use crate::rewrite::merge_commit_trees;
 use crate::rewrite::rebase_commit_with_options;
 use crate::rewrite::CommitRewriter;
@@ -91,6 +94,7 @@ use crate::rewrite::RewriteRefsOptions;
 use crate::settings::UserSettings;
 use crate::signing::SignInitError;
 use crate::signing::Signer;
+use crate::simple_backend::SimpleBackend;
 use crate::simple_op_heads_store::SimpleOpHeadsStore;
 use crate::simple_op_store::SimpleOpStore;
 use crate::store::Store;
@@ -175,7 +179,7 @@ impl ReadonlyRepo {
         &|_settings, store_path| Ok(Box::new(DefaultSubmoduleStore::init(store_path)))
     }
 
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub fn init(
         settings: &UserSettings,
         repo_path: &Path,
@@ -397,8 +401,8 @@ impl Default for StoreFactories {
 
         // Backends
         factories.add_backend(
-            LocalBackend::name(),
-            Box::new(|_settings, store_path| Ok(Box::new(LocalBackend::load(store_path)))),
+            SimpleBackend::name(),
+            Box::new(|_settings, store_path| Ok(Box::new(SimpleBackend::load(store_path)))),
         );
         #[cfg(feature = "git")]
         factories.add_backend(
@@ -1120,13 +1124,13 @@ impl MutableRepo {
             .view()
             .wc_commit_ids()
             .iter()
-            .filter_map(|(workspace_id, commit_id)| {
+            .filter_map(|(name, commit_id)| {
                 let change = rewrite_mapping.get_key_value(commit_id)?;
-                Some((workspace_id.to_owned(), change))
+                Some((name.to_owned(), change))
             })
             .collect_vec();
         let mut recreated_wc_commits: HashMap<&CommitId, Commit> = HashMap::new();
-        for (workspace_id, (old_commit_id, new_commit_ids)) in changed_wc_commits {
+        for (name, (old_commit_id, new_commit_ids)) in changed_wc_commits {
             let abandoned_old_commit = matches!(
                 self.parent_mapping.get(old_commit_id),
                 Some(Rewrite::Abandoned(_))
@@ -1148,7 +1152,7 @@ impl MutableRepo {
                 recreated_wc_commits.insert(old_commit_id, commit.clone());
                 commit
             };
-            self.edit(workspace_id, &new_wc_commit).unwrap();
+            self.edit(name, &new_wc_commit).unwrap();
         }
         Ok(())
     }
@@ -1349,61 +1353,56 @@ impl MutableRepo {
 
     pub fn set_wc_commit(
         &mut self,
-        workspace_id: WorkspaceId,
+        name: WorkspaceNameBuf,
         commit_id: CommitId,
     ) -> Result<(), RewriteRootCommit> {
         if &commit_id == self.store().root_commit_id() {
             return Err(RewriteRootCommit);
         }
-        self.view_mut().set_wc_commit(workspace_id, commit_id);
+        self.view_mut().set_wc_commit(name, commit_id);
         Ok(())
     }
 
-    pub fn remove_wc_commit(&mut self, workspace_id: &WorkspaceId) -> Result<(), EditCommitError> {
-        self.maybe_abandon_wc_commit(workspace_id)?;
-        self.view_mut().remove_wc_commit(workspace_id);
+    pub fn remove_wc_commit(&mut self, name: &WorkspaceName) -> Result<(), EditCommitError> {
+        self.maybe_abandon_wc_commit(name)?;
+        self.view_mut().remove_wc_commit(name);
         Ok(())
     }
 
     pub fn rename_workspace(
         &mut self,
-        old_workspace_id: &WorkspaceId,
-        new_workspace_id: WorkspaceId,
+        old_name: &WorkspaceName,
+        new_name: WorkspaceNameBuf,
     ) -> Result<(), RenameWorkspaceError> {
-        self.view_mut()
-            .rename_workspace(old_workspace_id, new_workspace_id)
+        self.view_mut().rename_workspace(old_name, new_name)
     }
 
     pub fn check_out(
         &mut self,
-        workspace_id: WorkspaceId,
+        name: WorkspaceNameBuf,
         commit: &Commit,
     ) -> Result<Commit, CheckOutCommitError> {
         let wc_commit = self
             .new_commit(vec![commit.id().clone()], commit.tree_id().clone())
             .write()?;
-        self.edit(workspace_id, &wc_commit)?;
+        self.edit(name, &wc_commit)?;
         Ok(wc_commit)
     }
 
-    pub fn edit(
-        &mut self,
-        workspace_id: WorkspaceId,
-        commit: &Commit,
-    ) -> Result<(), EditCommitError> {
-        self.maybe_abandon_wc_commit(&workspace_id)?;
+    pub fn edit(&mut self, name: WorkspaceNameBuf, commit: &Commit) -> Result<(), EditCommitError> {
+        self.maybe_abandon_wc_commit(&name)?;
         self.add_head(commit)?;
-        Ok(self.set_wc_commit(workspace_id, commit.id().clone())?)
+        Ok(self.set_wc_commit(name, commit.id().clone())?)
     }
 
     fn maybe_abandon_wc_commit(
         &mut self,
-        workspace_id: &WorkspaceId,
+        workspace_name: &WorkspaceName,
     ) -> Result<(), EditCommitError> {
         let is_commit_referenced = |view: &View, commit_id: &CommitId| -> bool {
             view.wc_commit_ids()
                 .iter()
-                .filter(|&(ws_id, _)| ws_id != workspace_id)
+                .filter(|&(name, _)| name != workspace_name)
                 .map(|(_, wc_id)| wc_id)
                 .chain(
                     view.local_bookmarks()
@@ -1414,7 +1413,7 @@ impl MutableRepo {
 
         let maybe_wc_commit_id = self
             .view
-            .with_ref(|v| v.get_wc_commit_id(workspace_id).cloned());
+            .with_ref(|v| v.get_wc_commit_id(workspace_name).cloned());
         if let Some(wc_commit_id) = maybe_wc_commit_id {
             let wc_commit = self
                 .store()
@@ -1524,11 +1523,11 @@ impl MutableRepo {
         self.view.mark_dirty();
     }
 
-    pub fn get_local_bookmark(&self, name: &str) -> RefTarget {
+    pub fn get_local_bookmark(&self, name: &RefName) -> RefTarget {
         self.view.with_ref(|v| v.get_local_bookmark(name).clone())
     }
 
-    pub fn set_local_bookmark_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_local_bookmark_target(&mut self, name: &RefName, target: RefTarget) {
         let view = self.view_mut();
         for id in target.added_ids() {
             view.add_head(id);
@@ -1538,7 +1537,7 @@ impl MutableRepo {
 
     pub fn merge_local_bookmark(
         &mut self,
-        name: &str,
+        name: &RefName,
         base_target: &RefTarget,
         other_target: &RefTarget,
     ) {
@@ -1575,9 +1574,9 @@ impl MutableRepo {
     /// tracking it.
     pub fn track_remote_bookmark(&mut self, symbol: RemoteRefSymbol<'_>) {
         let mut remote_ref = self.get_remote_bookmark(symbol);
-        let base_target = remote_ref.tracking_target();
+        let base_target = remote_ref.tracked_target();
         self.merge_local_bookmark(symbol.name, base_target, &remote_ref.target);
-        remote_ref.state = RemoteRefState::Tracking;
+        remote_ref.state = RemoteRefState::Tracked;
         self.set_remote_bookmark(symbol, remote_ref);
     }
 
@@ -1588,23 +1587,23 @@ impl MutableRepo {
         self.set_remote_bookmark(symbol, remote_ref);
     }
 
-    pub fn remove_remote(&mut self, remote_name: &str) {
+    pub fn remove_remote(&mut self, remote_name: &RemoteName) {
         self.view_mut().remove_remote(remote_name);
     }
 
-    pub fn rename_remote(&mut self, old: &str, new: &str) {
+    pub fn rename_remote(&mut self, old: &RemoteName, new: &RemoteName) {
         self.view_mut().rename_remote(old, new);
     }
 
-    pub fn get_tag(&self, name: &str) -> RefTarget {
+    pub fn get_tag(&self, name: &RefName) -> RefTarget {
         self.view.with_ref(|v| v.get_tag(name).clone())
     }
 
-    pub fn set_tag_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_tag_target(&mut self, name: &RefName, target: RefTarget) {
         self.view_mut().set_tag_target(name, target);
     }
 
-    pub fn merge_tag(&mut self, name: &str, base_target: &RefTarget, other_target: &RefTarget) {
+    pub fn merge_tag(&mut self, name: &RefName, base_target: &RefTarget, other_target: &RefTarget) {
         let view = self.view.get_mut();
         let index = self.index.as_index();
         let self_target = view.get_tag(name);
@@ -1612,15 +1611,20 @@ impl MutableRepo {
         view.set_tag_target(name, new_target);
     }
 
-    pub fn get_git_ref(&self, name: &str) -> RefTarget {
+    pub fn get_git_ref(&self, name: &GitRefName) -> RefTarget {
         self.view.with_ref(|v| v.get_git_ref(name).clone())
     }
 
-    pub fn set_git_ref_target(&mut self, name: &str, target: RefTarget) {
+    pub fn set_git_ref_target(&mut self, name: &GitRefName, target: RefTarget) {
         self.view_mut().set_git_ref_target(name, target);
     }
 
-    fn merge_git_ref(&mut self, name: &str, base_target: &RefTarget, other_target: &RefTarget) {
+    fn merge_git_ref(
+        &mut self,
+        name: &GitRefName,
+        base_target: &RefTarget,
+        other_target: &RefTarget,
+    ) {
         let view = self.view.get_mut();
         let index = self.index.as_index();
         let self_target = view.get_git_ref(name);
@@ -1664,31 +1668,31 @@ impl MutableRepo {
     }
 
     fn merge_view(&mut self, base: &View, other: &View) -> BackendResult<()> {
+        // TODO: Use `diff_named_commit_ids` to simplify this.
         // Merge working-copy commits. If there's a conflict, we keep the self side.
-        for (workspace_id, base_wc_commit) in base.wc_commit_ids() {
-            let self_wc_commit = self.view().get_wc_commit_id(workspace_id);
-            let other_wc_commit = other.get_wc_commit_id(workspace_id);
+        for (name, base_wc_commit) in base.wc_commit_ids() {
+            let self_wc_commit = self.view().get_wc_commit_id(name);
+            let other_wc_commit = other.get_wc_commit_id(name);
             if other_wc_commit == Some(base_wc_commit) || other_wc_commit == self_wc_commit {
                 // The other side didn't change or both sides changed in the
                 // same way.
             } else if let Some(other_wc_commit) = other_wc_commit {
                 if self_wc_commit == Some(base_wc_commit) {
                     self.view_mut()
-                        .set_wc_commit(workspace_id.clone(), other_wc_commit.clone());
+                        .set_wc_commit(name.clone(), other_wc_commit.clone());
                 }
             } else {
                 // The other side removed the workspace. We want to remove it even if the self
                 // side changed the working-copy commit.
-                self.view_mut().remove_wc_commit(workspace_id);
+                self.view_mut().remove_wc_commit(name);
             }
         }
-        for (workspace_id, other_wc_commit) in other.wc_commit_ids() {
-            if self.view().get_wc_commit_id(workspace_id).is_none()
-                && base.get_wc_commit_id(workspace_id).is_none()
+        for (name, other_wc_commit) in other.wc_commit_ids() {
+            if self.view().get_wc_commit_id(name).is_none() && base.get_wc_commit_id(name).is_none()
             {
                 // The other side added the workspace.
                 self.view_mut()
-                    .set_wc_commit(workspace_id.clone(), other_wc_commit.clone());
+                    .set_wc_commit(name.clone(), other_wc_commit.clone());
             }
         }
         let base_heads = base.heads().iter().cloned().collect_vec();
