@@ -26,7 +26,7 @@ use pest::iterators::Pairs;
 use pest::pratt_parser::Assoc;
 use pest::pratt_parser::Op;
 use pest::pratt_parser::PrattParser;
-use pest::Parser;
+use pest::Parser as _;
 use pest_derive::Parser;
 use thiserror::Error;
 
@@ -45,7 +45,9 @@ use crate::dsl_util::FoldableExpression;
 use crate::dsl_util::FunctionCallParser;
 use crate::dsl_util::InvalidArguments;
 use crate::dsl_util::StringLiteralParser;
-use crate::refs::RemoteRefSymbolBuf;
+use crate::ref_name::RefNameBuf;
+use crate::ref_name::RemoteNameBuf;
+use crate::ref_name::RemoteRefSymbolBuf;
 
 #[derive(Parser)]
 #[grammar = "revset.pest"]
@@ -303,10 +305,10 @@ fn rename_rules_in_pest_error(mut err: pest::error::Error<Rule>) -> pest::error:
     // (positive) suggestion.
     let mut known_syms = HashSet::new();
     positives.retain(|rule| {
-        !rule.is_compat() && rule.to_symbol().map_or(true, |sym| known_syms.insert(sym))
+        !rule.is_compat() && rule.to_symbol().is_none_or(|sym| known_syms.insert(sym))
     });
     let mut known_syms = HashSet::new();
-    negatives.retain(|rule| rule.to_symbol().map_or(true, |sym| known_syms.insert(sym)));
+    negatives.retain(|rule| rule.to_symbol().is_none_or(|sym| known_syms.insert(sym)));
     err.renamed_rules(|rule| {
         rule.to_symbol()
             .map(|sym| format!("`{sym}`"))
@@ -327,7 +329,7 @@ pub enum ExpressionKind<'i> {
     },
     /// `<name>@<remote>`
     RemoteSymbol(RemoteRefSymbolBuf),
-    /// `<workspace_id>@`
+    /// `<name>@`
     AtWorkspace(String),
     /// `@`
     AtCurrentWorkspace,
@@ -647,11 +649,12 @@ fn parse_primary_node(pair: Pair<Rule>) -> Result<ExpressionNode, RevsetParseErr
                 Some(op) => {
                     assert_eq!(op.as_rule(), Rule::at_op);
                     match pairs.next() {
-                        // postfix "<workspace_id>@"
+                        // postfix "<name>@"
                         None => ExpressionKind::AtWorkspace(name),
                         // infix "<name>@<remote>"
                         Some(second) => {
-                            let remote = parse_as_string_literal(second);
+                            let name: RefNameBuf = name.into();
+                            let remote: RemoteNameBuf = parse_as_string_literal(second).into();
                             ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf { name, remote })
                         }
                     }
@@ -840,33 +843,46 @@ pub(super) fn expect_expression_with<T>(
 
 #[cfg(test)]
 mod tests {
+    use std::collections::HashMap;
+
     use assert_matches::assert_matches;
 
     use super::*;
     use crate::dsl_util::KeywordArgument;
 
     #[derive(Debug)]
-    struct WithRevsetAliasesMap(RevsetAliasesMap);
+    struct WithRevsetAliasesMap<'i> {
+        aliases_map: RevsetAliasesMap,
+        locals: HashMap<&'i str, ExpressionNode<'i>>,
+    }
 
-    impl WithRevsetAliasesMap {
-        fn parse<'i>(&'i self, text: &'i str) -> Result<ExpressionNode<'i>, RevsetParseError> {
-            let node = parse_program(text)?;
-            dsl_util::expand_aliases(node, &self.0)
+    impl<'i> WithRevsetAliasesMap<'i> {
+        fn set_local(mut self, name: &'i str, value: &'i str) -> Self {
+            self.locals.insert(name, parse_program(value).unwrap());
+            self
         }
 
-        fn parse_normalized<'i>(&'i self, text: &'i str) -> ExpressionNode<'i> {
+        fn parse(&'i self, text: &'i str) -> Result<ExpressionNode<'i>, RevsetParseError> {
+            let node = parse_program(text)?;
+            dsl_util::expand_aliases_with_locals(node, &self.aliases_map, &self.locals)
+        }
+
+        fn parse_normalized(&'i self, text: &'i str) -> ExpressionNode<'i> {
             normalize_tree(self.parse(text).unwrap())
         }
     }
 
-    fn with_aliases(
+    fn with_aliases<'i>(
         aliases: impl IntoIterator<Item = (impl AsRef<str>, impl Into<String>)>,
-    ) -> WithRevsetAliasesMap {
+    ) -> WithRevsetAliasesMap<'i> {
         let mut aliases_map = RevsetAliasesMap::new();
         for (decl, defn) in aliases {
             aliases_map.insert(decl, defn).unwrap();
         }
-        WithRevsetAliasesMap(aliases_map)
+        WithRevsetAliasesMap {
+            aliases_map,
+            locals: HashMap::new(),
+        }
     }
 
     fn parse_into_kind(text: &str) -> Result<ExpressionKind, RevsetParseErrorKind> {
@@ -1337,8 +1353,8 @@ mod tests {
         assert_eq!(
             parse_into_kind("main@origin"),
             Ok(ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf {
-                name: "main".to_owned(),
-                remote: "origin".to_owned()
+                name: "main".into(),
+                remote: "origin".into()
             }))
         );
 
@@ -1350,22 +1366,22 @@ mod tests {
         assert_eq!(
             parse_into_kind(r#""foo bar"@origin"#),
             Ok(ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf {
-                name: "foo bar".to_owned(),
-                remote: "origin".to_owned()
+                name: "foo bar".into(),
+                remote: "origin".into()
             }))
         );
         assert_eq!(
             parse_into_kind(r#"main@"foo bar""#),
             Ok(ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf {
-                name: "main".to_owned(),
-                remote: "foo bar".to_owned()
+                name: "main".into(),
+                remote: "foo bar".into()
             }))
         );
         assert_eq!(
             parse_into_kind(r#"'foo bar'@'bar baz'"#),
             Ok(ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf {
-                name: "foo bar".to_owned(),
-                remote: "bar baz".to_owned()
+                name: "foo bar".into(),
+                remote: "bar baz".into()
             }))
         );
 
@@ -1391,8 +1407,8 @@ mod tests {
         assert_eq!(
             parse_into_kind("柔@術"),
             Ok(ExpressionKind::RemoteSymbol(RemoteRefSymbolBuf {
-                name: "柔".to_owned(),
-                remote: "術".to_owned()
+                name: "柔".into(),
+                remote: "術".into()
             }))
         );
     }
@@ -1835,4 +1851,23 @@ mod tests {
             RevsetParseErrorKind::InAliasExpansion("F(x)".to_owned())
         );
     }
+
+    #[test]
+    fn test_expand_with_locals() {
+        // Local variable should precede the symbol alias.
+        assert_eq!(
+            with_aliases([("A", "symbol")])
+                .set_local("A", "local")
+                .parse_normalized("A"),
+            parse_normalized("local")
+        );
+
+        // Local variable shouldn't be expanded within aliases.
+        assert_eq!(
+            with_aliases([("B", "A"), ("F(x)", "x&A")])
+                .set_local("A", "a")
+                .parse_normalized("A|B|F(A)"),
+            parse_normalized("a|A|(a&A)")
+        );
+    }
 }
