@@ -17,8 +17,8 @@ use std::path::PathBuf;
 
 use jj_lib::config::StackedConfig;
 use jj_lib::git_backend::GitBackend;
-use jj_lib::op_store::WorkspaceId;
-use jj_lib::repo::Repo;
+use jj_lib::ref_name::WorkspaceName;
+use jj_lib::repo::Repo as _;
 use jj_lib::settings::UserSettings;
 use jj_lib::workspace::Workspace;
 use test_case::test_case;
@@ -38,7 +38,7 @@ fn test_init_local() {
     let settings = testutils::user_settings();
     let temp_dir = testutils::new_temp_dir();
     let (canonical, uncanonical) = canonicalize(temp_dir.path());
-    let (workspace, repo) = Workspace::init_local(&settings, &uncanonical).unwrap();
+    let (workspace, repo) = Workspace::init_simple(&settings, &uncanonical).unwrap();
     assert!(repo
         .store()
         .backend_impl()
@@ -138,7 +138,7 @@ fn test_init_external_git() {
     write_random_commit(tx.repo_mut());
 }
 
-#[test_case(TestRepoBackend::Local ; "local backend")]
+#[test_case(TestRepoBackend::Simple ; "simple backend")]
 #[test_case(TestRepoBackend::Git ; "git backend")]
 fn test_init_with_default_config(backend: TestRepoBackend) {
     // Test that we can create a repo without setting any non-default config
@@ -147,7 +147,7 @@ fn test_init_with_default_config(backend: TestRepoBackend) {
     let repo = &test_workspace.repo;
     let wc_commit_id = repo
         .view()
-        .get_wc_commit_id(&WorkspaceId::default())
+        .get_wc_commit_id(WorkspaceName::DEFAULT)
         .unwrap();
     let wc_commit = repo.store().get_commit(wc_commit_id).unwrap();
     assert_eq!(wc_commit.author().name, "".to_string());
@@ -156,7 +156,7 @@ fn test_init_with_default_config(backend: TestRepoBackend) {
     assert_eq!(wc_commit.committer().email, "".to_string());
 }
 
-#[test_case(TestRepoBackend::Local ; "local backend")]
+#[test_case(TestRepoBackend::Simple ; "simple backend")]
 #[test_case(TestRepoBackend::Git ; "git backend")]
 fn test_init_checkout(backend: TestRepoBackend) {
     // Test the contents of the working-copy commit after init
@@ -165,7 +165,7 @@ fn test_init_checkout(backend: TestRepoBackend) {
     let repo = &test_workspace.repo;
     let wc_commit_id = repo
         .view()
-        .get_wc_commit_id(&WorkspaceId::default())
+        .get_wc_commit_id(WorkspaceName::DEFAULT)
         .unwrap();
     let wc_commit = repo.store().get_commit(wc_commit_id).unwrap();
     assert_eq!(*wc_commit.tree_id(), repo.store().empty_merged_tree_id());
