@@ -16,14 +16,13 @@ use std::collections::HashMap;
 use std::env;
 use std::fs;
 use std::fs::OpenOptions;
-use std::io::Read;
-use std::io::Write;
+use std::io::Read as _;
+use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
-use std::sync::Once;
 
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::backend;
 use jj_lib::backend::Backend;
 use jj_lib::backend::BackendInitError;
@@ -41,9 +40,8 @@ use jj_lib::config::ConfigLayer;
 use jj_lib::config::ConfigSource;
 use jj_lib::config::StackedConfig;
 use jj_lib::git_backend::GitBackend;
-use jj_lib::local_backend::LocalBackend;
 use jj_lib::merged_tree::MergedTree;
-use jj_lib::object_id::ObjectId;
+use jj_lib::object_id::ObjectId as _;
 use jj_lib::repo::MutableRepo;
 use jj_lib::repo::ReadonlyRepo;
 use jj_lib::repo::Repo;
@@ -56,6 +54,7 @@ use jj_lib::rewrite::RebasedCommit;
 use jj_lib::secret_backend::SecretBackend;
 use jj_lib::settings::UserSettings;
 use jj_lib::signing::Signer;
+use jj_lib::simple_backend::SimpleBackend;
 use jj_lib::store::Store;
 use jj_lib::transaction::Transaction;
 use jj_lib::tree::Tree;
@@ -64,7 +63,7 @@ use jj_lib::working_copy::SnapshotError;
 use jj_lib::working_copy::SnapshotOptions;
 use jj_lib::working_copy::SnapshotStats;
 use jj_lib::workspace::Workspace;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use tempfile::TempDir;
 
 use crate::test_backend::TestBackendFactory;
@@ -72,21 +71,29 @@ use crate::test_backend::TestBackendFactory;
 pub mod git;
 pub mod test_backend;
 
-pub fn hermetic_libgit2() {
-    // libgit2 respects init.defaultBranch (and possibly other config
-    // variables) in the user's config files. Disable access to them to make
-    // our tests hermetic.
-    //
-    // set_search_path is unsafe because it cannot guarantee thread safety (as
-    // its documentation states). For the same reason, we wrap these invocations
-    // in `call_once`.
-    static CONFIGURE_GIT2: Once = Once::new();
-    CONFIGURE_GIT2.call_once(|| unsafe {
-        git2::opts::set_search_path(git2::ConfigLevel::System, "").unwrap();
-        git2::opts::set_search_path(git2::ConfigLevel::Global, "").unwrap();
-        git2::opts::set_search_path(git2::ConfigLevel::XDG, "").unwrap();
-        git2::opts::set_search_path(git2::ConfigLevel::ProgramData, "").unwrap();
-    });
+// TODO: Consider figuring out a way to make `GitBackend` and `git(1)` calls in
+// tests ignore external configuration and removing this function. This is
+// somewhat tricky because `gix` looks at system and user configuration, and
+// `GitBackend` also calls into `git(1)` for things like garbage collection.
+pub fn hermetic_git() {
+    #[cfg(feature = "git2")]
+    {
+        // libgit2 respects init.defaultBranch (and possibly other config
+        // variables) in the user's config files. Disable access to them to make
+        // our tests hermetic.
+        //
+        // set_search_path is unsafe because it cannot guarantee thread safety (as
+        // its documentation states). For the same reason, we wrap these invocations
+        // in `call_once`.
+        use std::sync::Once;
+        static CONFIGURE_GIT2: Once = Once::new();
+        CONFIGURE_GIT2.call_once(|| unsafe {
+            git2::opts::set_search_path(git2::ConfigLevel::System, "").unwrap();
+            git2::opts::set_search_path(git2::ConfigLevel::Global, "").unwrap();
+            git2::opts::set_search_path(git2::ConfigLevel::XDG, "").unwrap();
+            git2::opts::set_search_path(git2::ConfigLevel::ProgramData, "").unwrap();
+        });
+    }
 
     // Prevent GitBackend from loading user and system configurations. For
     // gitoxide API use in tests, Config::isolated() is probably better.
@@ -100,7 +107,7 @@ pub fn hermetic_libgit2() {
 }
 
 pub fn new_temp_dir() -> TempDir {
-    hermetic_libgit2();
+    hermetic_git();
     tempfile::Builder::new()
         .prefix("jj-test-")
         .tempdir()
@@ -128,6 +135,16 @@ pub fn user_settings() -> UserSettings {
     UserSettings::from_config(base_user_config()).unwrap()
 }
 
+/// Panic if `CI` environment variable is set to a non-empty value
+///
+/// Most CI environments set this variable automatically. See e.g.
+/// <https://docs.github.com/en/actions/writing-workflows/choosing-what-your-workflow-does/store-information-in-variables#default-environment-variables>
+#[track_caller]
+pub fn ensure_running_outside_ci(reason: &str) {
+    let running_in_ci = std::env::var("CI").is_ok_and(|value| !value.is_empty());
+    assert!(!running_in_ci, "Running in CI, {reason}.");
+}
+
 #[derive(Debug)]
 pub struct TestEnvironment {
     temp_dir: TempDir,
@@ -182,7 +199,7 @@ pub struct TestRepo {
 #[derive(PartialEq, Eq, Copy, Clone)]
 pub enum TestRepoBackend {
     Git,
-    Local,
+    Simple,
     Test,
 }
 
@@ -195,7 +212,7 @@ impl TestRepoBackend {
     ) -> Result<Box<dyn Backend>, BackendInitError> {
         match self {
             TestRepoBackend::Git => Ok(Box::new(GitBackend::init_internal(settings, store_path)?)),
-            TestRepoBackend::Local => Ok(Box::new(LocalBackend::init(store_path))),
+            TestRepoBackend::Simple => Ok(Box::new(SimpleBackend::init(store_path))),
             TestRepoBackend::Test => Ok(Box::new(env.test_backend_factory.init(store_path))),
         }
     }
@@ -447,7 +464,7 @@ pub fn commit_with_tree(store: &Arc<Store>, tree_id: MergedTreeId) -> Commit {
 }
 
 pub fn dump_tree(store: &Arc<Store>, tree_id: &MergedTreeId) -> String {
-    use std::fmt::Write;
+    use std::fmt::Write as _;
     let mut buf = String::new();
     writeln!(
         &mut buf,
